/**
 * This is what final commands return.
 * It has a protected access to the node that was executed, allowing users to access the node's properties.
 *
 * This should only be used by users for advanced usage.
 */
export class FinalCommandOutput {
    node;
    constructor(node) {
        this.node = node;
    }
}
export class CommandArguments {
    sandstonePack;
    previousNode;
    autoCommit;
    NodeType;
    sandstoneCore;
    sandstoneCommands;
    constructor(sandstonePack, previousNode, autoCommit = true) {
        this.sandstonePack = sandstonePack;
        this.previousNode = previousNode;
        this.autoCommit = autoCommit;
        this.sandstoneCore = sandstonePack.core;
        this.sandstoneCommands = sandstonePack.commands;
    }
    getNode = () => {
        if (this.previousNode) {
            // This is not a root-level command, so we can use the previous node.
            return this.previousNode;
        }
        // Automatically create the node for root-level commands.
        if (this.NodeType) {
            /* Typescript does not manage to remove undefined for some reasons */
            /* @ts-ignore */
            return new this.NodeType(this.sandstonePack);
        }
        throw new Error('No node type specified & no previous node for a non-root-level command');
    };
    finalCommand = (args, currentNode) => {
        // No followup. We can add arguments & commit.
        const node = currentNode ?? this.getNode();
        if (args) {
            node.args.push(...args);
        }
        if (this.autoCommit) {
            node.commit();
        }
        return new FinalCommandOutput(node);
    };
    subCommand = (args, NextArgumentType, executable = false, additionalNextArgs = [], currentNode) => {
        const node = currentNode ?? this.getNode();
        /*
         * The command has followup arguments. We need to append the arguments to the node, and return a new instance of the followup command.
         * If the command is executable at this point, we can commit it.
         */
        if (args) {
            node.args.push(...args);
        }
        if (executable && this.autoCommit) {
            node.commit();
        }
        return new NextArgumentType(this.sandstonePack, node, this.autoCommit, ...additionalNextArgs);
    };
}
