"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
var clone_exports = {};
__export(clone_exports, {
  CloneCommand: () => CloneCommand,
  CloneCommandNode: () => CloneCommandNode,
  CloneOptionsCommand: () => CloneOptionsCommand
});
module.exports = __toCommonJS(clone_exports);
var import_nodes = require("sandstone/core/nodes");
var import_parsers = require("sandstone/variables/parsers");
var import_helpers = require("../../helpers.js");
class CloneCommandNode extends import_nodes.CommandNode {
  constructor() {
    super(...arguments);
    __publicField(this, "command", "clone");
  }
}
class CloneCommand extends import_helpers.CommandArguments {
  constructor() {
    super(...arguments);
    __publicField(this, "NodeType", CloneCommandNode);
    /**
     * Clones blocks from one region to another.
     *
     * @param begin Specifies the coordinates of the first corner blocks of the source region.
     *
     * @param end Specifies the coordinates of the opposing corner blocks of the source region.
     *
     * @param destination Specifies the lower northwest corner of the destination region.
     * It corresponds to the block with the lowest X-axis, Y-axis and Z-axis value.
     *
     * @example
     *
     * // Clone blocks from `0 0 0` to `8 8 8` at the current location.
     * clone(abs(0, 0, 0), abs(8, 8, 8), rel(0, 0, 0))
     *
     * // Same as above, but only clone dirt blocks.
     * clone(abs(0, 0, 0), abs(8, 8, 8), rel(0, 0, 0)).filtered('minecraft:dirt')
     *
     * // Move the current block 1 block above
     * clone(rel(0, 0, 0), rel(0, 0, 0), rel(0, 1, 0)).replace('move')
     */
    __publicField(this, "clone", (begin, end, destination) => this.subCommand([begin, end, destination].map(import_parsers.coordinatesParser), CloneOptionsCommand, true));
  }
}
class CloneOptionsCommand extends import_helpers.CommandArguments {
  constructor() {
    super(...arguments);
    /**
     * Copy all blocks, overwriting all blocks of the destination region with the blocks from the source region.
     *
     * @param mode One of these three:
     * - `force`: Force the clone even if the source and destination regions overlap.
     * - `move`: Clone the source region to the destination region, then replace the source region with air. When used in filtered mask mode, only the cloned blocks are replaced with air.
     * - `normal`: Don't move or force.
     */
    __publicField(this, "replace", (mode) => this.finalCommand(["replace", mode]));
    /**
     * Copy only non-air blocks. Blocks in the destination region that would otherwise be overwritten by air are left unmodified.
     *
     * @param mode One of these three:
     * - `force`: Force the clone even if the source and destination regions overlap.
     * - `move`: Clone the source region to the destination region, then replace the source region with air. When used in filtered mask mode, only the cloned blocks are replaced with air.
     * - `normal`: Don't move or force.
     */
    __publicField(this, "masked", (mode) => this.finalCommand(["masked", mode]));
    /**
     * Clones only blocks with the block id specified by `filter`.
     *
     * @param filter The block ID to clone.
     *
     * @param mode One of these three:
     * - `force`: Force the clone even if the source and destination regions overlap.
     * - `move`: Clone the source region to the destination region, then replace the source region with air. When used in filtered mask mode, only the cloned blocks are replaced with air.
     * - `normal`: Don't move or force.
     */
    __publicField(this, "filtered", (filter, mode) => this.finalCommand(["filtered", filter, mode]));
  }
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  CloneCommand,
  CloneCommandNode,
  CloneOptionsCommand
});
//# sourceMappingURL=clone.cjs.map