"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
var fill_exports = {};
__export(fill_exports, {
  FillArgumentsCommand: () => FillArgumentsCommand,
  FillCommand: () => FillCommand,
  FillCommandNode: () => FillCommandNode
});
module.exports = __toCommonJS(fill_exports);
var import_nodes = require("sandstone/core/nodes");
var import_parsers = require("sandstone/variables/parsers");
var import_helpers = require("../../helpers.js");
class FillCommandNode extends import_nodes.CommandNode {
  constructor() {
    super(...arguments);
    __publicField(this, "command", "fill");
  }
}
class FillArgumentsCommand extends import_helpers.CommandArguments {
  constructor() {
    super(...arguments);
    /**
     * Replaces all blocks (including air) in the fill region with the specified block,
     * dropping the existing blocks (including those that are unchanged) and block contents as entities,
     * as if they had been mined with an unenchanted diamond shovel or pickaxe.
     *
     * (Blocks that can be mined only with shears, such as vines, do not drop; neither do liquids.)
     */
    __publicField(this, "destroy", () => this.finalCommand(["destroy"]));
    /**
     * Replaces only the blocks on the outer edge of the fill region with the specified block.
     *
     * Inner blocks are changed to air, dropping their contents as entities but not themselves.
     *
     * If the fill region has no inner blocks (because it is smaller than three blocks in at least one dimension),
     * acts like `replace`.
     */
    __publicField(this, "hollow", () => this.finalCommand(["hollow"]));
    /**
     * Replaces only the air blocks in the fill region with the specified block.
     */
    __publicField(this, "keep", () => this.finalCommand(["keep"]));
    /**
     * Replaces only the blocks on the outer edge of the fill region with the specified block.
     *
     * Inner blocks are not affected.
     *
     * If the fill region has no inner blocks (because it is smaller than three blocks in at least one dimension),
     * acts like `replace`.
     */
    __publicField(this, "outline", () => this.finalCommand(["outline"]));
    /**
     * Replaces all blocks (including air) in the fill region with the specified block,
     * without dropping blocks or block contents as entities.
     *
     * Optionally, instead of specifying a data tag for the replacing block,
     * block ID and data values may be specified to limit which blocks are replaced.
     *
     * @example
     * // Replace only furnaces facing north, with a BurnTime of 200 ticks:
     * fill(...).replace('minecraft:furnace[facing=north]{BurnTime:200}')
     */
    __publicField(this, "replace", (filter) => this.finalCommand(["replace", filter]));
  }
}
class FillCommand extends import_helpers.CommandArguments {
  constructor() {
    super(...arguments);
    __publicField(this, "NodeType", FillCommandNode);
    /**
     * Fills all or parts of a region with a specific block.
     *
     * @param from
     * Specifies the first corner blocks of the region to be filled (the "fill region").
     *
     * Block position is the coordinates of the point at the lower northwest corner of a block,
     * corresponding to the lowest X-axis, Y-axis and Z-axis point of the block.
     *
     * @param to
     * Specifies the second, opposite corner blocks of the region to be filled (the "fill region").
     *
     * @param block Specifies the block to fill the region with.
     *
     * @example
     * // Fill a block of 9x9 blocks, centered on the player
     * fill('~-4 ~-4 ~-4', '~4 ~4 ~4', 'minecraft:diamond_block')
     *
     * // Fill a hollow block of 9x9 blocks, centered on the player
     * fill('~-4 ~-4 ~-4', '~4 ~4 ~4', 'minecraft:diamond_block').hollow()
     *
     * // Replace all dirt with grass in a 9x9 blocks, centered on the player
     * fill('~-4 ~-4 ~-4', '~4 ~4 ~4', 'minecraft:grass_block').replace('minecraft:dirt')
     */
    __publicField(this, "fill", (from, to, block) => this.subCommand(
      [(0, import_parsers.coordinatesParser)(from), (0, import_parsers.coordinatesParser)(to), block],
      FillArgumentsCommand,
      true
    ));
  }
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  FillArgumentsCommand,
  FillCommand,
  FillCommandNode
});
//# sourceMappingURL=fill.cjs.map