"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
var place_exports = {};
__export(place_exports, {
  PlaceCommand: () => PlaceCommand,
  PlaceCommandNode: () => PlaceCommandNode
});
module.exports = __toCommonJS(place_exports);
var import_nodes = require("sandstone/core/nodes");
var import_parsers = require("sandstone/variables/parsers");
var import_helpers = require("../../helpers.js");
var import_validators = require("../../validators.js");
class PlaceCommandNode extends import_nodes.CommandNode {
  constructor() {
    super(...arguments);
    __publicField(this, "command", "place");
  }
}
class PlaceCommand extends import_helpers.CommandArguments {
  constructor() {
    super(...arguments);
    __publicField(this, "NodeType", PlaceCommandNode);
    // TODO: When working on worldgen add Feature Class here
    /**
     * Attempts to place a worldgen feature.
     *
     * @param feature Specifies the configured feature to place.
     * @param pos Optional. Where the placement should be tried.
     */
    __publicField(this, "feature", (feature, pos = "~ ~ ~") => this.finalCommand(["feature", feature, (0, import_parsers.coordinatesParser)(pos)]));
    /**
     * Places from a pool with a jigsaw.
     *
     * @param pool Configured structure template pool.
     * @param target Jigsaw with the target name will be anchored to the `pos` or current location. If multiple jigsaws have this name a random one will be the anchor.
     * @param maxDepth Max depth of the jigsaw. Must be an integer between 1 & 7 (inclusive).
     * @param pos Optional. Where to place.
     */
    __publicField(this, "jigsaw", (pool, target, maxDepth, pos = "~ ~ ~") => this.finalCommand(["jigsaw", pool, target, `${(0, import_validators.validateIntegerRange)(maxDepth, "Jigsaw max depth", 0, 7)}`, (0, import_parsers.coordinatesParser)(pos)]));
    /**
     * Places a configured structure feature. (not from `data/<namespace>/structures`, see [the wiki](https://minecraft.wiki/w/Custom_structure#Configured_Structure_Feature))
     *
     * @param configuredStructure The configured structure feature to place.
     * @param pos Optional. Where to place.
     */
    __publicField(this, "structure", (configuredStructure, pos = "~ ~ ~") => this.finalCommand(["structure", configuredStructure, (0, import_parsers.coordinatesParser)(pos)]));
    /**
     * Places a structure (from `data/<namespace>/structures`, just like a LOAD structure block).
     *
     * @param structure The structure to place.
     * @param pos Optional. Where to place.
     * @param rotation Optional. Rotation to apply to the structure.
     * @param mirror Optional. Mirroring to apply to the structure.
     * @param integrity Optional. How complete the structure will be. Must be a float between 1.0 & 0.0 (inclusive), if below 1 the structure will be randomly degraded.
     * @param seed Optional. Integer applied to the integrity random. Defaults to 0.
     */
    __publicField(this, "template", (structure, pos = "~ ~ ~", rotation, mirror, integrity = 1, seed = 0) => {
      this.finalCommand(["template", `${structure}`, (0, import_parsers.coordinatesParser)(pos), (0, import_parsers.structureRotationParser)(rotation), (0, import_parsers.structureMirrorParser)(mirror), `${integrity}`, `${seed}`]);
    });
  }
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  PlaceCommand,
  PlaceCommandNode
});
//# sourceMappingURL=place.cjs.map