import { CommandNode } from 'sandstone/core/nodes';
import { CommandArguments } from '../../helpers.js';
import type { SingleEntityArgument } from 'sandstone/arguments';
import type { Macroable } from 'sandstone/core';
export declare class AttributeCommandNode extends CommandNode {
    command: "attribute";
}
export declare class AttributeOperationCommand<MACRO extends boolean> extends CommandArguments {
    /**
     * Returns the total value of the specified attribute.
     * @category attribute
     */
    get: (scale?: Macroable<number, MACRO>) => import("../../helpers.js").FinalCommandOutput;
    /**
     * Returns the base value of the specified attribute.
     * @category attribute
     */
    baseGet: (scale?: Macroable<number, MACRO>) => import("../../helpers.js").FinalCommandOutput;
    /**
     * Overwrites the base value of the specified attribute with the given value.
     * @category attribute
     */
    baseSet: (value: Macroable<number, MACRO>) => import("../../helpers.js").FinalCommandOutput;
    /**
     * Adds an attribute modifier with the specified properties if no modifier with the same UUID already existed.
     * @category attribute
     */
    add: (uuid: Macroable<string, MACRO>, name: Macroable<string, MACRO>, value: Macroable<number, MACRO>, modifier: Macroable<'add' | 'multiply' | 'multiply_base', MACRO>) => import("../../helpers.js").FinalCommandOutput;
    /** Removes the attribute modifier with the specified UUID. */
    remove: (uuid: Macroable<string, MACRO>) => import("../../helpers.js").FinalCommandOutput;
    /** Returns the value of the modifier with the specified UUID. */
    getModifierValue: (uuid: Macroable<string, MACRO>, scale?: Macroable<number, MACRO>) => import("../../helpers.js").FinalCommandOutput;
}
/**
 * Used to change or read attributes.
 */
export declare class AttributeCommand<MACRO extends boolean> extends CommandArguments {
    protected NodeType: typeof AttributeCommandNode;
    attribute: (target: Macroable<SingleEntityArgument<MACRO>, MACRO>, attribute: Macroable<string, MACRO>) => import("../../helpers.js").FinalCommandOutput;
}
