import { CommandNode } from 'sandstone/core/nodes';
import { targetParser } from 'sandstone/variables/parsers';
import { CommandArguments } from '../../helpers.js';
// Attribute command
export class AttributeCommandNode extends CommandNode {
    command = 'attribute';
}
export class AttributeOperationCommand extends CommandArguments {
    /**
     * Returns the total value of the specified attribute.
     * @category attribute
     */
    get = (scale) => this.finalCommand(['get', scale]);
    /**
     * Returns the base value of the specified attribute.
     * @category attribute
     */
    baseGet = (scale) => this.finalCommand(['base', 'get', scale]);
    /**
     * Overwrites the base value of the specified attribute with the given value.
     * @category attribute
     */
    baseSet = (value) => this.finalCommand(['base', 'set', value]);
    /**
     * Adds an attribute modifier with the specified properties if no modifier with the same UUID already existed.
     * @category attribute
     */
    add = (uuid, name, value, modifier) => this.finalCommand(['modifier', 'add', uuid, name, value, modifier]);
    /** Removes the attribute modifier with the specified UUID. */
    remove = (uuid) => this.finalCommand(['modifier', 'remove', uuid]);
    /** Returns the value of the modifier with the specified UUID. */
    getModifierValue = (uuid, scale) => this.finalCommand(['modifier', 'value', 'get', uuid, scale]);
}
/**
 * Used to change or read attributes.
 */
export class AttributeCommand extends CommandArguments {
    NodeType = AttributeCommandNode;
    attribute = (target, attribute) => this.finalCommand([targetParser(target), attribute]);
}
