import { CommandNode } from 'sandstone/core/nodes';
import { CommandArguments } from '../../helpers.js';
import type { Coordinates, DAMAGE_TYPES, SingleEntityArgument } from 'sandstone/arguments';
import type { DamageTypeClass } from 'sandstone/core';
import type { LiteralUnion } from 'sandstone/utils';
import type { Macroable } from 'sandstone/core';
export declare class DamageCommandNode extends CommandNode {
    command: "damage";
}
declare class DamageCauseCommand<MACRO extends boolean> extends CommandArguments {
    /**
     * @param cause Cause of the damage, in the case of indirect damage.
     */
    from: (cause: Macroable<SingleEntityArgument<MACRO>, MACRO>) => import("../../helpers.js").FinalCommandOutput;
}
export declare class DamageSourceCommand<MACRO extends boolean> extends CommandArguments {
    /**
     * @param entity Entity inflicting the damage.
     */
    by: (entity: Macroable<SingleEntityArgument<MACRO>, MACRO>) => DamageCauseCommand<boolean>;
    /**
     * Where the damage originated at (when no entity caused the damage).
     */
    at: (position: Macroable<Coordinates<MACRO>, MACRO>) => import("../../helpers.js").FinalCommandOutput;
}
export declare class DamageCommand<MACRO extends boolean> extends CommandArguments {
    protected NodeType: typeof DamageCommandNode;
    /**
     * Deals damage.
     *
     * @param target Specifies the target.
     *
     * @param amount Specifies the amount of damage to be dealt.
     *
     * @param damageType This determines how the damage affects the entity as well as which death message is displayed
     */
    damage: (target: Macroable<SingleEntityArgument<MACRO>, MACRO>, amount: Macroable<number, MACRO>, damageType?: Macroable<LiteralUnion<DAMAGE_TYPES> | DamageTypeClass, MACRO>) => DamageSourceCommand<boolean>;
}
export {};
