import { CommandNode } from 'sandstone/core/nodes';
import { CommandArguments } from '../../helpers.js';
import type { MOB_EFFECTS, MultipleEntitiesArgument } from 'sandstone/arguments';
import type { LiteralUnion } from 'sandstone/utils';
import type { Macroable } from 'sandstone/core';
export declare class EffectCommandNode extends CommandNode {
    command: "effect";
}
export declare class EffectCommand<MACRO extends boolean> extends CommandArguments {
    protected NodeType: typeof EffectCommandNode;
    /**
     * Gives an effect.
     *
     * @param targets Specifies the target(s).
     *
     * @param effect Specifies the effect to be added.
     *
     * @param seconds
     * Specifies the effect's duration in seconds
     * (or in gameticks for `instant_damage`, `instant_health`, and `saturation`).
     * If not specified, defaults to 30 seconds (or 1 gametick for `instant_damage`, `instant_health`, and `saturation`).
     *
     * It must be between `0` and `1_000_000` (inclusive).
     *
     * @param amplifier
     * Specifies the number of additional levels to add to the effect.
     * If not specified, defaults to `0`.
     *
     * Note that the first tier of a status effect (e.g. Regeneration I) is `0`, so the second tier,
     * for example Regeneration II, would be specified by an amplifier level of `1`.
     *
     * @param hideParticles
     * Specifies whether the particles and the HUD indicator‌ of the status effect should be hidden.
     * If not specified, defaults to `false`.
     */
    give: (targets: Macroable<MultipleEntitiesArgument<MACRO>, MACRO>, effect: Macroable<LiteralUnion<MOB_EFFECTS>, MACRO>, seconds?: number | 'infinite', amplifier?: number, hideParticles?: boolean) => import("../../helpers.js").FinalCommandOutput;
    /**
     * Removes an effect.
     *
     * @param targets Specifies the target(s). If unspecified, defaults to `@s`.
     *
     * @param effect Specifies the effect to be removed. If unspecified, clears all effects.
     */
    clear: (targets?: Macroable<MultipleEntitiesArgument<MACRO>, MACRO>, effect?: Macroable<LiteralUnion<MOB_EFFECTS>, MACRO>) => import("../../helpers.js").FinalCommandOutput;
}
