"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
var execute_exports = {};
__export(execute_exports, {
  ExecuteCommand: () => ExecuteCommand,
  ExecuteCommandNode: () => ExecuteCommandNode,
  ExecuteDataArgsCommand: () => ExecuteDataArgsCommand,
  ExecuteFacingEntityCommand: () => ExecuteFacingEntityCommand,
  ExecuteIfUnlessCommand: () => ExecuteIfUnlessCommand,
  ExecutePositionedCommand: () => ExecutePositionedCommand,
  ExecuteRotatedAsCommand: () => ExecuteRotatedAsCommand,
  ExecuteStoreArgsCommand: () => ExecuteStoreArgsCommand,
  ExecuteStoreCommand: () => ExecuteStoreCommand
});
module.exports = __toCommonJS(execute_exports);
var import_core = require("sandstone/core");
var import_nodes = require("sandstone/core/nodes");
var import_utils = require("sandstone/utils");
var import_parsers = require("sandstone/variables/parsers");
var import_helpers = require("../../helpers.js");
var import_function = require("../server/function.js");
const isObjective = (arg) => typeof arg === "object" && Object.hasOwn(arg, "reset");
const isScore = (arg) => typeof arg === "object" && Object.hasOwn(arg, "unaryOperation");
class ExecuteCommandPart extends import_helpers.CommandArguments {
  constructor() {
    super(...arguments);
    __publicField(this, "nestedExecute", (args, executable = true) => this.subCommand([args], ExecuteCommand, executable));
  }
}
class ExecuteCommandNode extends import_nodes.ContainerCommandNode {
  constructor(sandstonePack, args = [], {
    isFake = false,
    isSingleExecute = true,
    givenCallbackName = void 0,
    body = []
  } = {}) {
    super(sandstonePack, ...args);
    __publicField(this, "command", "execute");
    /**
     * Is being used by internals to generate a function call. (eg. Flow visitor)
     */
    __publicField(this, "isFake");
    /**
     * By default, the execute is treated as single (execute.run.stuff).
     * This is set to `false` if the execute is used as a function (execute.run(() => { stuff })).
     */
    __publicField(this, "isSingleExecute");
    /**
     * In case of a multiple execute, this specifies the base name of the MCFunction.
     */
    __publicField(this, "givenCallbackName");
    __publicField(this, "append", (...nodes) => {
      for (const node of nodes) {
        this.body.push(node);
        if (this.isSingleExecute) {
          this.sandstoneCore.getCurrentMCFunctionOrThrow().exitContext();
        }
      }
      return nodes.length === 1 ? nodes[0] : nodes;
    });
    __publicField(this, "getValue", () => {
      if (this.body.length > 1) {
        throw new Error("Execute nodes can only have one child node when toString is called.");
      }
      const flattenedArgs = this.args.flat(1);
      const args = [];
      for (const arg of flattenedArgs) {
        if (arg !== void 0 && arg !== null) {
          if (typeof arg === "object") {
            if ((0, import_core.isMacroArgument)(this.sandstoneCore, arg)) {
              this.isMacro = true;
              args.push(arg.toMacro());
            } else {
              args.push(arg);
            }
          } else {
            args.push(arg);
          }
        }
      }
      const executeString = `${this.command} ${args.join(" ")}`;
      if (this.body.length === 0) {
        return executeString;
      }
      if (this.isFake) {
        return this.body[0].getValue();
      }
      let command = this.body[0].getValue();
      if (command.startsWith("$")) {
        this.isMacro = true;
        command = command.slice(1);
      }
      return `${this.isMacro ? "$" : ""}${executeString} run ${command}`;
    });
    __publicField(this, "createMCFunction", (currentMCFunction) => {
      if (this.isSingleExecute || !currentMCFunction) {
        return { node: this };
      }
      const namespace = currentMCFunction.resource.name.includes(":") ? `${currentMCFunction.resource.name.split(":")[0]}:` : "";
      const mcFunction = this.sandstonePack.MCFunction(`${namespace}${currentMCFunction.resource.path.slice(2).join("/")}/${this.callbackName}`, {
        creator: "sandstone",
        onConflict: "rename"
      });
      const mcFunctionNode = mcFunction["node"];
      mcFunctionNode.body = this.body;
      const mcFunctionCall = new import_function.FunctionCommandNode(this.sandstonePack, mcFunction);
      this.body = [mcFunctionCall];
      return { node: this, mcFunction: mcFunctionNode };
    });
    this.givenCallbackName = givenCallbackName;
    this.isSingleExecute = isSingleExecute;
    this.isFake = isFake;
    this.append(...body);
  }
  get callbackName() {
    var _a;
    return (_a = this.givenCallbackName) != null ? _a : `execute_${this.args[0][0].split(" ")[0]}`;
  }
}
class ExecuteStoreArgsCommand extends ExecuteCommandPart {
  constructor() {
    super(...arguments);
    /**
     * Saves the final command's return value as tag data within a block entity.
     *
     * Store as a byte, short, int, long, float, or double. If the return value is a decimal, it is rounded first and then multiplied by `scale`.
     *
     * @param targetPos Position of target block
     *
     * @param path Location of the desired tag to hold the value in.
     *
     * @param type Desired data size/type.
     *
     * @param scale Multiplier to apply before storing value. Defaults to 1.
     */
    __publicField(this, "block", (targetPos, path, type, scale) => this.nestedExecute(["block", (0, import_parsers.coordinatesParser)(targetPos), path, type, scale]));
    /**
     * Saves the final command's return value in either a bossbar's current value or its maximum value.
     *
     * @param id ID of the bossbar to target for saving.
     *
     * @param type Whether to overwrite the bossbar's current value or its max value.
     */
    __publicField(this, "bossbar", (id, type) => this.nestedExecute(["bossbar", id, type]));
    /**
     * Save the final command's return value in one of an entity's data tags.
     *
     * Store as a byte, short, int, long, float, or double. If the return value is a decimal, it is rounded first and then multiplied by `scale`.
     *
     * Like the `/data` command, `/execute store <arguments>` cannot modify player NBT.
     *
     * @param target A single entity to store under.
     *
     * @param path Location of the desired tag to hold the value in.
     *
     * @param type Desired data size/type.
     *
     * @param scale Multiplier to apply before storing value. Defaults to 1.
     */
    __publicField(this, "entity", (target, path, type, scale) => this.nestedExecute(["entity", (0, import_parsers.targetParser)(target), path, type, scale]));
    /**
     * Uses the `path` within storage `target` to store the return value in.
     *
     * Store as a byte, short, int, long, float, or double. If the return value is a decimal, it is rounded first and then multiplied by `scale`.
     *
     * If the storage does not yet exist, it gets created.
     *
     * @param target Target storage container, as a namespaced ID.
     *
     * @param path Location of the desired tag to hold the value in.
     *
     * @param type Desired data size/type.
     *
     * @param scale Multiplier to apply before storing value. Defaults to 1.
     */
    __publicField(this, "storage", (target, path, type, scale = 1) => this.nestedExecute(["storage", target, path, type, scale]));
  }
  /**
   * Overrides the given score with the final command's return value.
   *
   * @param targets Specifies score holder(s) whose score is to be overridden.
   *
   * @param objective A scoreboard objective.
   *
   * @param playerScore The player's score to override.
   */
  score(...args) {
    if (isScore(args[0])) {
      return this.nestedExecute(["score", args[0]]);
    }
    return this.nestedExecute(["score", (0, import_parsers.targetParser)(args[0]), args[1]]);
  }
}
class ExecuteStoreCommand extends ExecuteCommandPart {
  /** Store the final command's result value. */
  get result() {
    return this.subCommand([["result"]], ExecuteStoreArgsCommand, false);
  }
  /** Store the final command's success value. */
  get success() {
    return this.subCommand([["success"]], ExecuteStoreArgsCommand, false);
  }
}
class ExecuteDataArgsCommand extends ExecuteCommandPart {
  constructor() {
    super(...arguments);
    /**
     * Checks whether the targeted block has any data for a given tag
     * @param pos Position of the block to be tested.
     * @param path Data tag to check for.
     */
    __publicField(this, "block", (pos, path) => this.nestedExecute(["block", (0, import_parsers.coordinatesParser)(pos), path], true));
    /**
     * Checks whether the targeted entity has any data for a given tag
     * @param target One single entity to be tested.
     * @param path Data tag to check for.
     */
    __publicField(this, "entity", (target, path) => this.nestedExecute(["entity", (0, import_parsers.targetParser)(target), path], true));
    /**
     * Checks whether the targeted storage has any data for a given tag
     * @param source The storage to check in.
     * @param path Data tag to check for.
     */
    __publicField(this, "storage", (source, path) => this.nestedExecute(["storage", source, path], true));
  }
}
class ExecuteIfUnlessCommand extends ExecuteCommandPart {
  constructor() {
    super(...arguments);
    /**
     * Compares the block at a given position to a given block.
     *
     * @param pos Position of a target block to test.
     *
     * @param block A block to test against.
     */
    __publicField(this, "block", (pos, block) => this.nestedExecute(["block", (0, import_parsers.coordinatesParser)(pos), block], true));
    /**
     * Checks whether one or more entities exist.
     *
     * @param targets The target entities to check.
     */
    __publicField(this, "entity", (targets) => this.nestedExecute(["entity", (0, import_parsers.targetParser)(targets)], true));
    /**
     * Checks whether the current dimension matches the given one.
     * @param dimension Dimension to check against.
     */
    __publicField(this, "dimension", (dimension) => this.nestedExecute(["dimension", dimension]));
    /**
     * Checks whether the targeted block is in a chunk that is fully loaded (entity-processing chunk).
     * @param pos Position of target block to test.
     */
    __publicField(this, "loaded", (pos) => this.nestedExecute(["loaded", (0, import_parsers.coordinatesParser)(pos)]));
  }
  /**
   * Checks whether the `predicate` evaluates to a positive result.
   *
   * @param predicate The predicate to test.
   */
  predicate(predicate) {
    if (typeof predicate === "string") {
      return this.nestedExecute(["predicate", predicate], true);
    }
    if (Object.hasOwn(predicate, "toMacro")) {
      return this.nestedExecute(["predicate", predicate], true);
    }
    return this.nestedExecute(["predicate", (0, import_utils.toMinecraftResourceName)(predicate.path)], true);
  }
  score(...args) {
    const finalArgs = [];
    if (isScore(args[0])) {
      finalArgs.push(args[0].target.toString(), args[0].objective.name, args[1]);
      if (isScore(args[2])) {
        finalArgs.push(args[2].target.toString(), args[2].objective.name);
      } else {
        finalArgs.push((0, import_parsers.rangeParser)(this.sandstoneCore, args[2]));
      }
    } else {
      finalArgs.push((0, import_parsers.targetParser)(args[0]), isObjective(args[1]) ? args[1].name : args[1], args[2]);
      if (args[4]) {
        finalArgs.push((0, import_parsers.targetParser)(args[3]), isObjective(args[4]) ? args[4].name : args[4]);
      } else {
        finalArgs.push((0, import_parsers.rangeParser)(this.sandstoneCore, args[3]));
      }
    }
    return this.nestedExecute(["score", ...finalArgs], true);
  }
  data(dataPoint) {
    if (dataPoint) {
      return this.nestedExecute(dataPoint._toMinecraftCondition().getCondition());
    }
    return this.subCommand([["data"]], ExecuteDataArgsCommand, false);
  }
  function(func) {
    if (typeof func === "object" && Object.hasOwn(func, "addToTag")) {
      return this.nestedExecute(["function", (0, import_utils.toMinecraftResourceName)(func.path)]);
    }
    if (typeof func === "string" || Object.hasOwn(func, "toMacro")) {
      return this.nestedExecute(["function", func]);
    }
    const name = `${this.sandstoneCore.getCurrentMCFunctionOrThrow().resource.path.slice(2).join("/")}/execute_if_function`;
    const _func = this.sandstonePack.MCFunction(name, {
      addToSandstoneCore: false,
      creator: "sandstone",
      onConflict: "rename",
      callback: func
    });
    return this.nestedExecute(["function", name]);
  }
}
class ExecuteFacingEntityCommand extends ExecuteCommandPart {
  constructor() {
    super(...arguments);
    /**
     * Sets the command rotation to face a given point, as viewed from its anchor (either the eyes or the feet).
     *
     * @param targets The target(s) to rotate towards. (if you target multiple entities subcommands will run for each entity)
     *
     * @param anchor Whether to point at the target's eyes or feet.
     */
    __publicField(this, "entity", (targets, anchor = "feet") => this.nestedExecute(["entity", (0, import_parsers.targetParser)(targets), anchor]));
  }
}
class ExecutePositionedCommand extends ExecuteCommandPart {
  constructor() {
    super(...arguments);
    /**
     * Sets the command position, without changing rotation or dimension, by matching an entity's position.
     *
     * @param targets Target entity/entities to match position with.
     */
    __publicField(this, "as", (targets) => this.nestedExecute(["as", (0, import_parsers.targetParser)(targets)]));
    /**
     * Sets the command position matching the height map (highest position in a column of blocks according to criteria) for the current position.
     *
     * - `world_surface` Any non-air block.
     * - `motion_blocking` Any motion blocking material (eg. ignores flowers and grass).
     * - `motion_blocking_no_leaves` Any non-leaf motion blocking material.
     * - `ocean_floor` Any non-fluid motion blocking material.
     */
    __publicField(this, "over", (heightMap) => this.nestedExecute(["over", heightMap]));
  }
}
class ExecuteRotatedAsCommand extends ExecuteCommandPart {
  constructor() {
    super(...arguments);
    /**
     * Sets the command rotation, by matching an entity's rotation.
     *
     * @param targets Target entity/entities to match rotation with.
     */
    __publicField(this, "as", (targets) => this.nestedExecute(["as", (0, import_parsers.targetParser)(targets)]));
  }
}
class ExecuteCommand extends ExecuteCommandPart {
  constructor() {
    super(...arguments);
    __publicField(this, "NodeType", ExecuteCommandNode);
    /**
     * Updates the command's position, aligning to its current block position (an integer). Only applies along specified axes.
     * This is akin to flooring the coordinates – i.e. rounding them downwards. It updates the meaning of `~ ~ ~` and `^ ^ ^`.
     *
     * @param axes Any non-repeating combination of the characters 'x', 'y', and 'z'.
     */
    __publicField(this, "align", (axes) => this.nestedExecute(["align", axes]));
    /**
     * Stores the distance from the feet to the eyes of the entity that is executing the command in the anchor, which is part of the command context.
     * Effectively re-centers `^ ^ ^` on either the eyes or feet, also changing the angle the `facing` sub-command works off of.
     *
     * @param anchor Whether to anchor the executed command to eye level or feet level
     */
    __publicField(this, "anchored", (anchor) => this.nestedExecute(["anchored", anchor]));
    /**
     * Sets the command sender to target entity, without changing position, rotation, dimension, or anchor
     *
     * @param targets Target entity/entities to become the new sender.
     */
    __publicField(this, "as", (targets) => this.nestedExecute(["as", (0, import_parsers.targetParser)(targets)]));
    /**
     * Sets the command position, rotation, and dimension to match those of an entity/entities; does not change sender
     * @param targets Target entity/entities to match position, rotation, and dimension with
     */
    __publicField(this, "at", (targets) => this.nestedExecute(["at", (0, import_parsers.targetParser)(targets)]));
    /**
     * Sets the command dimension.
     *
     * @param dimension Name of the new execution dimension.
     */
    __publicField(this, "in", (dimension) => this.nestedExecute(["in", dimension]));
    /**
     * Sets the command executor to a related entity.
     * @param relation Relation with the current executor.
     */
    __publicField(this, "on", (relation) => this.nestedExecute(["on", relation]));
  }
  facing(pos) {
    if (pos) {
      return this.nestedExecute(["facing", (0, import_parsers.coordinatesParser)(pos)]);
    }
    return this.subCommand([["facing"]], ExecuteFacingEntityCommand, false);
  }
  positioned(pos) {
    if (pos) {
      return this.nestedExecute(["positioned", (0, import_parsers.coordinatesParser)(pos)]);
    }
    return this.subCommand([["positioned"]], ExecutePositionedCommand, false);
  }
  rotated(rotation) {
    if (rotation) {
      return this.nestedExecute(["rotated", (0, import_parsers.rotationParser)(rotation)]);
    }
    return this.subCommand([["rotated"]], ExecuteRotatedAsCommand, false);
  }
  /**
   * Summons an entity at the command position and sets the command executor to said newly summoned entity.
   * @param entityType Entity to summon.
   */
  summon(entityType) {
    return this.nestedExecute(["summon", entityType]);
  }
  get if() {
    return this.subCommand([["if"]], ExecuteIfUnlessCommand, false);
  }
  get unless() {
    return this.subCommand([["unless"]], ExecuteIfUnlessCommand, false);
  }
  get store() {
    return this.subCommand([["store"]], ExecuteStoreCommand, false);
  }
  get run() {
    const node = this.getNode();
    const commands = new Proxy(this.sandstonePack.commands, {
      get: (target, p, receiver) => {
        this.sandstoneCore.getCurrentMCFunctionOrThrow().enterContext(node, false);
        return this.sandstonePack.commands[p];
      }
    });
    return (0, import_utils.makeCallable)(commands, (callback) => {
      node.isSingleExecute = false;
      this.sandstoneCore.insideContext(node, callback, false);
      return new import_helpers.FinalCommandOutput(node);
    }, true);
  }
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  ExecuteCommand,
  ExecuteCommandNode,
  ExecuteDataArgsCommand,
  ExecuteFacingEntityCommand,
  ExecuteIfUnlessCommand,
  ExecutePositionedCommand,
  ExecuteRotatedAsCommand,
  ExecuteStoreArgsCommand,
  ExecuteStoreCommand
});
//# sourceMappingURL=execute.cjs.map