"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
var scoreboard_exports = {};
__export(scoreboard_exports, {
  ScoreboardCommand: () => ScoreboardCommand,
  ScoreboardCommandNode: () => ScoreboardCommandNode,
  ScoreboardObjectivesModifyCommand: () => ScoreboardObjectivesModifyCommand
});
module.exports = __toCommonJS(scoreboard_exports);
var import_nodes = require("sandstone/core/nodes");
var import_JSONTextComponentClass = require("sandstone/variables/JSONTextComponentClass");
var import_parsers = require("sandstone/variables/parsers");
var import_helpers = require("../../helpers.js");
const isScore = (arg) => typeof arg === "object" && Object.hasOwn(arg, "unaryOperation");
function scoresParser(...args) {
  return args.map((_arg, i) => {
    const arg = _arg;
    if (isScore(arg)) {
      return [arg.target, arg.objective];
    }
    if (arg == null ? void 0 : arg._toSelector) {
      return [arg._toSelector()];
    }
    return [arg];
  }).flat(Infinity);
}
class ScoreboardCommandNode extends import_nodes.CommandNode {
  constructor() {
    super(...arguments);
    __publicField(this, "command", "scoreboard");
  }
}
class ScoreboardObjectivesModifyCommand extends import_helpers.CommandArguments {
  constructor() {
    super(...arguments);
    /**
     * Change the display name of the scoreboard in display slots.
     *
     * @param displayName The new display name. Must be a JSON text component.
     */
    __publicField(this, "displayname", (displayName) => this.finalCommand(["displayname", (0, import_JSONTextComponentClass.parseJSONText)(displayName)]));
    /**
     * Change the display format of health bars.
     *
     * @param display Whether to display the health bars as hearts or integers.
     */
    __publicField(this, "rendertype", (display) => this.finalCommand(["rendertype", display]));
  }
}
class ScoreboardCommand extends import_helpers.CommandArguments {
  constructor() {
    super(...arguments);
    __publicField(this, "NodeType", ScoreboardCommandNode);
    /** All commands related to scoreboard objectives. */
    __publicField(this, "objectives", {
      /** List all existing objectives with their display names and criteria. */
      list: () => this.finalCommand(["objectives", "list"]),
      /**
       * Create a new objective with the given internal objective name, specified criterion, and the optional display name.
       * All three arguments are case-sensitive.
       *
       * @param objective must be a plain text.
       *
       * @param criterion must be a valid criterion type.
       *
       * @param displayName must be a JSON text component, defaulting to `objective` when unspecified.
       */
      add: (objective, criteria, displayName) => this.finalCommand(
        /* @ts-ignore */
        ["objectives", "add", objective, criteria, (0, import_JSONTextComponentClass.parseJSONText)(displayName)]
      ),
      /**
       * Delete all references to the named objective in the scoreboard system.
       * Data is deleted from the objectives list and entity scores,
       * and if it was on a display list it is no longer displayed
       */
      remove: (objective) => this.finalCommand(["remove", objective]),
      /**
       * Display score info for the objective in the given slot.
       *
       * @param slot The slot to display the objective in.
       *
       * @param objective The objective to display. If not provided, this display slot is cleared.
       */
      setdisplay: (slot, objective) => this.finalCommand(["setdisplay", slot, objective]),
      /**
       * Modify the display of the objective.
       *
       * @param objective The objective to modify.
       */
      modify: (objective) => this.subCommand(["modify", objective], ScoreboardObjectivesModifyCommand, false)
    });
    /** All commands related to scoreboard players. */
    __publicField(this, "players", {
      /**
       * If `target` is omitted, lists all entities which are tracked in some way by the scoreboard system.
       *
       * Else, lists the scores of a particular entity.
       *
       * @param target The entity to list the scores from.
       */
      list: (target) => this.finalCommand(["players", "list", (0, import_parsers.targetParser)(target)]),
      /**
       * Return the scoreboard value of a given objective for a given target.
       *
       * @param target The entity to get the score from.
       *
       * @param objective The objective to get the score from.
       */
      get: (...args) => this.finalCommand(["players", "get", ...scoresParser(...args)]),
      set: (...args) => this.finalCommand(["players", "set", ...scoresParser(args)]),
      add: (...args) => this.finalCommand(["players", "add", ...scoresParser(args)]),
      remove: (...args) => this.finalCommand(["players", "remove", ...scoresParser(args)]),
      reset: (...args) => this.finalCommand(["players", "reset", ...scoresParser(...args)]),
      enable: (...args) => this.finalCommand(["players", "enable", ...scoresParser(...args)]),
      operation: (...args) => this.finalCommand(["players", "operation", ...scoresParser(...args)]),
      display: {
        name: (...args) => {
          if (isScore(args[1])) {
            args[1] = (0, import_JSONTextComponentClass.parseJSONText)(args[1]);
          } else {
            args[2] = (0, import_JSONTextComponentClass.parseJSONText)(args[2]);
          }
          return this.finalCommand(["players", "display", "name", ...scoresParser(...args)]);
        },
        numberformat: (...args) => {
          if (args[1] === "styled") {
            args[2] = (0, import_JSONTextComponentClass.parseJSONText)(args[2]);
          }
          if (args[2] === "styled") {
            args[3] = (0, import_JSONTextComponentClass.parseJSONText)(args[3]);
          }
          if (args[1] === "fixed") {
            args[2] = (0, import_JSONTextComponentClass.parseJSONText)(args[2]);
          }
          if (args[2] === "fixed") {
            args[3] = (0, import_JSONTextComponentClass.parseJSONText)(args[3]);
          }
          return this.finalCommand(["players", "display", "numberformat", ...scoresParser(...args)]);
        }
      }
    });
  }
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  ScoreboardCommand,
  ScoreboardCommandNode,
  ScoreboardObjectivesModifyCommand
});
//# sourceMappingURL=scoreboard.cjs.map