/* eslint-disable max-len */
import { CommandNode } from 'sandstone/core/nodes';
import { parseJSONText } from 'sandstone/variables/JSONTextComponentClass';
import { targetParser } from 'sandstone/variables/parsers';
import { CommandArguments } from '../../helpers.js';
// Yes this sucks
const isScore = (arg) => typeof arg === 'object' && Object.hasOwn(arg, 'unaryOperation');
function scoresParser(...args) {
    return args.map((_arg, i) => {
        const arg = _arg;
        if (isScore(arg)) {
            return [arg.target, arg.objective];
        }
        if (arg?._toSelector) {
            return [arg._toSelector()];
        }
        return [arg];
    }).flat(+Infinity);
}
export class ScoreboardCommandNode extends CommandNode {
    command = 'scoreboard';
}
export class ScoreboardObjectivesModifyCommand extends CommandArguments {
    /**
     * Change the display name of the scoreboard in display slots.
     *
     * @param displayName The new display name. Must be a JSON text component.
     */
    displayname = (displayName) => this.finalCommand(['displayname', parseJSONText(displayName)]);
    /**
     * Change the display format of health bars.
     *
     * @param display Whether to display the health bars as hearts or integers.
     */
    rendertype = (display) => this.finalCommand(['rendertype', display]);
}
export class ScoreboardCommand extends CommandArguments {
    NodeType = ScoreboardCommandNode;
    /** All commands related to scoreboard objectives. */
    objectives = {
        /** List all existing objectives with their display names and criteria. */
        list: () => this.finalCommand(['objectives', 'list']),
        /**
         * Create a new objective with the given internal objective name, specified criterion, and the optional display name.
         * All three arguments are case-sensitive.
         *
         * @param objective must be a plain text.
         *
         * @param criterion must be a valid criterion type.
         *
         * @param displayName must be a JSON text component, defaulting to `objective` when unspecified.
         */
        add: (objective, criteria, displayName) => this.finalCommand(
        /* @ts-ignore */
        ['objectives', 'add', objective, criteria, parseJSONText(displayName)]),
        /**
         * Delete all references to the named objective in the scoreboard system.
         * Data is deleted from the objectives list and entity scores,
         * and if it was on a display list it is no longer displayed
         */
        remove: (objective) => this.finalCommand(['remove', objective]),
        /**
         * Display score info for the objective in the given slot.
         *
         * @param slot The slot to display the objective in.
         *
         * @param objective The objective to display. If not provided, this display slot is cleared.
         */
        setdisplay: (slot, objective) => this.finalCommand(['setdisplay', slot, objective]),
        /**
         * Modify the display of the objective.
         *
         * @param objective The objective to modify.
         */
        modify: (objective) => this.subCommand(['modify', objective], ScoreboardObjectivesModifyCommand, false),
    };
    /** All commands related to scoreboard players. */
    players = {
        /**
         * If `target` is omitted, lists all entities which are tracked in some way by the scoreboard system.
         *
         * Else, lists the scores of a particular entity.
         *
         * @param target The entity to list the scores from.
         */
        list: (target) => this.finalCommand(['players', 'list', targetParser(target)]),
        /**
         * Return the scoreboard value of a given objective for a given target.
         *
         * @param target The entity to get the score from.
         *
         * @param objective The objective to get the score from.
         */
        get: (...args) => this.finalCommand(['players', 'get', ...scoresParser(...args)]),
        set: (...args) => this.finalCommand(['players', 'set', ...scoresParser(args)]),
        add: (...args) => this.finalCommand(['players', 'add', ...scoresParser(args)]),
        remove: (...args) => this.finalCommand(['players', 'remove', ...scoresParser(args)]),
        reset: (...args) => this.finalCommand(['players', 'reset', ...scoresParser(...args)]),
        enable: (...args) => this.finalCommand(['players', 'enable', ...scoresParser(...args)]),
        operation: (...args) => this.finalCommand(['players', 'operation', ...scoresParser(...args)]),
        display: {
            name: (...args) => {
                if (isScore(args[1])) {
                    args[1] = parseJSONText(args[1]);
                }
                else {
                    args[2] = parseJSONText(args[2]);
                }
                return this.finalCommand(['players', 'display', 'name', ...scoresParser(...args)]);
            },
            numberformat: (...args) => {
                if (args[1] === 'styled') {
                    args[2] = parseJSONText(args[2]);
                }
                if (args[2] === 'styled') {
                    args[3] = parseJSONText(args[3]);
                }
                if (args[1] === 'fixed') {
                    args[2] = parseJSONText(args[2]);
                }
                if (args[2] === 'fixed') {
                    args[3] = parseJSONText(args[3]);
                }
                return this.finalCommand(['players', 'display', 'numberformat', ...scoresParser(...args)]);
            },
        },
    };
}
