import { CommandNode } from 'sandstone/core/nodes';
import { CommandArguments } from '../../helpers.js';
import type { MultipleEntitiesArgument } from 'sandstone/arguments';
import type { Macroable } from 'sandstone/core';
export declare class TagCommandNode extends CommandNode {
    command: "tag";
}
export declare class TagArgumentsCommand<MACRO extends boolean> extends CommandArguments {
    /**
     * Adds a tag to the targets.
     *
     * @param name Specifies the name of the tag to be added into the targets.
     * Valid characters are uppercase and lowercase letters, numbers, hyphen, underscore, plus sign and period.
     */
    add: (name: Macroable<string, MACRO>) => import("../../helpers.js").FinalCommandOutput;
    /**
     * Lists all tags on the targets.
     */
    list: () => import("../../helpers.js").FinalCommandOutput;
    /**
     * Removes a tag from the targets.
     *
     * @param name Specifies the name of the tag to be removed from the targets.
     * Valid characters are uppercase and lowercase letters, numbers, hyphen, underscore, plus sign and period.
     */
    remove: (name: Macroable<string, MACRO>) => import("../../helpers.js").FinalCommandOutput;
}
export declare class TagCommand<MACRO extends boolean> extends CommandArguments {
    protected NodeType: typeof TagCommandNode;
    /**
     * Controls scoreboard tags on individual entities.
     *
     * In Java Edition, the number of tags owned by an entity cannot exceed 1024.
     *
     * @param targets Specifies the command's target.
     */
    tag: (targets: Macroable<MultipleEntitiesArgument<MACRO>, MACRO>) => TagArgumentsCommand<MACRO>;
}
