import { CommandNode } from 'sandstone/core/nodes';
import { parseJSONText } from 'sandstone/variables/JSONTextComponentClass';
import { targetParser } from 'sandstone/variables/parsers';
import { CommandArguments } from '../../helpers.js';
export class TeamCommandNode extends CommandNode {
    command = 'team';
}
export class TeamCommand extends CommandArguments {
    NodeType = TeamCommandNode;
    /**
     * Creates a new team.
     *
     * @param team Specifies the name of the team.
     *
     * @param displayName Specifies the team name to be displayed.
     */
    add = (team, displayName) => this.finalCommand(['add', team, parseJSONText(displayName)]);
    /**
     * Removes all members from a team.
     *
     * @param team Specifies the name of the team.
     */
    empty = (team) => this.finalCommand(['empty', team]);
    /**
     * Makes specified entities join a team.
     *
     * @param team Specifies the name of the team.
     *
     * @param members Specifies the entities to join the team.
     * `'*'` may be used to represent all entities tracked by the scoreboard
     * If unspecified, defaults to the executor.
     */
    join = (team, members) => this.finalCommand(['join', team, targetParser(members)]);
    /**
     * Makes specified entities leave a team.
     *
     * @param members Specifies the entities to leave the team.
     * `'*'` may be used to represent all entities tracked by the scoreboard
     */
    leave = (members) => this.finalCommand(['leave', targetParser(members)]);
    /**
     * Lists all teams, or lists all members of a team if `team` is set.
     *
     * @param team Specifies the name of the team.
     */
    list = (team) => this.finalCommand(['list', targetParser(team)]);
    modify = (...args) => this.finalCommand(['modify', ...args]);
    /**
     * Removes a team.
     *
     * @param team Specifies the name of the team.
     */
    remove = (team) => this.finalCommand(['remove', team]);
}
