import { CommandNode } from 'sandstone/core/nodes';
import { VectorClass } from 'sandstone/variables/Coordinates';
import { coordinatesParser, targetParser } from 'sandstone/variables/parsers';
import { CommandArguments } from '../../helpers.js';
export class TeleportCommandNode extends CommandNode {
    command = 'tp';
}
export class TeleportFacingCommand extends CommandArguments {
    /**
     * Specifies the coordinates to make the target(s) facing to.
     * May use tilde and caret notation to specify a position relative to the position where the command is executed.
     */
    facing = (location) => this.finalCommand(['facing', coordinatesParser(location)]);
    /**
     * Specifies the entity to make the target(s) facing to.
     *
     * @param entity Must be a player name, a target selector, or a UUID‌.
     *             Permits entity other than players.
     *
     * @param anchor Specifies whether the entity'eyes or feet to make the target(s) facing to.
     *             Must be one of eyes and feet. If not specified, defaults to eyes.
     */
    facingEntity = (entity, anchor) => this.finalCommand(['facing', targetParser(entity), anchor]);
}
export class TeleportCommand extends CommandArguments {
    NodeType = TeleportCommandNode;
    tp = (...args) => {
        const parsedArgs = [...args.slice(0, 3).map(coordinatesParser), ...args.slice(3).map(targetParser)];
        if (parsedArgs.length === 2 && (parsedArgs[1] instanceof VectorClass || typeof parsedArgs[1] === 'string')) {
            return this.subCommand(parsedArgs, (TeleportFacingCommand), true);
        }
        return this.finalCommand(parsedArgs);
    };
}
