"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
var advancement_exports = {};
__export(advancement_exports, {
  AdvancementArgumentsCommand: () => AdvancementArgumentsCommand,
  AdvancementCommand: () => AdvancementCommand,
  AdvancementCommandNode: () => AdvancementCommandNode
});
module.exports = __toCommonJS(advancement_exports);
var import_nodes = require("sandstone/core/nodes");
var import_parsers = require("sandstone/variables/parsers");
var import_helpers = require("../../helpers.js");
class AdvancementCommandNode extends import_nodes.CommandNode {
  constructor() {
    super(...arguments);
    __publicField(this, "command", "advancement");
  }
}
class AdvancementArgumentsCommand extends import_helpers.CommandArguments {
  constructor() {
    super(...arguments);
    /** Adds or removes all loaded advancements. */
    __publicField(this, "everything", () => this.finalCommand(["everything"]));
    /**
     * Adds or removes a single advancement or criterion.
     *
     * @param advancement Specifies a valid namespaced id of the advancement to target.
     *
     * @param criterion Specifies a valid criterion of the advancement to manipulate.
     * The command defaults to the entire advancement.
     * If specified, the command refers to merely the criterion and not the entire advancement.
     */
    __publicField(this, "only", (advancement, criterion) => this.finalCommand(["only", advancement, criterion]));
    /**
     * Adds or removes an advancement and all its children advancements.
     * Think of specifying everything from that advancement to the end.
     *
     * The exact order the operation is carried out in is:
     *
     *     specified advancement > child > child's child > ...
     *
     * When it operates on a child that branches, it iterates through all its children before continuing.
     *
     * @param advancement Specifies a valid namespaced id of the advancement to target.
     */
    __publicField(this, "from", (advancement) => this.finalCommand(["from", advancement]));
    /**
     * Specifies an advancement, and adds or removes all its parent advancements, and all its children advancements.
     * Think of specifying everything through the specified advancement, going both backwards and forwards.
     *
     * The exact order the operation is as if the command were executed with "until" specified, then with "from" specified:
     *
     *     parent > parent's parent > ... > root > specified advancement > child > child's child > ...
     *
     * @param advancement Specifies a valid namespaced id of the advancement to target.
     */
    __publicField(this, "through", (advancement) => this.finalCommand(["through", advancement]));
    /**
     * Adds or removes an advancement and all its parent advancements until the root for addition/removal.
     * Think of specifying everything from the start until that advancement.
     *
     * The exact order the operation is carried out in is:
     *
     *     parent > parent's parent > ... > root > specified advancement.
     *
     * @param advancement Specifies a valid namespaced id of the advancement to target.
     */
    __publicField(this, "until", (advancement) => this.finalCommand(["until", advancement]));
  }
}
class AdvancementCommand extends import_helpers.CommandArguments {
  constructor() {
    super(...arguments);
    __publicField(this, "NodeType", AdvancementCommandNode);
    /**
     * Adds specified advancements.
     *
     * @param targets Specifies one player or more.
     */
    __publicField(this, "grant", (targets) => this.subCommand(["grant", (0, import_parsers.targetParser)(targets)], AdvancementArgumentsCommand, false));
    /**
     * Removes specified advancements.
     *
     * @param targets Specifies one player or more.
     */
    __publicField(this, "revoke", (targets) => this.subCommand(["revoke", (0, import_parsers.targetParser)(targets)], AdvancementArgumentsCommand, false));
  }
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  AdvancementArgumentsCommand,
  AdvancementCommand,
  AdvancementCommandNode
});
//# sourceMappingURL=advancement.cjs.map