"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
var experience_exports = {};
__export(experience_exports, {
  ExperienceCommand: () => ExperienceCommand,
  ExperienceCommandNode: () => ExperienceCommandNode
});
module.exports = __toCommonJS(experience_exports);
var import_validators = require("sandstone/commands/validators");
var import_nodes = require("sandstone/core/nodes");
var import_parsers = require("sandstone/variables/parsers");
var import_helpers = require("../../helpers.js");
class ExperienceCommandNode extends import_nodes.CommandNode {
  constructor() {
    super(...arguments);
    // We always use the shorthand version for compactness purposes
    __publicField(this, "command", "xp");
  }
}
class ExperienceCommand extends import_helpers.CommandArguments {
  constructor() {
    super(...arguments);
    __publicField(this, "NodeType", ExperienceCommandNode);
    /**
     * Adds experience to the player(s).
     *
     * @param targets Specifies the target(s) of the command.
     *
     * @param amount
     * Specifies the amount of experience points or levels to give to the player.
     * Negative values remove experience instead of adding them.
     *
     * Must be between `-2147483648` and `2147483647` (inclusive).
     *
     * @param type
     * If `level`, adds levels of experience to the player.
     * If `points`, add points of experience to the player.
     *
     * If unspecified, defaults to `points`.
     */
    __publicField(this, "add", (targets, amount, type) => {
      (0, import_validators.validateIntegerRange)(amount, "amount", -2147483648, 2147483647);
      return this.finalCommand([(0, import_parsers.targetParser)(targets), amount, type]);
    });
    /**
     * Sets the experience of the player(s).
     *
     * @param targets Specifies the target(s) of the command.
     *
     * @param amount
     * Specifies the amount of experience points or levels to be set to the player.
     *
     * Must be between `0` and `2147483647` (inclusive).
     *
     * @param type
     * If `level`, sets the levels of experience of the player.
     * If `points`, sets the points of experience of the player.
     *
     * If unspecified, defaults to `points`.
     */
    __publicField(this, "set", (targets, amount, type) => {
      (0, import_validators.validateIntegerRange)(amount, "amount", 0, 2147483647);
      return this.finalCommand([targets, amount, type]);
    });
    /**
     * Queries the experience of the player.
     *
     * @param target Specifies the target of the command.
     *
     * @param type
     * If `level`, queries the levels of experience of the player.
     * If `points`, queries the points of experience of the player.
     *
     * If unspecified, defaults to `points`.
     */
    __publicField(this, "query", (target, type) => this.finalCommand([target, type]));
  }
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  ExperienceCommand,
  ExperienceCommandNode
});
//# sourceMappingURL=experience.cjs.map