import { CommandNode } from 'sandstone/core';
import { CommandArguments } from '../../helpers.js';
import type { ITEMS, MultiplePlayersArgument, RootNBT } from 'sandstone/arguments';
import type { Macroable } from 'sandstone/core';
import type { LiteralUnion } from 'sandstone/utils';
import type { FinalCommandOutput } from '../../helpers.js';
export declare class GiveCommandNode extends CommandNode {
    command: "give";
}
export declare class GiveCommand<MACRO extends boolean> extends CommandArguments {
    protected NodeType: typeof GiveCommandNode;
    /**
     * Gives an item to one or more players.
     *
     * @param targets Specifies the target(s) to give item(s) to.
     *
     * @param item Specifies the item to give.
     *
     * @param count Specifies the number of items to give. If not specified, defaults to `1`.
     */
    give(targets: Macroable<MultiplePlayersArgument<MACRO>, MACRO>, item: Macroable<LiteralUnion<ITEMS>, MACRO>, count?: Macroable<number, MACRO>): FinalCommandOutput;
    /**
     * Gives an item to one or more players with nbt.
     *
     * @param targets Specifies the target(s) to give item(s) to.
     *
     * @param item Specifies the item to give.
     *
     * @param nbt Specifies the nbt of the item to give.
     *
     * @param count Specifies the number of items to give. If not specified, defaults to `1`.
     */
    give(targets: Macroable<MultiplePlayersArgument<MACRO>, MACRO>, item: Macroable<LiteralUnion<ITEMS>, MACRO>, nbt: Macroable<RootNBT, MACRO>, count?: Macroable<number, MACRO>): FinalCommandOutput;
}
