import { CommandNode } from 'sandstone/core/nodes';
import { CommandArguments } from '../../helpers.js';
import type { ITEMS, MultiplePlayersArgument } from 'sandstone/arguments';
import type { RecipeClass } from 'sandstone/core';
import type { LiteralUnion } from 'sandstone/utils';
import type { Macroable } from 'sandstone/core';
export declare class RecipeCommandNode extends CommandNode {
    command: "recipe";
}
/** Gives or takes (unlocks or locks) recipes for players. */
export declare class RecipeCommand<MACRO extends boolean> extends CommandArguments {
    protected NodeType: typeof RecipeCommandNode;
    /**
     * Gives recipes to the player.
     *
     * @param targets Specifies the player(s) to give the recipe to.
     * @param recipe Specifies a recipe to give. If `*` is specified, then all recipes will be given.
     */
    give: (targets: Macroable<MultiplePlayersArgument<MACRO>, MACRO>, recipe: Macroable<LiteralUnion<'*' | ITEMS> | RecipeClass, MACRO>) => import("../../helpers.js").FinalCommandOutput;
    /**
     * Takes recipes to the player.
     *
     * @param targets Specifies the player(s) to take the recipe from.
     * @param recipe Specifies a recipe to take. If `*` is specified, then all recipes will be taken.
     */
    take: (targets: Macroable<MultiplePlayersArgument<MACRO>, MACRO>, recipe: Macroable<LiteralUnion<'*' | ITEMS> | RecipeClass, MACRO>) => import("../../helpers.js").FinalCommandOutput;
}
