import { CommandNode } from 'sandstone/core/nodes';
import { CommandArguments } from '../../helpers.js';
import type { JSONTextComponent, MultiplePlayersArgument, TimeArgument } from 'sandstone/arguments';
import type { Macroable } from 'sandstone/core';
export declare class TitleCommandNode extends CommandNode {
    command: "title";
}
export declare class TitleArgumentsCommand<MACRO extends boolean> extends CommandArguments {
    clear: () => import("../../helpers.js").FinalCommandOutput;
    reset: () => import("../../helpers.js").FinalCommandOutput;
    title: (title: Macroable<JSONTextComponent, MACRO>) => import("../../helpers.js").FinalCommandOutput;
    subtitle: (subtitle: Macroable<JSONTextComponent, MACRO>) => import("../../helpers.js").FinalCommandOutput;
    actionbar: (actionbarText: Macroable<JSONTextComponent, MACRO>) => import("../../helpers.js").FinalCommandOutput;
    times: (fadeIn: Macroable<TimeArgument, MACRO>, stay: Macroable<TimeArgument, MACRO>, fadeOut: Macroable<TimeArgument, MACRO>) => import("../../helpers.js").FinalCommandOutput;
}
export declare class TitleCommand<MACRO extends boolean> extends CommandArguments {
    protected NodeType: typeof TitleCommandNode;
    /**
     * Controls text displayed on the screen.
     *
     * @param targets Specifies the player(s) to display a screen title to.
     *
     * @example
     * // Displays a red "Hello World" as title
     * title('@a').title([{text: 'Hello World', color: 'red'}])
     *
     * // Displays a target to kill in the actionbar
     * const target = Selector('@r')
     * title('@a').actionbar(['You target is: ', target])
     */
    title: (targets: Macroable<MultiplePlayersArgument<MACRO>, MACRO>) => TitleArgumentsCommand<MACRO>;
}
