import { CommandNode } from 'sandstone/core/nodes';
import { parseJSONText } from 'sandstone/variables/JSONTextComponentClass';
import { targetParser } from 'sandstone/variables/parsers';
import { CommandArguments } from '../../helpers.js';
export class TitleCommandNode extends CommandNode {
    command = 'title';
}
export class TitleArgumentsCommand extends CommandArguments {
    clear = () => this.finalCommand(['clear']);
    reset = () => this.finalCommand(['reset']);
    title = (title) => this.finalCommand(['title', parseJSONText(title)]);
    subtitle = (subtitle) => this.finalCommand(['subtitle', parseJSONText(subtitle)]);
    actionbar = (actionbarText) => this.finalCommand(['actionbar', parseJSONText(actionbarText)]);
    times = (fadeIn, stay, fadeOut) => this.finalCommand(['times', fadeIn, stay, fadeOut]);
}
export class TitleCommand extends CommandArguments {
    NodeType = TitleCommandNode;
    /**
     * Controls text displayed on the screen.
     *
     * @param targets Specifies the player(s) to display a screen title to.
     *
     * @example
     * // Displays a red "Hello World" as title
     * title('@a').title([{text: 'Hello World', color: 'red'}])
     *
     * // Displays a target to kill in the actionbar
     * const target = Selector('@r')
     * title('@a').actionbar(['You target is: ', target])
     */
    title = (targets) => this.subCommand([targetParser(targets)], (TitleArgumentsCommand), false);
}
