import { CommandNode } from 'sandstone/core/nodes';
import { CommandArguments } from '../../helpers.js';
export declare class DataPackCommandNode extends CommandNode {
    command: "datapack";
}
export declare class DataPackEnableCommand extends CommandArguments {
    /**
     * Load this pack before all others. It will have the lowest priority.
     */
    first: () => import("../../helpers.js").FinalCommandOutput;
    /**
     * Load this pack after all others. It will have the highest priority.
     */
    last: () => import("../../helpers.js").FinalCommandOutput;
    /**
     * Load this pack just before an existing pack.
     * It will have a lower priority than the existing one.
     *
     * @param name The name of the existing datapack.
     */
    before: (name: string) => import("../../helpers.js").FinalCommandOutput;
    /**
     * Load this pack just after an existing pack.
     * It will have a higher priority than the existing one.
     *
     * @param name The name of the existing datapack.
     */
    after: (name: string) => import("../../helpers.js").FinalCommandOutput;
}
export declare class DataPackCommand extends CommandArguments {
    protected NodeType: typeof DataPackCommandNode;
    /**
     * Disable the specified pack.
     *
     * @param name Specifies the name of the datapack.
     */
    disable: (name: string) => import("../../helpers.js").FinalCommandOutput;
    /**
     * Enable the specified pack.
     *
     * @param name Specifies the name of the datapack.
     *
     * @example
     *
     * // Enable `myDatapack`, and give it the highest priority
     * datapack.enable('myDatapack').last()
     *
     * // Enable `myDatapack`, just before `importantDatapack`.
     * // `mydatapack` will have a lower priority.
     * datapack.enable('myDatapack').before('importantDatapack')
     */
    enable: (name: string) => DataPackEnableCommand;
    /**
     * List all datapacks, or list only the available/enabled ones.
     *
     * Hovering over the datapacks in the chat output shows their description defined in their pack.mcmeta.
     *
     * @param typ `"available"` to only show available datapacks, `"enabled"` to only show enabled ones.
     */
    list: (type: 'available' | 'enabled') => import("../../helpers.js").FinalCommandOutput;
}
