import { CommandNode } from 'sandstone/core/nodes';
import { CommandArguments } from '../../helpers.js';
import type { Range } from 'sandstone/arguments';
import type { Macroable } from 'sandstone/core';
import type { LiteralUnion } from 'sandstone/utils.js';
export declare class RandomCommandNode extends CommandNode {
    command: "random";
}
export declare class RandomCommand<MACRO extends boolean> extends CommandArguments {
    protected NodeType: typeof RandomCommandNode;
    /**
     * Generate a random integer.
     *
     * @param range Integer between minimum and maximum.
     * @param sequence Namespaced ID of random sequence to use, created upon use.
     */
    value: (range: Macroable<Range<MACRO>, MACRO>, sequence: Macroable<string, MACRO>) => import("../../helpers.js").FinalCommandOutput;
    /**
     * Generate a random integer and broadcasts it to all players.
     *
     * @param range Integer between minimum and maximum.
     * @param sequence Optional. Namespaced ID of random sequence to use, created upon use.
     */
    roll: (range: Macroable<Range<MACRO>, MACRO>, sequence?: Macroable<string, MACRO>) => import("../../helpers.js").FinalCommandOutput;
    /**
     * Reset a random number sequence.
     *
     * @param sequence Namespaced ID of random sequence to reset, created if non-existent.
     * @param seed Optional. Long that seeds the new sequence.
     * @param includeWorldSeed Optional. Whether to include the world seed in the random. Defaults to false.
     * @param includeSequenceID Optional. Whether to include the sequence ID in the random. Defaults to false.
     * @returns
     */
    reset: (sequence: Macroable<LiteralUnion<'*'>, MACRO>, seed?: Macroable<number, MACRO>, includeWorldSeed?: Macroable<boolean, MACRO>, includeSequenceID?: Macroable<boolean, MACRO>) => import("../../helpers.js").FinalCommandOutput;
}
