import { CommandNode } from 'sandstone/core/nodes';
import { rangeParser } from 'sandstone/variables/parsers';
import { CommandArguments } from '../../helpers.js';
export class RandomCommandNode extends CommandNode {
    command = 'random';
}
export class RandomCommand extends CommandArguments {
    NodeType = RandomCommandNode;
    /**
     * Generate a random integer.
     *
     * @param range Integer between minimum and maximum.
     * @param sequence Namespaced ID of random sequence to use, created upon use.
     */
    value = (range, sequence) => {
        let seq = sequence;
        if (typeof sequence === 'string' && !sequence.includes(':')) {
            seq = `${this.sandstonePack.defaultNamespace}:${seq}`;
        }
        return this.finalCommand(['value', rangeParser(this.sandstoneCore, range), seq]);
    };
    /**
     * Generate a random integer and broadcasts it to all players.
     *
     * @param range Integer between minimum and maximum.
     * @param sequence Optional. Namespaced ID of random sequence to use, created upon use.
     */
    roll = (range, sequence) => {
        let seq = sequence;
        if (sequence && typeof sequence === 'string' && !sequence.includes(':')) {
            seq = `${this.sandstonePack.defaultNamespace}:${seq}`;
        }
        return this.finalCommand(['roll', rangeParser(this.sandstoneCore, range), seq]);
    };
    /**
     * Reset a random number sequence.
     *
     * @param sequence Namespaced ID of random sequence to reset, created if non-existent.
     * @param seed Optional. Long that seeds the new sequence.
     * @param includeWorldSeed Optional. Whether to include the world seed in the random. Defaults to false.
     * @param includeSequenceID Optional. Whether to include the sequence ID in the random. Defaults to false.
     * @returns
     */
    reset = (sequence, seed, includeWorldSeed, includeSequenceID) => {
        let seq = sequence;
        if (sequence && typeof sequence === 'string' && !sequence.includes(':')) {
            seq = `${this.sandstonePack.defaultNamespace}:${seq}`;
        }
        return this.finalCommand(['reset', seq, seed, includeWorldSeed, includeSequenceID]);
    };
}
