"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
var schedule_exports = {};
__export(schedule_exports, {
  ScheduleCommand: () => ScheduleCommand,
  ScheduleCommandNode: () => ScheduleCommandNode
});
module.exports = __toCommonJS(schedule_exports);
var import_nodes = require("sandstone/core/nodes");
var import_tag = require("sandstone/core/resources/datapack/tag");
var import_utils = require("sandstone/utils");
var import_helpers = require("../../helpers.js");
class ScheduleCommandNode extends import_nodes.ContainerCommandNode {
  constructor() {
    super(...arguments);
    __publicField(this, "command", "schedule");
    __publicField(this, "createMCFunction", (currentMCFunction) => {
      if (this.body.length === 0 || this.args[0] === "clear" || !currentMCFunction) {
        return { node: this };
      }
      const [_, func, time, append] = this.args;
      if (typeof func !== "string" && func !== void 0) {
        throw new Error("Cannot have something else than a string as the argument for a scheduled function with callback");
      }
      const mcFunction = this.sandstonePack.MCFunction(`${(0, import_utils.toMinecraftResourceName)(currentMCFunction.resource.path)}/${func != null ? func : "schedule"}`, {
        addToSandstoneCore: false,
        creator: "sandstone",
        onConflict: "rename"
      });
      const mcFunctionNode = mcFunction["node"];
      mcFunctionNode.body = this.body;
      this.body = [];
      this.args = ["function", mcFunction.name, time, append];
      return { node: this, mcFunction: mcFunctionNode };
    });
  }
}
class ScheduleCommand extends import_helpers.CommandArguments {
  constructor() {
    super(...arguments);
    __publicField(this, "NodeType", ScheduleCommandNode);
    /**
     * Removes a scheduled function.
     *
     * @param functionName Specify the scheduled function or `MCFunction` to be cleared.
     *
     */
    __publicField(this, "clear", (func) => {
      const result = this.finalCommand(["clear", func instanceof import_tag.TagClass ? func.name : func]);
      return result;
    });
    /**
     * Delays the execution of a function. Executes the function after specified amount of time passes.
     *
     * @param functionName Specify the function, the `MCFunction` or the callback to be scheduled.
     *
     * @param delay Specify the delay time.
     *
     * Must be a time in Minecraft. It must be a single-precision floating point number suffixed with a unit. Units include:
     * - d: an in-game day, 24000 gameticks;
     * - s: a second, 20 gameticks;
     * - t (default and omitable): a single gametick; the default unit.
     *
     * The time is set to the closest integer tick after unit conversion. For example. .5d is same as 12000 ticks.
     *
     * @param type
     * `replace` simply replaces the current function's schedule time. `append` allows multiple schedules to exist at different times.
     * If unspecified, defaults to `replace`.
     */
    __publicField(this, "function", (func, delay, type) => {
      const node = this.getNode();
      if (typeof func === "object" && Object.hasOwn(func, "addToTag")) {
        return this.finalCommand(["function", func.name, delay, type], node);
      }
      if (typeof func === "function") {
        this.sandstoneCore.insideContext(node, func, false);
        return this.finalCommand(["function", void 0, delay, type], node);
      }
      if (func instanceof import_tag.TagClass) {
        return this.finalCommand(["function", func.name, delay, type], node);
      }
      return this.finalCommand(["function", func, delay, type], node);
    });
  }
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  ScheduleCommand,
  ScheduleCommandNode
});
//# sourceMappingURL=schedule.cjs.map