import { ContainerCommandNode } from 'sandstone/core/nodes';
import { TagClass } from 'sandstone/core/resources/datapack/tag';
import { CommandArguments } from '../../helpers.js';
import type { TimeArgument } from 'sandstone/arguments';
import type { Macroable, Node } from 'sandstone/core';
import type { MCFunctionClass, MCFunctionNode } from 'sandstone/core/resources/datapack/index';
type ScheduledFunction = string | TagClass<'functions'> | MCFunctionClass<any, any> | (() => (any | Promise<any>));
export declare class ScheduleCommandNode extends ContainerCommandNode {
    command: "schedule";
    createMCFunction: (currentMCFunction: MCFunctionNode | null) => {
        node: Node | Node[];
        mcFunction?: MCFunctionNode | undefined;
    };
}
export type ScheduleType = 'append' | 'replace';
export declare class ScheduleCommand<MACRO extends boolean> extends CommandArguments<typeof ScheduleCommandNode> {
    protected NodeType: typeof ScheduleCommandNode;
    /**
     * Removes a scheduled function.
     *
     * @param functionName Specify the scheduled function or `MCFunction` to be cleared.
     *
     */
    clear: (func: Macroable<MCFunctionClass<any, any> | string | TagClass<'functions'>, MACRO>) => import("../../helpers.js").FinalCommandOutput;
    /**
     * Delays the execution of a function. Executes the function after specified amount of time passes.
     *
     * @param functionName Specify the function, the `MCFunction` or the callback to be scheduled.
     *
     * @param delay Specify the delay time.
     *
     * Must be a time in Minecraft. It must be a single-precision floating point number suffixed with a unit. Units include:
     * - d: an in-game day, 24000 gameticks;
     * - s: a second, 20 gameticks;
     * - t (default and omitable): a single gametick; the default unit.
     *
     * The time is set to the closest integer tick after unit conversion. For example. .5d is same as 12000 ticks.
     *
     * @param type
     * `replace` simply replaces the current function's schedule time. `append` allows multiple schedules to exist at different times.
     * If unspecified, defaults to `replace`.
     */
    function: (func: Macroable<ScheduledFunction, MACRO>, delay: Macroable<TimeArgument, MACRO>, type?: Macroable<ScheduleType, MACRO>) => import("../../helpers.js").FinalCommandOutput;
}
export {};
