"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
var data_exports = {};
__export(data_exports, {
  DataCommand: () => DataCommand,
  DataCommandNode: () => DataCommandNode,
  DataGetCommand: () => DataGetCommand,
  DataMergeCommand: () => DataMergeCommand,
  DataModifyCommand: () => DataModifyCommand,
  DataModifyTypeCommand: () => DataModifyTypeCommand,
  DataModifyValuesCommand: () => DataModifyValuesCommand,
  DataRemoveCommand: () => DataRemoveCommand
});
module.exports = __toCommonJS(data_exports);
var import_nodes = require("sandstone/core/nodes");
var import_NBTs = require("sandstone/variables/nbt/NBTs");
var import_parsers = require("sandstone/variables/parsers");
var import_helpers = require("../../helpers.js");
class DataCommandNode extends import_nodes.CommandNode {
  constructor() {
    super(...arguments);
    __publicField(this, "command", "data");
  }
}
class DataCommand extends import_helpers.CommandArguments {
  constructor() {
    super(...arguments);
    __publicField(this, "NodeType", DataCommandNode);
  }
  /** Read off the entire NBT data or the subsection of the NBT data from the targeted block position or entity, scaled by `scale` if specified. */
  get get() {
    return this.subCommand(["get"], DataGetCommand, false);
  }
  /** Merge the NBT data from the sourced block position or entity with the specified `nbt` data. */
  get merge() {
    return this.subCommand(["merge"], DataMergeCommand, false);
  }
  /** Modify the NBT data from the sourced block position or entity, with the specified operation and the given NBT. */
  get modify() {
    return this.subCommand(["modify"], DataModifyCommand, false);
  }
  /** Removes NBT data at `path` from the targeted block position or entity. Player NBT data cannot be removed. */
  get remove() {
    return this.subCommand(["remove"], DataRemoveCommand, false);
  }
}
class DataGetCommand extends import_helpers.CommandArguments {
  constructor() {
    super(...arguments);
    /**
     * Get the NBT of a block at the given position.
     *
     * @param targetPos The coordinates of the block to get the NBT from.
     * @param path The path of the NBT to get.
     * @param scale The scale to multiply the NBT value by.
     */
    __publicField(this, "block", (targetPos, path, scale) => this.finalCommand(["block", (0, import_parsers.coordinatesParser)(targetPos), path, scale]));
    /**
     * Get the NBT of a given entity.
     *
     * @param target The entity to get the NBT from.
     * @param path The path of the NBT to get.
     * @param scale The scale to multiply the NBT value by.
     */
    __publicField(this, "entity", (target, path, scale) => this.finalCommand(["entity", (0, import_parsers.targetParser)(target), path, scale]));
    /**
     * Get the NBT from a given storage path.
     *
     * @param target The storage to get the NBT from.
     * @param path The path of the NBT to get.
     * @param scale The scale to multiply the NBT value by.
     */
    __publicField(this, "storage", (target, path, scale) => this.finalCommand(["storage", target, path, scale]));
  }
}
class DataMergeCommand extends import_helpers.CommandArguments {
  constructor() {
    super(...arguments);
    /**
     * Merge the NBT of a block at the given position, with the given NBT.
     *
     * @param targetPos The coordinates of the block to merge the NBT with.
     * @param nbt The NBT to merge with.
     */
    __publicField(this, "block", (targetPos, nbt) => this.finalCommand(["block", (0, import_parsers.coordinatesParser)(targetPos), (0, import_NBTs.nbtStringifier)(nbt)]));
    /**
     * Merge the NBT of the given entity, with the given NBT.
     *
     * @param target The entity to merge the NBT with.
     * @param nbt The NBT to merge with.
     */
    __publicField(this, "entity", (target, nbt) => this.finalCommand(["entity", (0, import_parsers.targetParser)(target), (0, import_NBTs.nbtStringifier)(nbt)]));
    /**
     * Merge the NBT of the given storage path, with the given NBT.
     *
     * @param target The storage to merge the NBT with.
     * @param nbt The NBT to merge with.
     */
    __publicField(this, "storage", (target, nbt) => this.finalCommand(["storage", target, (0, import_NBTs.nbtStringifier)(nbt)]));
  }
}
class DataModifyValuesCommand extends import_helpers.CommandArguments {
  constructor() {
    super(...arguments);
    __publicField(this, "from", {
      /**
       * Modify with the NBT of a block at the given position.
       *
       * @param sourcePosition The coordinates of the block to modify the NBT with.
       * @param sourcePath The path of the NBT to modify with.
       */
      block: (sourcePosition, sourcePath) => this.finalCommand(["from", "block", (0, import_parsers.coordinatesParser)(sourcePosition), sourcePath]),
      /**
       * Modify with the NBT of a given entity.
       *
       * @param source The entity to modify the NBT with.
       * @param sourcePath The path of the NBT to modify with.
       */
      entity: (source, sourcePath) => this.finalCommand(["from", "entity", (0, import_parsers.targetParser)(source), sourcePath]),
      /**
       * Modify with the NBT of a given storage path.
       *
       * @param source The storage path to modify the NBT with.
       * @param sourcePath The path of the NBT to modify with.
       */
      storage: (source, sourcePath) => this.finalCommand(["from", "storage", source, sourcePath])
    });
    __publicField(this, "string", {
      /**
       * Modify with the NBT String of a block at the given position.
       *
       * @param sourcePosition The coordinates of the block to modify the NBT with.
       * @param sourcePath The path of the NBT to modify with.
       * @param start Optional. Index of first character to include at the start of the string.
       * @param end Optional. Index of the first character to exclude at the end of the string
       */
      block: (sourcePosition, sourcePath, start, end) => {
        const command = ["string", "block", (0, import_parsers.coordinatesParser)(sourcePosition), sourcePath];
        if (start) {
          command.push(start);
          if (end)
            command.push(end);
        }
        this.finalCommand(command);
      },
      /**
       * Modify with the NBT String of a given entity.
       *
       * @param source The entity to modify the NBT with.
       * @param sourcePath The path of the NBT to modify with.
       * @param start Optional. Index of first character to include at the start of the string.
       * @param end Optional. Index of the first character to exclude at the end of the string
       */
      entity: (source, sourcePath, start, end) => {
        const command = ["string", "entity", (0, import_parsers.targetParser)(source), sourcePath];
        if (start) {
          command.push(start);
          if (end)
            command.push(end);
        }
        this.finalCommand(command);
      },
      /**
       * Modify with the NBT String of a given storage path.
       *
       * @param source The storage path to modify the NBT with.
       * @param sourcePath The path of the NBT to modify with.
       * @param start Optional. Index of first character to include at the start of the string.
       * @param end Optional. Index of the first character to exclude at the end of the string
       */
      storage: (source, sourcePath, start, end) => {
        const command = ["string", "storage", source, sourcePath];
        if (start) {
          command.push(start);
          if (end)
            command.push(end);
        }
        this.finalCommand(command);
      }
    });
    /**
     * Modify the NBT with the given value.
     */
    __publicField(this, "value", (value) => this.finalCommand(["value", (0, import_NBTs.nbtStringifier)(value)]));
  }
}
class DataModifyTypeCommand extends import_helpers.CommandArguments {
  constructor() {
    super(...arguments);
    /**
     * Insert the source data into the pointed-to list as element `index`, then shift higher elements one position upwards.
     *
     * @param index The index to insert the NBT to.
     */
    __publicField(this, "insert", (index) => this.subCommand(["insert", index], DataModifyValuesCommand, false));
  }
  /** Append the source data onto the end of the pointed-to list. */
  get append() {
    return this.subCommand(["append"], DataModifyValuesCommand, false);
  }
  /** Merge the source data into the pointed-to object. */
  get merge() {
    return this.subCommand(["merge"], DataModifyValuesCommand, false);
  }
  /** Prepend the source data onto the beginning of the pointed-to list. */
  get prepend() {
    return this.subCommand(["prepend"], DataModifyValuesCommand, false);
  }
  /** Set the tag specified by `targetPath` to the source data. */
  get set() {
    return this.subCommand(["set"], DataModifyValuesCommand, false);
  }
}
class DataModifyCommand extends import_helpers.CommandArguments {
  constructor() {
    super(...arguments);
    /**
     * Modify the NBT of a block at the given position.
     *
     * @param targetPos The coordinates of the block to modify the NBT from.
     * @param path The path of the NBT to modify.
     */
    __publicField(this, "block", (targetPos, targetPath) => this.subCommand(["block", (0, import_parsers.coordinatesParser)(targetPos), targetPath], DataModifyTypeCommand, false));
    /**
     * Modify the NBT of a given entity.
     *
     * @param target The entity to modify the NBT from.
     * @param path The path of the NBT to modify.
     */
    __publicField(this, "entity", (target, targetPath) => this.subCommand(["entity", (0, import_parsers.targetParser)(target), targetPath], DataModifyTypeCommand, false));
    /**
     * Modify the NBT from a given storage path.
     *
     * @param target The storage to modify the NBT from.
     * @param path The path of the NBT to modify.
     */
    __publicField(this, "storage", (target, targetPath) => this.subCommand(["storage", target, targetPath], DataModifyTypeCommand, false));
  }
}
class DataRemoveCommand extends import_helpers.CommandArguments {
  constructor() {
    super(...arguments);
    /**
     * Remove the NBT of a block at the given position.
     *
     * @param targetPos The coordinates of the block to remove the NBT from.
     * @param path The path of the NBT to remove.
     */
    __publicField(this, "block", (targetPos, targetPath) => this.finalCommand(["block", (0, import_parsers.coordinatesParser)(targetPos), targetPath]));
    /**
     * Remove the NBT of a given entity.
     *
     * @param target The entity to remove the NBT from.
     * @param path The path of the NBT to remove.
     */
    __publicField(this, "entity", (target, targetPath) => this.finalCommand(["entity", (0, import_parsers.targetParser)(target), targetPath]));
    /**
     * Remove the NBT from a given storage path.
     *
     * @param target The storage to remove the NBT from.
     * @param path The path of the NBT to remove.
     */
    __publicField(this, "storage", (target, targetPath) => this.finalCommand(["storage", target, targetPath]));
  }
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  DataCommand,
  DataCommandNode,
  DataGetCommand,
  DataMergeCommand,
  DataModifyCommand,
  DataModifyTypeCommand,
  DataModifyValuesCommand,
  DataRemoveCommand
});
//# sourceMappingURL=data.cjs.map