"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
var forceload_exports = {};
__export(forceload_exports, {
  ForceLoadCommand: () => ForceLoadCommand,
  ForceLoadCommandNode: () => ForceLoadCommandNode
});
module.exports = __toCommonJS(forceload_exports);
var import_nodes = require("sandstone/core/nodes");
var import_parsers = require("sandstone/variables/parsers");
var import_helpers = require("../../helpers.js");
function coordinatesToNumbers(coords) {
  if (typeof coords === "string") {
    return coords.replace(/[~^]/, "").split(" ").map((i) => Number(i));
  }
  const realCoords = Array.isArray(coords) ? coords : coords.values;
  return realCoords.map((coord) => parseInt(coord.replace(/[\^~]/, "") || "0", 10));
}
function validateNumberOfChunks(from, to) {
  if (!to) {
    return;
  }
  if (typeof from === "object" || typeof to === "object") {
    return;
  }
  const coords = [...from, ...to].map(import_parsers.coordinatesParser);
  if (
    // If all coordinates are:
    coords.every((c) => c[0] === "~") || coords.every((c) => c[0] === "^") || coords.every((c) => c[0].match(/0-9/))
  ) {
    const [fromX, fromZ] = coordinatesToNumbers(from);
    const [toX, toZ] = coordinatesToNumbers(to);
    const chunksX = Math.ceil((Math.abs(fromX - toX) + 1) / 16);
    const chunksZ = Math.ceil((Math.abs(fromZ - toZ) + 1) / 16);
    const affectedChunks = chunksX * chunksZ;
    if (affectedChunks > 256) {
      throw new Error(`Impossible to forceload more than 256 chunks. From "${from}" to "${to}", at least ${affectedChunks} would be forceloaded.`);
    }
  }
}
class ForceLoadCommandNode extends import_nodes.CommandNode {
  constructor() {
    super(...arguments);
    __publicField(this, "command", "forceload");
  }
}
class ForceLoadCommand extends import_helpers.CommandArguments {
  constructor() {
    super(...arguments);
    __publicField(this, "NodeType", ForceLoadCommandNode);
    /**
     * Forces the chunk at the `from` position (through to `to` if set) in the dimension of the command's execution to be loaded constantly.
     *
     * Fails if more than 256 chunks are added at once.
     *
     * @param from Specifies the start of the targeted chunks, in block coordinates.
     * @param to Specified the end of the targeted chunks, in block coordinates.
     * If unspecified, only targets the chunk specified by `from`.
     *
     * @example
     * // Forceload current chunk
     * forceload.add('~ ~')
     */
    __publicField(this, "add", (from, to) => {
      validateNumberOfChunks(from, to);
      return this.finalCommand(["add", (0, import_parsers.coordinatesParser)(from), (0, import_parsers.coordinatesParser)(to)]);
    });
    /**
     * Unforces the chunk at the `from` position (through to `to` if set) in the dimension of the command's execution to be loaded constantly.
     *
     * @param from Specifies the start of the targeted chunks, in block coordinates.
     * @param to Specified the end of the targeted chunks, in block coordinates.
     * If unspecified, only targets the chunk specified by `from`.
     */
    __publicField(this, "remove", (from) => this.finalCommand(["remove", (0, import_parsers.coordinatesParser)(from)]));
    /** Unforces all chunks in the dimension of the command's execution to be loaded constantly. */
    __publicField(this, "removeAll", () => this.finalCommand(["remove", "all"]));
    /**
     * If chunk coordinates are given, displays whether the specified chunk in the dimension of the command's execution is force loaded.
     * Otherwise, lists which chunks in the dimension of the command's execution are force loaded.
     *
     * @param pos Specifies a chunk to query, in block coordinates.
     * If unspecified, lists which chunks are force loaded.
     *
     * @example
     */
    __publicField(this, "query", (pos) => this.finalCommand(["query", (0, import_parsers.coordinatesParser)(pos)]));
  }
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  ForceLoadCommand,
  ForceLoadCommandNode
});
//# sourceMappingURL=forceload.cjs.map