import { CommandNode } from 'sandstone/core/nodes';
import { CommandArguments } from '../../helpers.js';
import type { ColumnCoordinates } from 'sandstone/arguments';
import type { Macroable } from 'sandstone/core';
export declare class ForceLoadCommandNode extends CommandNode {
    command: "forceload";
}
/**
 * Force chunks to load constantly or not.
 */
export declare class ForceLoadCommand<MACRO extends boolean> extends CommandArguments {
    protected NodeType: typeof ForceLoadCommandNode;
    /**
     * Forces the chunk at the `from` position (through to `to` if set) in the dimension of the command's execution to be loaded constantly.
     *
     * Fails if more than 256 chunks are added at once.
     *
     * @param from Specifies the start of the targeted chunks, in block coordinates.
     * @param to Specified the end of the targeted chunks, in block coordinates.
     * If unspecified, only targets the chunk specified by `from`.
     *
     * @example
     * // Forceload current chunk
     * forceload.add('~ ~')
     */
    add: (from: Macroable<ColumnCoordinates<MACRO>, MACRO>, to?: Macroable<ColumnCoordinates<MACRO>, MACRO>) => import("../../helpers.js").FinalCommandOutput;
    /**
     * Unforces the chunk at the `from` position (through to `to` if set) in the dimension of the command's execution to be loaded constantly.
     *
     * @param from Specifies the start of the targeted chunks, in block coordinates.
     * @param to Specified the end of the targeted chunks, in block coordinates.
     * If unspecified, only targets the chunk specified by `from`.
     */
    remove: (from: Macroable<ColumnCoordinates<MACRO>, MACRO>) => import("../../helpers.js").FinalCommandOutput;
    /** Unforces all chunks in the dimension of the command's execution to be loaded constantly. */
    removeAll: () => import("../../helpers.js").FinalCommandOutput;
    /**
     * If chunk coordinates are given, displays whether the specified chunk in the dimension of the command's execution is force loaded.
     * Otherwise, lists which chunks in the dimension of the command's execution are force loaded.
     *
     * @param pos Specifies a chunk to query, in block coordinates.
     * If unspecified, lists which chunks are force loaded.
     *
     * @example
     */
    query: (pos?: Macroable<ColumnCoordinates<MACRO>, MACRO>) => import("../../helpers.js").FinalCommandOutput;
}
