import { CommandNode } from 'sandstone/core/nodes';
import { coordinatesParser, targetParser } from 'sandstone/variables/parsers';
import { CommandArguments } from '../../helpers.js';
export class ItemCommandNode extends CommandNode {
    command = 'item';
}
export class ItemSourceCommand extends CommandArguments {
    /**
     * Replace the slot with a specific item.
     * @param item The item to replace the slot with.
     * @param count The amount of items.
     */
    with = (item, count) => this.finalCommand(['with', item, count]);
    from = {
        /**
         * @param pos The coordinates of the container to copy items from.
         * @param slot The slot to copy the items from.
         * @param [modifier] An optional modifier to apply.
         */
        block: (pos, slot, modifier) => this.finalCommand(['from', 'block', pos, slot, modifier]),
        /**
         * @param targets The entity to copy items from.
         * @param slot The slot to copy the items from.
         * @param [modifier] An optional modifier to apply.
         */
        entity: (targets, slot, modifier) => this.finalCommand(['from', 'entity', targetParser(targets), slot, modifier]),
    };
}
/** Replaces or modifies items in inventories */
export class ItemCommand extends CommandArguments {
    NodeType = ItemCommandNode;
    /** Applies a modifier to a slot in an inventory. */
    modify = {
        /**
         * @param pos The position of the container containing the slot to apply the modifier to.
         * @param slot The slot to apply the modifier to.
         * @param modifier The name of the modifier.
         */
        block: (pos, slot, modifier) => this.finalCommand(['modify', 'block', coordinatesParser(pos), slot, modifier]),
        /**
         * @param targets The entity/entities containing the slot to apply the modifier to.
         * @param slot The slot to apply the modifier to.
         * @param modifier The name of the modifier.
         */
        entity: (targets, slot, modifier) => this.finalCommand(['modify', 'entity', targetParser(targets), slot, modifier]),
    };
    /** Replaces the content of a specific slot of an inventory with another. */
    replace = {
        /**
         * @param pos The position of the container containing the slot to be replaced.
         * @param slot The slot to be replaced.
         */
        block: (pos, slot) => this.subCommand(['block', pos, slot], (ItemSourceCommand), false),
        /**
         * @param targets one or more entities to modify.
         *.
         * @param slot The slot to be replaced.
         */
        entity: (targets, slot) => this.subCommand(['entity', targetParser(targets), slot], (ItemSourceCommand), false),
    };
}
