"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
var loot_exports = {};
__export(loot_exports, {
  LootCommand: () => LootCommand,
  LootCommandNode: () => LootCommandNode
});
module.exports = __toCommonJS(loot_exports);
var import_validators = require("sandstone/commands/validators");
var import_nodes = require("sandstone/core/nodes");
var import_parsers = require("sandstone/variables/parsers");
var import_helpers = require("../../helpers.js");
class LootCommandNode extends import_nodes.CommandNode {
  constructor() {
    super(...arguments);
    __publicField(this, "command", "loot");
  }
}
class LootSourceCommand extends import_helpers.CommandArguments {
  constructor() {
    super(...arguments);
    /**
     * Drops items that would be dropped by fishing with the given loot table, and the given tool.
     *
     * @param lootTable Specifies which loot table to use.
     *
     * @param pos Specifies the position of a block.
     *
     * @param tool Specifies an tool to fish.
     */
    __publicField(this, "fish", (lootTable, pos, tool) => this.finalCommand(["fish", lootTable, (0, import_parsers.coordinatesParser)(pos), tool]));
    /**
     * Drops items that would be dropped by the given loot table.
     *
     * @param lootTable Specifies which loot table to use.
     */
    __publicField(this, "loot", (lootTable) => this.finalCommand(["loot", lootTable]));
    /**
     * Drops items that would be dropped by killing the given entity.
     *
     * @param target Specifies one entity to kill simulatively.
     */
    __publicField(this, "kill", (target) => this.finalCommand(["kill", (0, import_parsers.targetParser)(target)]));
    /**
     * Drops items that would be dropped by mining the given block, with the given tool.
     *
     * @param pos Specifies the position of a block.
     *
     * @param tool Specifies an tool to mine.
     */
    __publicField(this, "mine", (pos, tool) => this.finalCommand(["mine", (0, import_parsers.coordinatesParser)(pos), tool]));
  }
}
class LootCommand extends import_helpers.CommandArguments {
  constructor() {
    super(...arguments);
    __publicField(this, "NodeType", LootCommandNode);
    /**
     * Spawns item entities.
     * @param targetPos Specifies the location where item drops.
     */
    __publicField(this, "spawn", (targetPos) => this.subCommand(["spawn", (0, import_parsers.coordinatesParser)(targetPos)], LootCommand, false));
    __publicField(this, "replace", {
      /**
       * Distributes items to entities.
       *
       * @param entities Specifies one or more entities to modify
       *
       * @param slot Specifies the inventory slot to be modified. Must be one of the following:
       *
       * - `armor.chest`, `armor.feet`, `armor.head`, `armor.legs`, `weapon.mainhand`, `weapon.offhand`:
       *   for armor stands, mobs, and players only (though not all mobs show or make use of the items).
       *
       * - `container.<slot_number>`: for players, item frames (slot 0), and minecarts only.
       *
       * - `enderchest.slot_number`, `hotbar.slot_number`, `inventory.slot_number`: for players only.
       *
       * - `horse.saddle`: horses, donkeys, and mules only; *item* must be a saddle.
       *
       * - `horse.chest`: donkeys, and mules only; *item* must be a chest.
       *
       * - `horse.armor`: horses and llamas only; *item* must be a type of horse armor (if a horse) or a carpet (if a llama).
       *
       * - `horse.slot_number`: donkeys and mules with chests only.
       *
       * - `villager.slot_number`: villagers only.
       *
       * @param count Specifies the number of consecutive slots to be filled. Must be between 0 and 2147483647 (inclusive).
       */
      entity: (entities, slot, count) => {
        if (count)
          (0, import_validators.validateIntegerRange)(count, "count", 0, 2147483647);
        return this.subCommand(["replace", "entity", (0, import_parsers.targetParser)(entities), slot, count], LootSourceCommand, false);
      },
      /**
       * Replace an entity slot with the items.
       *
       * @param targetPos Specifies the position of a block.
       *
       * @param slot Specifies the inventory slot to be modified.
       * Must be must be `container.<slot_number>` where `<slot_number>` is replaced with a number specifying the slot.
       *
       * - Chests, dispensers, droppers, hoppers, and trapped chests are numbered 0 for the top-left slot and then increase
       *   first horizontally, then vertically (so, for example, a chest's top row slots are numbered 0 to 8 from left to right).
       *   Double chests and double trapped chests are treated as two single container blocks.
       * - A brewing stand's bottom slots are numbered 0 to 2 from left to right, its top slot is 3 and the fuel slot is 4.
       * - A furnace's slots are numbered 0 for the input slot, 1 for the fuel slot, and 2 for the output slot.
       *
       * @param count Specifies the number of consecutive slots to be filled. Must be between 0 and 2147483647 (inclusive).
       */
      block: (targetPos, slot, count) => {
        if (count)
          (0, import_validators.validateIntegerRange)(count, "count", 0, 2147483647);
        return this.subCommand(["replace", "block", (0, import_parsers.coordinatesParser)(targetPos), slot, count], LootSourceCommand, false);
      }
    });
    /**
     * Gives items to players, ignoring empty item stacks.
     *
     * @param players Specifies one or more players to give.
     */
    __publicField(this, "give", (players) => this.subCommand(["give", (0, import_parsers.targetParser)(players)], LootSourceCommand, false));
    /**
     * Distributes items to a container block.
     *
     * @param targetPos Specifies the position of a block.
     */
    __publicField(this, "insert", (targetPos) => this.subCommand(["insert", (0, import_parsers.coordinatesParser)(targetPos)], LootSourceCommand, false));
  }
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  LootCommand,
  LootCommandNode
});
//# sourceMappingURL=loot.cjs.map