import { CommandNode } from 'sandstone/core/nodes';
import { CommandArguments } from '../../helpers.js';
import type { TimeArgument } from 'sandstone/arguments';
import type { Macroable } from 'sandstone/core';
export declare class WeatherCommandNode extends CommandNode {
    command: "weather";
}
export declare class WeatherCommand<MACRO extends boolean> extends CommandArguments {
    protected NodeType: typeof WeatherCommandNode;
    /**
     * Set the weather to clear.
     *
     * @param duration Specifies the time in seconds for the clear weather to last.
     * If not specified, the duration defaults to 5 minutes.
     * If 0, resets to random weather duration.
     */
    clear: (duration: Macroable<TimeArgument, MACRO>) => import("../../helpers.js").FinalCommandOutput;
    /**
     * Set the weather to rain (or snow in cold biomes).
     *
     * @param duration Specifies the time in seconds for the rainy weather to last.
     * If not specified, the duration defaults to 5 minutes.
     * If 0, resets to random weather duration.
     */
    rain: (duration: Macroable<TimeArgument, MACRO>) => import("../../helpers.js").FinalCommandOutput;
    /**
     * Set the weather to a thunderstorm (or blizzard cold biomes).
     *
     * @param duration Specifies the time in seconds for the thunderstorm to last.
     * If not specified, the duration defaults to 5 minutes.
     * If 0, resets to random weather duration.
     */
    thunder: (duration: Macroable<TimeArgument, MACRO>) => import("../../helpers.js").FinalCommandOutput;
}
