"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
var worldborder_exports = {};
__export(worldborder_exports, {
  WorldBorderCommand: () => WorldBorderCommand,
  WorldBorderNode: () => WorldBorderNode
});
module.exports = __toCommonJS(worldborder_exports);
var import_nodes = require("sandstone/core/nodes");
var import_parsers = require("sandstone/variables/parsers");
var import_helpers = require("../../helpers.js");
class WorldBorderNode extends import_nodes.CommandNode {
  constructor() {
    super(...arguments);
    __publicField(this, "command", "worldborder");
  }
}
class WorldBorderCommand extends import_helpers.CommandArguments {
  constructor() {
    super(...arguments);
    __publicField(this, "NodeType", WorldBorderNode);
    /**
     * Increases or decreases the world border diameter.
     *
     * @param distance Specifies the number of blocks to add to the world border diameter.
     *
     * @param time Specifies the number of seconds it should take for the world border to move from its current diameter to the new diameter.
     * If not specified, defaults to 0.
     */
    __publicField(this, "add", (distance, time) => this.finalCommand(["add", distance, time]));
    /**
     * Recenters the world boundary.
     *
     * @param pos Specifies the horizontal coordinates of the world border's center.
     */
    __publicField(this, "center", (pos) => this.finalCommand(["center", (0, import_parsers.coordinatesParser)(pos)]));
    __publicField(this, "damage", {
      /**
       * Sets the world border damage amount to the specified value.
       * Any player outside the world border buffer takes this amount of damage per second per block past the world border buffer distance.
       *
       * @param damagePerBlock Specifies the damage a player takes per second per block past the world border buffer.
       * For example, if `damagePerBlock` is 0.1, a player 5 blocks outside the world border buffer takes 0.5 damage per second
       * (damage less than half a heart might not change the visual health display, but still accumulates). Initially set to 0.2.
       */
      amount: (damagePerBlock) => this.finalCommand(["damage", "amount", damagePerBlock]),
      /**
       * Sets the world border buffer distance to the specified value.
       * Players won't take damage until they move past this distance from the world border.
       *
       * @param distance Specifies the distance outside the world buffer a player must be before they start taking damage.
       * Initially set to 5.0.
       */
      buffer: (distance) => this.finalCommand(["damage", "buffer", distance])
    });
    /**
     * Returns the current world border diameter.
     */
    __publicField(this, "get", () => this.finalCommand(["get"]));
    /**
     * Sets the world border diameter.
     *
     * @param distance Specifies the new diameter for the world border.
     *
     * @param time Specifies the number of seconds it should take for the world border to move from its current diameter to the new diameter.
     * If not specified, defaults to 0.
     */
    __publicField(this, "set", (distance, time) => this.finalCommand(["set", distance, time]));
    __publicField(this, "warning", {
      /**
       * Sets the world border warning distance to the specified value.
       * Players will start to see a warning message when they are this distance from the world border.
       *
       * @param distance Specifies the distance from the world border at which players begins to see a visual warning of the world border's proximity.
       * Initially set to 5.
       */
      distance: (distance) => this.finalCommand(["warning", "distance", distance]),
      /**
       * Sets the world border warning time to the specified value.
       *
       * @param time Specifies the number of seconds that a player begins to see a visual warning before a moving world border passes their position.
       * Initially set to 15.
       */
      time: (time) => this.finalCommand(["warning", "time", time])
    });
  }
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  WorldBorderCommand,
  WorldBorderNode
});
//# sourceMappingURL=worldborder.cjs.map