import type { SandstonePack } from 'sandstone/pack';
import type { MCFunctionClass, MCFunctionNode } from './resources/datapack/index.js';
import type { SandstoneCore } from './sandstoneCore.js';
export declare abstract class Node {
    sandstoneCore: SandstoneCore;
    constructor(sandstoneCore: SandstoneCore);
    abstract getValue(): any;
}
/**
 * A node that includes other nodes.
 */
export declare abstract class ContainerNode extends Node {
    body: Node[];
    constructor(sandstoneCore: SandstoneCore);
    /**
     * Appends a node at the end of this node's body.
     */
    append<NODE extends Node>(node: NODE): NODE;
    /**
     * Appends several nodes at the end of this node's body.
     */
    append<NODES extends Node[]>(...nodes: NODES): NODES;
    /**
     * Prepends a node to the beginning of this node's body.
     */
    prepend<NODE extends Node>(node: NODE): NODE;
    /**
     * Prepends several nodes to the beginning of this node's body.
     */
    prepend<NODES extends Node[]>(...nodes: NODES): NODES;
}
/**
 * A node that represents a generic command.
 */
export declare abstract class CommandNode<ARGS extends unknown[] = unknown[]> extends Node {
    sandstonePack: SandstonePack;
    abstract command: string;
    args: ARGS;
    commited: boolean;
    isMacro: boolean;
    constructor(sandstonePack: SandstonePack, ...args: ARGS);
    getValue(): string;
    /**
     * Commits the command to the current MCFunction context.
     */
    commit(): void | this;
}
/**
 * A node that includes other nodes.
 */
export declare abstract class ContainerCommandNode<ARGS extends unknown[] = unknown[]> extends CommandNode<ARGS> implements ContainerNode {
    abstract command: string;
    body: Node[];
    constructor(sandstonePack: SandstonePack, ...args: ARGS);
    /**
     * Appends a node at the end of this node's body.
     */
    append(node: Node): Node;
    /**
     * Appends a node at the end of this node's body.
     */
    prepend(node: Node): Node;
    /**
     * Create a MCFunction from this node.
     * It shouldn't be added to Sandstone's core.
     *
     * The returned node will replace
     */
    createMCFunction: (currentMCFunction: MCFunctionNode | null) => {
        node: Node | Node[];
        mcFunction?: MCFunctionNode;
    };
}
export declare abstract class AwaitNode extends ContainerCommandNode {
    mcfunction: MCFunctionClass<any, any>;
}
