import { isMacroArgument } from './Macro.js';
export class Node {
    sandstoneCore;
    constructor(sandstoneCore) {
        this.sandstoneCore = sandstoneCore;
    }
}
/**
 * A node that includes other nodes.
 */
export class ContainerNode extends Node {
    body;
    constructor(sandstoneCore) {
        super(sandstoneCore);
        this.body = [];
    }
    append(...nodes) {
        this.body.push(...nodes);
        return nodes.length === 1 ? nodes[0] : nodes;
    }
    prepend(...nodes) {
        this.body.unshift(...nodes);
        return nodes.length === 1 ? nodes[0] : nodes;
    }
}
/**
 * A node that represents a generic command.
 */
export class CommandNode extends Node {
    sandstonePack;
    args;
    commited = false;
    isMacro = false;
    constructor(sandstonePack, ...args) {
        super(sandstonePack.core);
        this.sandstonePack = sandstonePack;
        this.args = args;
    }
    getValue() {
        const filteredArgs = [];
        for (const arg of this.args) {
            if (arg !== undefined && arg !== null) {
                // Yes these are cursed, unfortunately, there's not really a better way to do this as visitors only visit the root nodes.
                if (typeof arg === 'object') {
                    if (isMacroArgument(this.sandstoneCore, arg)) {
                        this.isMacro = true;
                        filteredArgs.push(arg.toMacro());
                    }
                    else if (Object.hasOwn(arg, 'toLoop')) {
                        filteredArgs.push(arg.toLoop());
                    }
                    else {
                        filteredArgs.push(arg);
                    }
                }
                else {
                    filteredArgs.push(arg);
                }
            }
        }
        return `${this.isMacro ? '$' : ''}${this.command} ${filteredArgs.join(' ')}`;
    }
    /**
     * Commits the command to the current MCFunction context.
     */
    commit() {
        if (this.commited) {
            return this;
        }
        this.commited = true;
        return this.sandstonePack.appendNode(this);
    }
}
/**
 * A node that includes other nodes.
 */
export class ContainerCommandNode extends CommandNode {
    body;
    constructor(sandstonePack, ...args) {
        super(sandstonePack, ...args);
        this.body = [];
    }
    /**
     * Appends a node at the end of this node's body.
     */
    append(node) {
        this.body.push(node);
        return node;
    }
    /**
     * Appends a node at the end of this node's body.
     */
    prepend(node) {
        this.body.unshift(node);
        return node;
    }
    /**
     * Create a MCFunction from this node.
     * It shouldn't be added to Sandstone's core.
     *
     * The returned node will replace
     */
    createMCFunction = (currentMCFunction) => ({ node: this });
}
export class AwaitNode extends ContainerCommandNode {
    mcfunction = undefined;
}
