/// <reference types="node" resolution-mode="require"/>
/// <reference types="node" resolution-mode="require"/>
import { ContainerNode } from '../nodes.js';
import { ResourceClass } from './resource.js';
import type fs from 'fs-extra';
import type { PackType } from 'sandstone/pack/packType';
import type { SandstoneCore } from '../sandstoneCore.js';
import type { ResourceClassArguments, ResourceNode } from './resource.js';
/**
 * A node representing a custom resource.
 */
export declare class CustomResourceNode extends ContainerNode implements ResourceNode<CustomResourceClass> {
    resource: CustomResourceClass;
    constructor(sandstoneCore: SandstoneCore, resource: CustomResourceClass);
    getValue: () => string | Buffer | Promise<Buffer>;
}
export type CustomResourceClassArguments = {
    /** The custom resource type. */
    type: string;
    /** Optional. The Pack this resource is exported into. Defaults to the datapack. */
    packType?: PackType;
    /** Optional. The folder in the pack where the resource gets stored. Defaults to the root of the pack without a namespace. */
    folder?: string[];
    /** Optional. File extension of the resource. Defaults to `json`. */
    extension?: string;
    /** Optional. How `SandstoneCore.getExistingResource` should read this resource. If `false` the encoding will not be provided to `fs.readFile`. Defaults to `'utf8'`. */
    encoding?: false | fs.EncodingOption;
} & ResourceClassArguments<'default'>;
export declare abstract class CustomResourceClass extends ResourceClass<CustomResourceNode> {
    type: string;
    folder?: string[];
    constructor(sandstoneCore: SandstoneCore, name: string, args: CustomResourceClassArguments);
    getValue(): string | Buffer | Promise<Buffer>;
    toString: () => string;
}
