/* eslint-disable max-len */
import { toMinecraftResourceName } from 'sandstone/utils';
import { ContainerNode } from '../nodes.js';
import { ResourceClass } from './resource.js';
/**
 * A node representing a custom resource.
 */
export class CustomResourceNode extends ContainerNode {
    resource;
    constructor(sandstoneCore, resource) {
        super(sandstoneCore);
        this.resource = resource;
    }
    getValue = () => this.resource.getValue();
}
export class CustomResourceClass extends ResourceClass {
    type;
    folder;
    constructor(sandstoneCore, name, args) {
        sandstoneCore.pack.registerCustomResource(args.type);
        super(sandstoneCore, { packType: args.packType || sandstoneCore.pack.dataPack(), extension: args.extension || 'json', encoding: args.encoding }, CustomResourceNode, 
        // eslint-disable-next-line no-nested-ternary
        args.folder ? name.includes('/') ? name.split('/') : [name] : sandstoneCore.pack.resourceToPath(name, args.folder), args);
        this.type = args.type;
        this.folder = args.folder;
        this.handleConflicts();
    }
    getValue() {
        throw new Error(`Custom Resource '${this.type}' getValue function is not defined!`);
    }
    toString = () => (this.folder ? toMinecraftResourceName(this.path, this.folder.length) : this.name);
}
