"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
var advancement_exports = {};
__export(advancement_exports, {
  AdvancementClass: () => AdvancementClass,
  AdvancementNode: () => AdvancementNode
});
module.exports = __toCommonJS(advancement_exports);
var import_nodes = require("../../nodes.js");
var import_resource = require("../resource.js");
class AdvancementNode extends import_nodes.ContainerNode {
  constructor(sandstoneCore, resource) {
    super(sandstoneCore);
    this.resource = resource;
    __publicField(this, "getValue", () => JSON.stringify(this.resource.advancementJSON));
  }
}
class AdvancementClass extends import_resource.ResourceClass {
  constructor(sandstoneCore, name, args) {
    super(sandstoneCore, { packType: sandstoneCore.pack.dataPack(), extension: "json" }, AdvancementNode, sandstoneCore.pack.resourceToPath(name, ["advancements"]), args);
    __publicField(this, "advancementJSON");
    /**
     * @internal
     */
    __publicField(this, "_toMinecraftCondition", () => new this.pack.conditions.Advancement(this.core, this.name));
    this.advancementJSON = args.advancement;
    this.handleConflicts();
  }
  /**
   * Grant this advancement.
   * @param players Optional. Specifies the player(s). Defaults to `@s`.
   */
  grant(players = "@s", criterion) {
    this.pack.commands.advancement.grant(players).only(this.name, criterion);
  }
  /**
   * Revoke this advancement.
   * @param players Optional. Specifies the player(s). Defaults to `@s`.
   */
  revoke(players = "@s", criterion) {
    this.pack.commands.advancement.revoke(players).only(this.name, criterion);
  }
  /**
   * Grant this advancement and all its parent advancements.
   * Think of specifying everything from the start *until* that advancement.
   *
   * The exact order the operation is carried out in is: `parent > parent's parent > ... > root > this advancement.`
   *
   * @param players Optional. Specifies the player(s). Defaults to `@s`.
   */
  grantUntilThis(players = "@s") {
    this.pack.commands.advancement.grant(players).until(this.name);
  }
  /**
   * Revoke this advancement and all its parent advancements.
   * Think of specifying everything from the start *until* that advancement.
   *
   * The exact order the operation is carried out in is: `parent > parent's parent > ... > root > this advancement.`
   *
   * @param players Optional. Specifies the player(s). Defaults to `@s`.
   */
  revokeUntilThis(players = "@s") {
    this.pack.commands.advancement.revoke(players).until(this.name);
  }
  /**
   * Grant this advancement and all its children advancements.
   * Think of specifying everything *from* that advancement to the end.
   *
   * The exact order the operation is carried out in is specified `advancement > child > child's child > ...`.
   * When it operates on a child that branches, it iterates through all its children before continuing.
   *
   * @param players Optional. Specifies the player(s). Defaults to `@s`.
   */
  grantFromThis(players = "@s") {
    this.pack.commands.advancement.grant(players).from(this.name);
  }
  /**
   * Revoke this advancement and all its children advancements.
   * Think of specifying everything *from* that advancement to the end.
   *
   * The exact order the operation is carried out in is specified `advancement > child > child's child > ...`.
   * When it operates on a child that branches, it iterates through all its children before continuing.
   *
   * @param players Optional. Specifies the player(s). Defaults to `@s`.
   */
  revokeFromThis(players = "@s") {
    this.pack.commands.advancement.revoke(players).from(this.name);
  }
  /**
   * Grant this advancement, all its parent advancements, and all its children advancements.
   * Think of specifying everything through the specified advancement, going both backwards and forwards.
   *
   * The exact order the operation is as if the command were executed with `until` specified, then with `from` specified:
   * `parent > parent's parent > ... > root > specified advancement > child > child's child > ...`
   *
   * @param players Optional. Specifies the player(s). Defaults to `@s`.
   */
  grantThroughThis(players = "@s") {
    this.pack.commands.advancement.grant(players).through(this.name);
  }
  /**
   * Revoke this advancement, all its parent advancements, and all its children advancements.
   * Think of specifying everything through the specified advancement, going both backwards and forwards.
   *
   * The exact order the operation is as if the command were executed with `until` specified, then with `from` specified:
   * `parent > parent's parent > ... > root > specified advancement > child > child's child > ...`
   *
   * @param players Optional. Specifies the player(s). Defaults to `@s`.
   */
  revokeThroughThis(players = "@s") {
    this.pack.commands.advancement.revoke(players).through(this.name);
  }
  /**
   * Test if the currently executing player has a specific advancement criterion.
   */
  criterion(criterion) {
    return new this.pack.conditions.Advancement(this.core, this.name, criterion);
  }
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  AdvancementClass,
  AdvancementNode
});
//# sourceMappingURL=advancement.cjs.map