import { ContainerNode } from '../../nodes.js';
import { ResourceClass } from '../resource.js';
import type { Coordinates, DamageTypeJSON, SingleEntityArgument, TAG_DAMAGE_TYPES } from 'sandstone/arguments';
import type { ComponentClass } from 'sandstone/variables';
import type { SandstoneCore } from '../../sandstoneCore.js';
import type { ResourceClassArguments, ResourceNode } from '../resource.js';
/**
 * A node representing a Minecraft damage type.
 */
export declare class DamageTypeNode extends ContainerNode implements ResourceNode<DamageTypeClass> {
    resource: DamageTypeClass;
    constructor(sandstoneCore: SandstoneCore, resource: DamageTypeClass);
    getValue: () => string;
}
export type DamageTypeClassArguments = {
    /**
     * The damage type's JSON.
     */
    damageType?: DamageTypeJSON;
} & ResourceClassArguments<'default'> & {
    /**
     * Optional. Automatically adds damage type to minecraft damage type group tag flags.
     */
    flags?: (TAG_DAMAGE_TYPES | 'bypasses_cooldown')[];
};
export declare class DamageTypeClass extends ResourceClass<DamageTypeNode> implements ComponentClass {
    damageTypeJSON: NonNullable<DamageTypeClassArguments['damageType']>;
    constructor(sandstoneCore: SandstoneCore, name: string, args: DamageTypeClassArguments);
    get translationKey(): string;
    damage(amount: number, context?: 'entity' | 'at', source?: Coordinates<false> | SingleEntityArgument<false>, cause?: SingleEntityArgument<false>): void;
    damage(target: SingleEntityArgument<false>, amount: number, context?: 'entity' | 'at', source?: Coordinates<false> | SingleEntityArgument<false>, cause?: SingleEntityArgument<false>): void;
    /** @internal */
    _toChatComponent: () => {
        translate: string;
    };
    toString: () => string;
}
