/* eslint-disable max-len */
import { toMinecraftResourceName } from 'sandstone/utils';
import { ContainerNode } from '../../nodes.js';
import { ResourceClass } from '../resource.js';
const damageTypes = new Map();
/**
 * A node representing a Minecraft damage type.
 */
export class DamageTypeNode extends ContainerNode {
    resource;
    constructor(sandstoneCore, resource) {
        super(sandstoneCore);
        this.resource = resource;
    }
    getValue = () => JSON.stringify(this.resource.damageTypeJSON);
}
export class DamageTypeClass extends ResourceClass {
    damageTypeJSON;
    constructor(sandstoneCore, name, args) {
        super(sandstoneCore, { packType: sandstoneCore.pack.dataPack(), extension: 'json' }, DamageTypeNode, sandstoneCore.pack.resourceToPath(name, ['trim_materials']), args);
        this.damageTypeJSON = args.damageType;
        if (args.flags) {
            for (const flag of args.flags) {
                let tag = damageTypes.get(flag);
                if (!tag) {
                    damageTypes.set(flag, sandstoneCore.pack.Tag('damage_type', `minecraft:${flag}`, []));
                    tag = damageTypes.get(flag);
                }
                tag?.push(this.name);
            }
        }
        this.handleConflicts();
    }
    get translationKey() {
        return `death.attack.${this.damageTypeJSON.message_id}${this.damageTypeJSON.death_message_type === 'intentional_game_design' ? '.link' : ''}`;
    }
    damage(...args
    // eslint-disable-next-line function-paren-newline
    ) {
        const command = this.core.pack.commands.damage;
        if (typeof args[0] === 'number') {
            if (args[1] === 'entity') {
                if (args[3]) {
                    return command('@s', args[0], this).by(args[2]).from(args[3]);
                }
                return command('@s', args[0], this).by(args[2]);
            }
            if (args[1] === 'at') {
                return command('@s', args[0], this).at(args[1]);
            }
            return command('@s', args[0], this);
        }
        if (args[2] === 'entity') {
            if (args[4]) {
                return command(args[0], args[1], this).by(args[2]).from(args[4]);
            }
            return command(args[0], args[1], this).by(args[2]);
        }
        return command(args[0], args[1], this);
    }
    /** @internal */
    _toChatComponent = () => ({ translate: this.translationKey });
    toString = () => toMinecraftResourceName(this.path);
}
