import { ContainerNode } from '../../nodes.js';
import { ResourceClass } from '../resource.js';
import type { CONTAINER_SLOTS, Coordinates, ENTITY_SLOTS, ItemModifierJSON, MultipleEntitiesArgument } from 'sandstone/arguments';
import type { SandstoneCore } from '../../sandstoneCore.js';
import type { ListResource, ResourceClassArguments, ResourceNode } from '../resource.js';
/**
 * A node representing a Minecraft item modifier.
 */
export declare class ItemModifierNode extends ContainerNode implements ResourceNode<ItemModifierClass> {
    resource: ItemModifierClass;
    constructor(sandstoneCore: SandstoneCore, resource: ItemModifierClass);
    getValue: () => string;
}
export type ItemModifierClassArguments = {
    /**
     * The item modifier's JSON.
     */
    itemModifier?: ItemModifierJSON;
} & ResourceClassArguments<'list'>;
type Modifier = ItemModifierJSON | ItemModifierClass;
export declare class ItemModifierClass extends ResourceClass<ItemModifierNode> implements ListResource {
    itemModifierJSON: NonNullable<ItemModifierClassArguments['itemModifier']>;
    constructor(sandstoneCore: SandstoneCore, name: string, args: ItemModifierClassArguments);
    push(...modifiers: Modifier[]): void;
    unshift(...modifiers: Modifier[]): void;
    get modify(): {
        /**
         * @param pos The position of the container containing the slot to apply the modifier to.
         * @param slot The slot to apply the modifier to.
         */
        block: (pos: Coordinates<false>, slot: CONTAINER_SLOTS) => void;
        /**
         * @param targets The entity/entities containing the slot to apply the modifier to.
         * @param slot The slot to apply the modifier to.
         * @param modifier The name of the modifier.
         */
        entity: (targets: MultipleEntitiesArgument<false>, slot: ENTITY_SLOTS) => void;
    };
}
export {};
