import { targetParser } from 'sandstone/variables/parsers';
import { ContainerNode } from '../../nodes.js';
import { ResourceClass } from '../resource.js';
/**
 * A node representing a Minecraft item modifier.
 */
export class ItemModifierNode extends ContainerNode {
    resource;
    constructor(sandstoneCore, resource) {
        super(sandstoneCore);
        this.resource = resource;
    }
    getValue = () => JSON.stringify(this.resource.itemModifierJSON);
}
export class ItemModifierClass extends ResourceClass {
    itemModifierJSON;
    constructor(sandstoneCore, name, args) {
        super(sandstoneCore, { packType: sandstoneCore.pack.dataPack(), extension: 'json' }, ItemModifierNode, sandstoneCore.pack.resourceToPath(name, ['item_modifiers']), args);
        this.itemModifierJSON = args.itemModifier;
        this.handleConflicts();
    }
    push(...modifiers) {
        if (!Array.isArray(this.itemModifierJSON)) {
            this.itemModifierJSON = [this.itemModifierJSON];
        }
        for (const modifier of modifiers) {
            let modifierJSON;
            if (modifier instanceof ItemModifierClass) {
                modifierJSON = modifier.itemModifierJSON;
            }
            else {
                modifierJSON = modifier;
            }
            if (Array.isArray(modifierJSON)) {
                this.itemModifierJSON.push(...modifierJSON);
            }
            else {
                this.itemModifierJSON.push(modifierJSON);
            }
        }
    }
    unshift(...modifiers) {
        if (!Array.isArray(this.itemModifierJSON)) {
            this.itemModifierJSON = [this.itemModifierJSON];
        }
        for (const modifier of modifiers) {
            let modifierJSON;
            if (modifier instanceof ItemModifierClass) {
                modifierJSON = modifier.itemModifierJSON;
            }
            else {
                modifierJSON = modifier;
            }
            if (Array.isArray(modifierJSON)) {
                this.itemModifierJSON.unshift(...modifierJSON);
            }
            else {
                this.itemModifierJSON.unshift(modifierJSON);
            }
        }
    }
    get modify() {
        return {
            /**
             * @param pos The position of the container containing the slot to apply the modifier to.
             * @param slot The slot to apply the modifier to.
             */
            block: (pos, slot) => {
                this.pack.commands.item.modify.block(pos, slot, this);
            },
            /**
             * @param targets The entity/entities containing the slot to apply the modifier to.
             * @param slot The slot to apply the modifier to.
             * @param modifier The name of the modifier.
             */
            entity: (targets, slot) => {
                this.pack.commands.item.modify.entity(targetParser(targets), slot, this);
            },
        };
    }
}
