import { ContainerNode } from '../../nodes.js';
import { ResourceClass } from '../resource.js';
import type { Coordinates, ENTITY_SLOTS, LootTableJSON, MultipleEntitiesArgument, MultiplePlayersArgument } from 'sandstone/arguments';
import type { LiteralUnion } from 'sandstone/utils';
import type { SandstoneCore } from '../../sandstoneCore.js';
import type { ResourceClassArguments, ResourceNode } from '../resource.js';
/**
 * A node representing a Minecraft loot table.
 */
export declare class LootTableNode extends ContainerNode implements ResourceNode<LootTableClass> {
    resource: LootTableClass;
    constructor(sandstoneCore: SandstoneCore, resource: LootTableClass);
    getValue: () => string;
}
export type LootTableClassArguments = {
    /**
     * The loot table's JSON.
     */
    lootTable?: LootTableJSON;
} & ResourceClassArguments<'default'>;
export declare class LootTableClass extends ResourceClass<LootTableNode> {
    lootTableJSON: NonNullable<LootTableClassArguments['lootTable']>;
    constructor(sandstoneCore: SandstoneCore, name: string, args: LootTableClassArguments);
    /**
     * Gives items, ignoring empty item stacks.
     * @param players Optional. Specifies the player(s). Defaults to `@s`.
     */
    give: (players?: MultiplePlayersArgument<false>) => import("../../../commands/helpers.js").FinalCommandOutput;
    /**
     * Distributes items to container.
     * @param targetPos Optional. Specifies the position of the block. Defaults to `~ ~ ~`.
     */
    insert: (targetPos?: Coordinates<false>) => import("../../../commands/helpers.js").FinalCommandOutput;
    /**
     * Replace a container block slot with the items.
     * @param targetPos Specifies the position of a block.
     *
     * @param slot Specifies the inventory slot to be modified.
     * Must be must be `container.<slot_number>` where `<slot_number>` is replaced with a number specifying the slot.
     *
     * - Chests, dispensers, droppers, hoppers, and trapped chests are numbered 0 for the top-left slot and then increase
     *   first horizontally, then vertically (so, for example, a chest's top row slots are numbered 0 to 8 from left to right).
     *   Double chests and double trapped chests are treated as two single container blocks.
     * - A brewing stand's bottom slots are numbered 0 to 2 from left to right, its top slot is 3 and the fuel slot is 4.
     * - A furnace's slots are numbered 0 for the input slot, 1 for the fuel slot, and 2 for the output slot.
     *
     * @param count Specifies the number of consecutive slots to be filled. Must be between 0 and 2147483647 (inclusive).
     */
    replaceBlock: (targetPos: Coordinates<false>, slot: string, count?: number) => import("../../../commands/helpers.js").FinalCommandOutput;
    /**
     * Replace an entity slot with the items.
     *
     * @param targetPos Specifies the position of a block.
     *
     * @param slot Specifies the inventory slot to be modified.
     * Must be must be `container.<slot_number>` where `<slot_number>` is replaced with a number specifying the slot.
     *
     * - Chests, dispensers, droppers, hoppers, and trapped chests are numbered 0 for the top-left slot and then increase
     *   first horizontally, then vertically (so, for example, a chest's top row slots are numbered 0 to 8 from left to right).
     *   Double chests and double trapped chests are treated as two single container blocks.
     * - A brewing stand's bottom slots are numbered 0 to 2 from left to right, its top slot is 3 and the fuel slot is 4.
     * - A furnace's slots are numbered 0 for the input slot, 1 for the fuel slot, and 2 for the output slot.
     *
     * @param count Specifies the number of consecutive slots to be filled. Must be between 0 and 2147483647 (inclusive).
     */
    replaceEntity: (entities: MultipleEntitiesArgument<false>, slot: LiteralUnion<ENTITY_SLOTS>, count?: number) => import("../../../commands/helpers.js").FinalCommandOutput;
    toString: () => string;
    toJSON: () => string;
}
