import { ContainerNode } from '../../nodes.js';
import { ResourceClass } from '../resource.js';
import type { PredicateJSON } from 'sandstone/arguments';
import type { ConditionClass } from 'sandstone/variables';
import type { SandstoneCore } from '../../sandstoneCore.js';
import type { ListResource, ResourceClassArguments, ResourceNode } from '../resource.js';
/**
 * A node representing a Minecraft predicate.
 */
export declare class PredicateNode extends ContainerNode implements ResourceNode<PredicateClass> {
    resource: PredicateClass;
    constructor(sandstoneCore: SandstoneCore, resource: PredicateClass);
    getValue: () => string;
}
export type PredicateClassArguments = {
    /**
     * The predicate's JSON.
     */
    predicate?: PredicateJSON;
} & ResourceClassArguments<'list'>;
type Predicate = PredicateJSON | PredicateClass;
export declare class PredicateClass extends ResourceClass<PredicateNode> implements ListResource, ConditionClass {
    predicateJSON: NonNullable<PredicateClassArguments['predicate']>;
    constructor(sandstoneCore: SandstoneCore, name: string, args: PredicateClassArguments);
    push(...predicates: Predicate[]): void;
    unshift(...predicates: Predicate[]): void;
    /** @internal */
    _toMinecraftCondition: () => import("../../../flow/conditions/resources/predicate.js").PredicateConditionNode;
    /** @internal */
    toJSON(): NonNullable<PredicateJSON | undefined>;
}
export {};
