import { ContainerNode } from '../../nodes.js';
import { ResourceClass } from '../resource.js';
/**
 * A node representing a Minecraft predicate.
 */
export class PredicateNode extends ContainerNode {
    resource;
    constructor(sandstoneCore, resource) {
        super(sandstoneCore);
        this.resource = resource;
    }
    getValue = () => JSON.stringify(this.resource.predicateJSON);
}
export class PredicateClass extends ResourceClass {
    predicateJSON;
    constructor(sandstoneCore, name, args) {
        super(sandstoneCore, { packType: sandstoneCore.pack.dataPack(), extension: 'json' }, PredicateNode, sandstoneCore.pack.resourceToPath(name, ['predicates']), args);
        this.predicateJSON = args.predicate;
        this.handleConflicts();
    }
    push(...predicates) {
        if (!Array.isArray(this.predicateJSON)) {
            this.predicateJSON = [this.predicateJSON];
        }
        for (const predicate of predicates) {
            let predicateJSON;
            if (predicate instanceof PredicateClass) {
                predicateJSON = predicate.predicateJSON;
            }
            else {
                predicateJSON = predicate;
            }
            if (Array.isArray(predicateJSON)) {
                this.predicateJSON.push(...predicateJSON);
            }
            else {
                this.predicateJSON.push(predicateJSON);
            }
        }
    }
    unshift(...predicates) {
        if (!Array.isArray(this.predicateJSON)) {
            this.predicateJSON = [this.predicateJSON];
        }
        for (const predicate of predicates) {
            let predicateJSON;
            if (predicate instanceof PredicateClass) {
                predicateJSON = predicate.predicateJSON;
            }
            else {
                predicateJSON = predicate;
            }
            if (Array.isArray(predicateJSON)) {
                this.predicateJSON.unshift(...predicateJSON);
            }
            else {
                this.predicateJSON.unshift(predicateJSON);
            }
        }
    }
    /** @internal */
    _toMinecraftCondition = () => new this.pack.conditions.Predicate(this.core, this.name);
    /** @internal */
    toJSON() {
        return this.predicateJSON;
    }
}
