import { ContainerNode } from '../../nodes.js';
import { ResourceClass } from '../resource.js';
import type { MultiplePlayersArgument, RecipeJSON } from 'sandstone/arguments';
import type { SandstoneCore } from '../../sandstoneCore.js';
import type { ResourceClassArguments, ResourceNode } from '../resource.js';
/**
 * A node representing a Minecraft recipe.
 */
export declare class RecipeNode extends ContainerNode implements ResourceNode<RecipeClass> {
    resource: RecipeClass;
    constructor(sandstoneCore: SandstoneCore, resource: RecipeClass);
    getValue: () => string;
}
export type RecipeClassArguments = {
    /**
     * The recipe's JSON.
     */
    recipe: RecipeJSON;
} & ResourceClassArguments<'default'>;
export declare class RecipeClass extends ResourceClass<RecipeNode> {
    recipeJSON: NonNullable<RecipeClassArguments['recipe']>;
    constructor(sandstoneCore: SandstoneCore, name: string, args: RecipeClassArguments);
    /**
     * Give this recipe.
     * @param players Optional. Specifies the player(s). Defaults to `@s`.
     */
    give: (players?: MultiplePlayersArgument<false>) => void;
    /**
     * Take this recipe.
     * @param players Optional. Specifies the player(s). Defaults to `@s`.
     */
    take: (players?: MultiplePlayersArgument<false>) => void;
}
