"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
var structure_exports = {};
__export(structure_exports, {
  StructureClass: () => StructureClass,
  StructureNode: () => StructureNode
});
module.exports = __toCommonJS(structure_exports);
var import_lodash = __toESM(require("lodash"), 1);
var import_prismarine_nbt = __toESM(require("prismarine-nbt"), 1);
var import_utils = require("sandstone/utils");
var import_variables = require("sandstone/variables");
var import_nodes = require("../../nodes.js");
var import_resource = require("../resource.js");
const same = import_lodash.default.isEqual;
class StructureNode extends import_nodes.ContainerNode {
  constructor(sandstoneCore, resource) {
    super(sandstoneCore);
    this.resource = resource;
    __publicField(this, "getValue", () => this.resource.structureNBT ? encodeStructure(this.resource.structure()) : this.resource.structureBuffer);
  }
}
class StructureClass extends import_resource.ResourceClass {
  constructor(sandstoneCore, name, args) {
    super(sandstoneCore, { packType: sandstoneCore.pack.dataPack(), extension: "nbt", encoding: false }, StructureNode, sandstoneCore.pack.resourceToPath(name, ["structures"]), args);
    __publicField(this, "structureBuffer");
    __publicField(this, "structureNBT");
    if (args.structure === void 0) {
      this.structureBuffer = sandstoneCore.getExistingResource(this, false);
    } else if (typeof args.structure === "string") {
      this.structureBuffer = sandstoneCore.getExistingResource(args.structure, false);
    } else if (args.structure instanceof StructureClass) {
      if (args.structure.structureNBT) {
        this.structureNBT = args.structure.structureNBT;
      } else if (args.structure.structureBuffer) {
        this.structureBuffer = args.structure.structureBuffer;
      } else {
        this.structureBuffer = args.structure.structure();
      }
    } else if (Array.isArray(args.structure)) {
      this.structureNBT = this.arrayToNBT(args.structure);
    } else {
      this.structureNBT = args.structure;
    }
    this.handleConflicts();
  }
  structure() {
    if (this.structureNBT) {
      return this.structureNBT;
    }
    if (this.structureBuffer) {
      return this.structureBuffer;
    }
    this.structureBuffer = this.node.sandstoneCore.getExistingResource(this.name, false);
    return this.structureBuffer;
  }
  async readBuffer() {
    this.structureNBT = await decodeStructure(await this.structureBuffer);
  }
  async array() {
    if (!this.structureNBT) {
      await this.readBuffer();
    }
    const _NBT = this.structureNBT;
    const array = [[[]]];
    for (let x = 0; x < _NBT.size[0]; x++) {
      array.push([]);
      for (let y = _NBT.size[1]; y--; y > 0) {
        array[x].push([]);
        for (let z = _NBT.size[2]; z--; z > 0) {
          array[x][y].push({});
        }
      }
    }
    for (const block of _NBT.blocks) {
      const convert = (_block) => ({ id: _block.Name, state: _block.Properties, ...(0, import_utils.add)({ nbt: block.nbt }) });
      array[block.pos[0]][block.pos[1]][block.pos[2]] = {
        /* @ts-ignore */
        block: import_prismarine_nbt.NBT.palettes ? import_prismarine_nbt.NBT.palettes.map((palette) => convert(palette[block.state])) : convert(import_prismarine_nbt.NBT.palette[block.state])
      };
    }
    for (const entity of _NBT.entities) {
      array[entity.blockPos[0]][entity.blockPos[1]][entity.blockPos[2]] += {
        entity: [{
          id: entity.nbt.id,
          nbt: (() => {
            const nbt = { ...entity.nbt };
            delete nbt.id;
            return nbt;
          })(),
          offset: (() => {
            const diff = (num1, num2) => Math.sign(num1) === -1 ? (Math.abs(num1) - Math.abs(num2)) * -1 : num1 - num2;
            return [
              diff(entity.pos[0], entity.blockPos[0]),
              diff(entity.pos[1], entity.blockPos[1]),
              diff(entity.pos[2], entity.blockPos[2])
            ];
          })()
        }]
      };
    }
    return array;
  }
  arrayToNBT(array) {
    const nbt = {
      DataVersion: 3329,
      size: [0, 0, 0],
      blocks: [],
      entities: []
    };
    nbt.size[0] = array.length;
    for (const layer of array.entries()) {
      if (layer.length > nbt.size[1]) {
        nbt.size[1] = layer.length;
      }
      for (const row of layer[1].entries()) {
        if (row.length > nbt.size[2]) {
          nbt.size[2] = row.length;
        }
        for (const entry of row[1].entries()) {
          if (entry[1].block) {
            if (Array.isArray(entry[1].block)) {
              let paletteIndex = -1;
              if (!nbt.palettes) {
                nbt.palettes = [];
                for (let i = 0; i < entry[1].block.length; i++) {
                  nbt.palettes.push([]);
                }
              } else {
                for (const block of nbt.palettes[0]) {
                  if (same(entry[1].block[0], block[1])) {
                    paletteIndex = block[0];
                  }
                }
                if (paletteIndex === -1) {
                  for (const palette of nbt.pallettes.entries()) {
                    palette[1].push({
                      Name: entry[1].block[palette[0]].id,
                      ...(0, import_utils.add)({
                        Properties: entry[1].block[palette[0]].state
                      })
                    });
                  }
                }
              }
              nbt.blocks.push({
                state: paletteIndex === -1 ? nbt.palettes.length - 1 : paletteIndex,
                pos: [layer[0], row[0], entry[0]],
                ...(0, import_utils.add)({
                  nbt: entry[1].block[0].nbt
                })
              });
            } else {
              let paletteIndex = -1;
              if (!nbt.palette) {
                nbt.palette = [];
              }
              for (const block of nbt.palette) {
                if (same(entry[1].block, block[1])) {
                  paletteIndex = block[0];
                }
              }
              if (paletteIndex === -1) {
                nbt.palette.push({
                  Name: entry[1].block.id,
                  ...(0, import_utils.add)({
                    Properties: entry[1].block.state
                  })
                });
              }
              nbt.blocks.push({
                state: paletteIndex === -1 ? nbt.palettes.length - 1 : paletteIndex,
                pos: [layer[0], row[0], entry[0]],
                ...(0, import_utils.add)({
                  nbt: entry[1].block.nbt
                })
              });
            }
          }
          if (entry[1].entities) {
            for (const entity of entry[1].entities) {
              nbt.push({
                pos: entity.offset ? [layer[0] + entity.offset[0], row[0] + entity.offset[1], entry[0] + entity.offset[2]] : [layer[0], row[0], entry[0]],
                blockPos: [layer[0], row[0], entry[0]],
                nbt: {
                  id: entity.id,
                  ...(0, import_utils.add)(entity.nbt)
                }
              });
            }
          }
        }
      }
    }
    return nbt;
  }
  /**
   * Places the structure.
   *
   * Usage of `VariableInsertion` or `xz` will result in a structure block being used.
   *
   * @param pos Optional. Where to place relative to the current location.
   * @param rotation Optional. Rotation to apply to the structure.
   * @param mirror Optional. Mirroring to apply to the structure. `xz` will clear all existing blocks in the region, only works for structures with defined size, & will await structures not in memory.
   * @param integrity Optional. How complete the structure will be. Must be a float between 1.0 & 0.0 (inclusive), if below 1 the structure will be randomly degraded.
   * @param seed Optional. Integer applied to the integrity random. Defaults to 0.
   */
  async load(pos = [0, 0, 0], rotation, mirror, integrity, seed) {
    const {
      Variable,
      Data,
      ResolveNBT
    } = this.pack;
    const {
      place,
      fill,
      setblock,
      execute
    } = this.pack.commands;
    const structure = place.template;
    const rel = (0, import_variables.relative)(...pos);
    if (rotation instanceof import_variables.ConditionClass || mirror instanceof import_variables.ConditionClass || integrity instanceof import_variables.ConditionClass || seed instanceof import_variables.ConditionClass) {
      const currentLevel = Variable();
      const bottomCoordinate = Variable();
      currentLevel.set(Data("entity", "@s", "Position[1]"));
      execute.summon("marker").run(() => {
        Data("entity", "@s", "Position[1]").set(bottomCoordinate, "double");
        const nbt = {
          mode: "LOAD",
          showboundingbox: false,
          powered: false,
          ignoreEntities: false,
          name: structure.name,
          posX: pos[0],
          posY: (0, import_variables.ResolveNBTPart)(currentLevel["+"](pos[1])),
          posZ: pos[2]
        };
        setblock(`~${pos[0]} ~ ~${pos[2]}`, "structure_block");
        Data("block", `~${pos[0]} ~ ~${pos[2]}`, "{}").set(ResolveNBT(nbt));
        setblock(`~${pos[0]} ~ ~${pos[2] + 1}`, "redstone_block");
        setblock(`~${pos[0]} ~ ~${pos[2]}`, "bedrock");
        setblock(`~${pos[0]} ~ ~${pos[2] + 1}`, "bedrock");
      });
    } else if (mirror && (mirror === "^xz" || mirror === "xz")) {
      if (!this.structureNBT) {
        await this.readBuffer();
      }
      const temp = new StructureClass(this.core, "__sandstone:temp", { addToSandstoneCore: false, creator: "sandstone" });
      const coords = this.structureNBT.size;
      const outerCorner = (0, import_variables.relative)(...coords);
      fill(rel, outerCorner, "air");
      structure(this, rel, rotation, "^x");
      temp.save(rel, coords);
      fill(rel, outerCorner, "air");
      structure(temp, rel, "none", "^z", integrity, seed);
    } else {
      structure(this, rel, rotation, mirror, integrity, seed);
    }
  }
  save(pos = "~ ~ ~", size) {
  }
  toString() {
    return this.name;
  }
}
const objectMap = (obj, fn) => Object.fromEntries(Object.entries(obj).map(([k, v], i) => fn(v, k, i)));
function encodeStructure(nbt) {
  const {
    comp,
    list,
    int,
    double,
    string,
    writeUncompressed
  } = import_prismarine_nbt.default;
  return writeUncompressed(comp({
    DataVersion: int(nbt.DataVersion),
    size: list(comp(nbt.size.map((axis) => int(axis)))),
    blocks: list(comp(nbt.blocks.map((block) => ({
      state: int(block.state),
      pos: list(comp(block.pos.map((axis) => int(axis))))
      /**
       *  ...add({
       * nbt: fun(block.nbt), // TODO
       *}),
       */
    })))),
    entities: list(comp(nbt.entities.map((entity) => ({
      pos: list(comp(entity.pos.map((axis) => double(axis)))),
      blockPos: list(comp(entity.blockPos.map((axis) => int(axis))))
      // nbt: fun(entity.nbt), // TODO
    })))),
    // eslint-disable-next-line no-nested-ternary
    ...nbt.palette ? {
      palette: list(comp(nbt.palette.map((block) => ({
        Name: string(block.Name),
        Properties: list(comp(objectMap(block.Properties, (v, k) => [string(k), string(v)])))
      }))))
    } : nbt.palettes ? {
      palettes: list(comp(nbt.palettes.map((palette) => palette.map((block) => ({
        Name: string(block.Name),
        Properties: list(comp(objectMap(block.Properties, (v, k) => [string(k), string(v)])))
      })))))
    } : {}
  }));
}
async function decodeStructure(buffer) {
  const nbt = await import_prismarine_nbt.default.parse(buffer);
  return import_prismarine_nbt.default.simplify(nbt.parsed);
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  StructureClass,
  StructureNode
});
//# sourceMappingURL=structure.cjs.map