import { ContainerNode } from '../../nodes.js';
import { ResourceClass } from '../resource.js';
import type { HintedTagStringType, REGISTRIES, TagSingleValue, TagValuesJSON } from 'sandstone/arguments';
import type { LiteralUnion } from 'sandstone/utils';
import type { ConditionClass } from 'sandstone/variables';
import type { SandstoneCore } from '../../sandstoneCore.js';
import type { ListResource, ResourceClassArguments, ResourceNode } from '../resource.js';
import type { MCFunctionClass } from './mcfunction.js';
type TagJSON<REGISTRY extends LiteralUnion<REGISTRIES>> = {
    replace: boolean;
    values: TagValuesJSON<REGISTRY>;
};
/**
 * A node representing a Minecraft tag.
 */
export declare class TagNode extends ContainerNode implements ResourceNode<TagClass<LiteralUnion<REGISTRIES>>> {
    resource: TagClass<LiteralUnion<REGISTRIES>>;
    constructor(sandstoneCore: SandstoneCore, resource: TagClass<LiteralUnion<REGISTRIES>>);
    getValue: () => string;
}
export type TagClassArguments<REGISTRY extends LiteralUnion<REGISTRIES>> = {
    /**
     * The tag's entry list.
     */
    values?: TagValuesJSON<REGISTRY>;
    /**
     * Whether to replace existing Tags with the same name.
     */
    replace?: boolean;
} & ResourceClassArguments<'list'> & (REGISTRY extends 'functions' ? ({
    /**
     * Whether the tag should run on load
     */
    runOnLoad?: boolean;
    /**
     * Whether the function should run each tick.
     */
    runEveryTick?: boolean;
}) : unknown);
type Resource<T extends LiteralUnion<REGISTRIES>> = TagSingleValue<HintedTagStringType<T>> | TagClass<T>;
export declare class TagClass<REGISTRY extends LiteralUnion<REGISTRIES>> extends ResourceClass implements ListResource, ConditionClass {
    readonly type: REGISTRY;
    readonly tagJSON: NonNullable<TagJSON<REGISTRY>>;
    constructor(sandstoneCore: SandstoneCore, type: REGISTRY, name: string, args: TagClassArguments<REGISTRY>);
    get name(): `#${string}`;
    push(...resources: Resource<REGISTRY>[]): void;
    unshift(..._resources: Resource<REGISTRY>[]): void;
    /** Checks whether a given resource is in this tag. */
    has(resource: string | MCFunctionClass<undefined, undefined> | TagClass<REGISTRY>): void;
    toString(): `#${string}`;
    toJSON(): string;
    /**
     * @internal
     */
    _toMinecraftCondition: () => import("../../../flow/conditions/resources/tag.js").TagConditionNode;
}
export {};
