import { ContainerNode } from '../../nodes.js';
import { ResourceClass } from '../resource.js';
import type { TrimMaterialJSON } from 'sandstone/arguments';
import type { ConditionClass } from 'sandstone/variables';
import type { SandstoneCore } from '../../sandstoneCore.js';
import type { ResourceClassArguments, ResourceNode } from '../resource.js';
export type EquipmentSlots = 'mainhand' | 'offhand' | 'head' | 'chest' | 'legs' | 'feet';
/**
 * A node representing a Minecraft trim material.
 */
export declare class TrimMaterialNode extends ContainerNode implements ResourceNode<TrimMaterialClass> {
    resource: TrimMaterialClass;
    constructor(sandstoneCore: SandstoneCore, resource: TrimMaterialClass);
    getValue: () => string;
}
export type TrimMaterialClassArguments = {
    /**
     * The trim material's JSON.
     */
    trimMaterial?: TrimMaterialJSON;
} & ResourceClassArguments<'default'> & {
    /**
     * Optional. Defaults to true. Automatically adds trim material to #minecraft:trim_materials.
     */
    registerMaterial?: boolean;
    /**
     * Defaults to all equipment slots. Equipment slots to check in predicate condition, `whole_inventory` will use an `if data` check.
     */
    equipmentCheck?: 'whole_inventory' | EquipmentSlots | EquipmentSlots[];
};
export declare class TrimMaterialClass extends ResourceClass<TrimMaterialNode> implements ConditionClass {
    trimMaterialJSON: NonNullable<TrimMaterialClassArguments['trimMaterial']>;
    readonly equipmentCheck: "whole_inventory" | EquipmentSlots | EquipmentSlots[] | undefined;
    constructor(sandstoneCore: SandstoneCore, name: string, args: TrimMaterialClassArguments);
    /** String which will be used in the resource pack. */
    get assetName(): string;
    /** Model override predicate float between 1 & 0. */
    get overrideIndex(): number;
    /** The item used in the smithing table for this material. */
    get material(): import("../../../utils.js").LiteralUnion<import("sandstone/arguments").ITEMS>;
    /**
     * @internal
     */
    _toMinecraftCondition: () => import("../../../flow/conditions/resources/trimMaterial.js").TrimMaterialConditionNode;
}
