import { ContainerNode } from '../../nodes.js';
import { ResourceClass } from '../resource.js';
let trimMaterials;
/**
 * A node representing a Minecraft trim material.
 */
export class TrimMaterialNode extends ContainerNode {
    resource;
    constructor(sandstoneCore, resource) {
        super(sandstoneCore);
        this.resource = resource;
    }
    getValue = () => JSON.stringify(this.resource.trimMaterialJSON);
}
export class TrimMaterialClass extends ResourceClass {
    trimMaterialJSON;
    equipmentCheck;
    constructor(sandstoneCore, name, args) {
        super(sandstoneCore, { packType: sandstoneCore.pack.dataPack(), extension: 'json' }, TrimMaterialNode, sandstoneCore.pack.resourceToPath(name, ['trim_materials']), args);
        this.trimMaterialJSON = args.trimMaterial;
        this.equipmentCheck = args.equipmentCheck;
        if (args.registerMaterial !== false) {
            if (!trimMaterials)
                trimMaterials = sandstoneCore.pack.Tag('items', 'minecraft:trim_materials', [this.material]);
            else
                trimMaterials.push(this.material);
        }
        this.handleConflicts();
    }
    /** String which will be used in the resource pack. */
    get assetName() {
        return this.trimMaterialJSON.asset_name;
    }
    /** Model override predicate float between 1 & 0. */
    get overrideIndex() {
        return this.trimMaterialJSON.item_model_index;
    }
    /** The item used in the smithing table for this material. */
    get material() {
        return this.trimMaterialJSON.ingredient;
    }
    /**
     * @internal
     */
    _toMinecraftCondition = () => new this.pack.conditions.TrimMaterial(this.core, this);
}
