import { ContainerNode } from '../../nodes.js';
import { ResourceClass } from '../resource.js';
import type { TrimPatternJSON } from 'sandstone/arguments';
import type { ConditionClass } from 'sandstone/variables';
import type { SandstoneCore } from '../../sandstoneCore.js';
import type { ResourceClassArguments, ResourceNode } from '../resource.js';
import type { EquipmentSlots } from './trimMaterial.js';
/**
 * A node representing a Minecraft trim pattern.
 */
export declare class TrimPatternNode extends ContainerNode implements ResourceNode<TrimPatternClass> {
    resource: TrimPatternClass;
    constructor(sandstoneCore: SandstoneCore, resource: TrimPatternClass);
    getValue: () => string;
}
export type TrimPatternClassArguments = {
    /**
     * The trim pattern's JSON.
     */
    trimPattern?: TrimPatternJSON;
} & ResourceClassArguments<'default'> & {
    /**
     * Optional. Defaults to true. Automatically adds trim pattern to #minecraft:trim_templates.
     */
    registerPatternTag?: boolean;
    /**
     * Optional. Defaults to true. Automatically adds armor trim pattern recipe.
     */
    registerPatternRecipe?: boolean;
    /**
     * Defaults to all equipment slots. Equipment slots to check in predicate condition, `whole_inventory` will use an `if data` check.
     */
    equipmentCheck?: 'whole_inventory' | EquipmentSlots | EquipmentSlots[];
};
export declare class TrimPatternClass extends ResourceClass<TrimPatternNode> implements ConditionClass {
    trimPatternJSON: NonNullable<TrimPatternClassArguments['trimPattern']>;
    readonly equipmentCheck: "whole_inventory" | EquipmentSlots | EquipmentSlots[] | undefined;
    constructor(sandstoneCore: SandstoneCore, name: string, args: TrimPatternClassArguments);
    /** The item used in the smithing table for this pattern. */
    get template(): import("../../../utils.js").LiteralUnion<import("sandstone/arguments").ITEMS>;
    /**
     * @internal
     */
    _toMinecraftCondition: () => import("../../../flow/conditions/resources/trimPattern.js").TrimPatternConditionNode;
}
