import { ContainerNode } from '../../nodes.js';
import { ResourceClass } from '../resource.js';
/**
 * A node representing a Minecraft trim pattern.
 */
export class TrimPatternNode extends ContainerNode {
    resource;
    constructor(sandstoneCore, resource) {
        super(sandstoneCore);
        this.resource = resource;
    }
    getValue = () => JSON.stringify(this.resource.trimPatternJSON);
}
export class TrimPatternClass extends ResourceClass {
    trimPatternJSON;
    equipmentCheck;
    constructor(sandstoneCore, name, args) {
        super(sandstoneCore, { packType: sandstoneCore.pack.dataPack(), extension: 'json' }, TrimPatternNode, sandstoneCore.pack.resourceToPath(name, ['trim_patterns']), args);
        this.trimPatternJSON = args.trimPattern;
        this.equipmentCheck = args.equipmentCheck;
        if (args.registerPatternTag !== false) {
            sandstoneCore.pack.Tag('items', 'minecraft:trim_templates', [this.template], { onConflict: 'append' });
        }
        if (args.registerPatternRecipe !== false) {
            let assetID = this.trimPatternJSON.asset_id;
            if (assetID.includes(':')) {
                assetID = assetID.split(':')[1];
            }
            sandstoneCore.pack.Recipe(assetID, {
                type: 'smithing_trim',
                base: { tag: 'minecraft:trimmable_armor' },
                addition: { tag: 'minecraft:trim_materials' },
                template: { item: this.trimPatternJSON.template_item },
            });
        }
        this.handleConflicts();
    }
    /** The item used in the smithing table for this pattern. */
    get template() {
        return this.trimPatternJSON.template_item;
    }
    /**
     * @internal
     */
    _toMinecraftCondition = () => new this.pack.conditions.TrimPattern(this.core, this);
}
