"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
var resource_exports = {};
__export(resource_exports, {
  CallableResourceClass: () => CallableResourceClass,
  ListResource: () => ListResource,
  ResourceClass: () => ResourceClass
});
module.exports = __toCommonJS(resource_exports);
class ResourceClass {
  // eslint-disable-next-line max-len
  constructor(core, file, NodeType, path, args) {
    this.core = core;
    __publicField(this, "node");
    __publicField(this, "packType");
    __publicField(this, "fileExtension");
    __publicField(this, "fileEncoding");
    __publicField(this, "creator");
    __publicField(this, "commands");
    __publicField(this, "pack");
    __publicField(this, "path");
    __publicField(this, "addToSandstoneCore");
    __publicField(this, "onConflict");
    __publicField(this, "renameIndex", 2);
    __publicField(this, "getNode", () => this.node);
    __publicField(this, "generate", () => {
    });
    var _a, _b;
    this.node = new NodeType(core, this);
    this.packType = args.packType || file.packType;
    this.fileExtension = file.extension || "json";
    this.fileEncoding = (_a = file.encoding) != null ? _a : "utf8";
    this.pack = core.pack;
    this.commands = core.pack.commands;
    this.path = path;
    this.addToSandstoneCore = args.addToSandstoneCore;
    this.creator = (_b = args.creator) != null ? _b : "sandstone";
    const scopedStrategy = this.node.resource.path[1] ? process.env[`${this.node.resource.path[1].toUpperCase()}_CONFLICT_STRATEGY`] : false;
    this.onConflict = args.onConflict || scopedStrategy || process.env.DEFAULT_CONFLICT_STRATEGY || "throw";
  }
  handleConflicts() {
    const resourceType = this.node.resource.path[1] || "resources";
    const conflict = [...this.core.resourceNodes].find((node) => {
      if (node.resource.packType.constructor.name !== this.node.resource.packType.constructor.name) {
        return false;
      }
      return node.resource.packType.constructor === this.node.resource.packType.constructor && node.resource.path.join("") === this.node.resource.path.join("");
    });
    let add = false;
    if (conflict) {
      const oldResource = conflict.resource;
      const newResource = this.node.resource;
      switch (this.onConflict) {
        case "throw": {
          throw new Error(`Created a ${resourceType.substring(0, resourceType.length - 1)} with the duplicate name ${newResource.name}, and onConflict was set to "throw".`);
        }
        case "replace":
          {
            this.core.resourceNodes.forEach((node) => {
              if (node.resource.path.join("") === oldResource.path.join("")) {
                this.core.resourceNodes.delete(node);
              }
            });
            add = true;
          }
          break;
        case "warn":
          {
            console.warn([
              "Warning:",
              `Tried to create a ${resourceType.substring(0, resourceType.length - 1)} named "${newResource.name}", but found an already existing one.`,
              "The new one has replaced the old one. To remove this warning, please change the options of the resource to { onConflict: '/* other option */' }."
            ].join("\n"));
            this.core.resourceNodes.forEach((node) => {
              if (node.resource.path.join("") === oldResource.path.join("")) {
                this.core.resourceNodes.delete(node);
              }
            });
            add = true;
          }
          break;
        case "rename":
          {
            this.path[this.path.length - 1] += `${oldResource.renameIndex++}`;
            add = true;
          }
          break;
        case "prepend":
          {
            oldResource.unshift(newResource);
          }
          break;
        case "append":
          {
            oldResource.push(newResource);
          }
          break;
        default:
          break;
      }
    } else {
      add = true;
    }
    if (this.addToSandstoneCore && add) {
      this.core.resourceNodes.add(this.node);
    }
  }
  get name() {
    return `${this.path[0]}:${this.path.slice(2).join("/")}`;
  }
  toString() {
    return this.name;
  }
}
class CallableResourceClass extends ResourceClass {
  constructor() {
    super(...arguments);
    // This is the resource, but as a callable
    __publicField(this, "_that");
  }
  get asCallable() {
    if (!this._that) {
      throw new Error("Class has not been made callable.");
    }
    return this._that;
  }
  makeCallable(mcFunctionClass) {
    this._that = mcFunctionClass;
    this.node.resource = mcFunctionClass;
  }
}
class ListResource {
  push(...args) {
  }
  unshift(...args) {
  }
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  CallableResourceClass,
  ListResource,
  ResourceClass
});
//# sourceMappingURL=resource.cjs.map