/// <reference types="node" resolution-mode="require"/>
import type fs from 'fs-extra';
import type { SandstoneCommands } from 'sandstone/commands/commands';
import type { Node, SandstoneCore } from 'sandstone/core';
import type { ResourcePath, SandstonePack } from 'sandstone/pack';
import type { PackType } from 'sandstone/pack/packType';
import type { BASIC_CONFLICT_STRATEGIES, LiteralUnion, MakeInstanceCallable } from 'sandstone/utils';
export type ResourceClassArguments<ConflictType extends 'default' | 'list' | 'function'> = {
    /**
     * Whether the associated Node should be added to Sandstone Core.
     */
    addToSandstoneCore: boolean;
    /**
     * Whether the resource has been created explicitely by a user, or automatically generated by Sandstone.
     */
    creator: 'user' | 'sandstone';
    /**
     * What to do if another resource of the same type has the same name.
     *
     * - `throw`: Throw an error.
     * - `replace`: Replace silently the old resource with the new one.
     * - `ignore`: Keep silently the old resource, discarding the new one.
     * - `append`: Append the new resource entries to the old one.
     * - `prepend`: Prepend the new resource entries to the old one.
     * - `rename`: Rename the new file to an iterating number (ie. func1, func2, func3)
     */
    onConflict?: ConflictType extends 'default' ? BASIC_CONFLICT_STRATEGIES : BASIC_CONFLICT_STRATEGIES | 'append' | 'prepend';
    /**
     * Overrides the default pack for the resource.
     */
    packType?: PackType;
};
export type ResourceNode<T = ResourceClass<any>> = Node & {
    resource: T;
};
export type ResourceNodeConstructor<N extends Node> = new (sandstoneCore: SandstoneCore, resource: any) => N;
export declare abstract class ResourceClass<N extends ResourceNode = ResourceNode<any>> {
    protected core: SandstoneCore;
    protected node: N;
    packType: PackType;
    fileExtension: string;
    fileEncoding: false | "ascii" | "utf8" | "utf-8" | "utf16le" | "ucs2" | "ucs-2" | "base64" | "base64url" | "latin1" | "binary" | "hex" | fs.ObjectEncodingOptions;
    protected creator: NonNullable<ResourceClassArguments<any>['creator']>;
    protected commands: SandstoneCommands<false>;
    protected pack: SandstonePack;
    path: ResourcePath;
    addToSandstoneCore: boolean;
    onConflict: LiteralUnion<BASIC_CONFLICT_STRATEGIES>;
    renameIndex: number;
    constructor(core: SandstoneCore, file: {
        packType: PackType;
        extension?: string;
        encoding?: fs.EncodingOption | false;
    }, NodeType: ResourceNodeConstructor<N>, path: ResourcePath, args: ResourceClassArguments<any>);
    protected handleConflicts(): void;
    protected getNode: () => N;
    get name(): string;
    protected generate: () => void;
    toString(): string;
}
export declare abstract class CallableResourceClass<N extends ResourceNode = ResourceNode<any>> extends ResourceClass<N> {
    private _that;
    protected get asCallable(): MakeInstanceCallable<this>;
    private makeCallable;
    abstract __call__: (...args: any) => any;
}
export declare abstract class ListResource {
    push(...args: any[]): void;
    unshift(...args: any[]): void;
}
