export class ResourceClass {
    core;
    node;
    packType;
    fileExtension;
    fileEncoding;
    creator;
    commands;
    pack;
    path;
    addToSandstoneCore;
    onConflict;
    renameIndex = 2;
    // eslint-disable-next-line max-len
    constructor(core, file, NodeType, path, args) {
        this.core = core;
        this.node = new NodeType(core, this);
        this.packType = args.packType || file.packType;
        this.fileExtension = file.extension || 'json';
        this.fileEncoding = file.encoding ?? 'utf8';
        this.pack = core.pack;
        this.commands = core.pack.commands;
        this.path = path;
        this.addToSandstoneCore = args.addToSandstoneCore;
        this.creator = args.creator ?? 'sandstone';
        const scopedStrategy = this.node.resource.path[1] ? process.env[`${this.node.resource.path[1].toUpperCase()}_CONFLICT_STRATEGY`] : false;
        this.onConflict = args.onConflict || scopedStrategy || process.env.DEFAULT_CONFLICT_STRATEGY || 'throw';
    }
    handleConflicts() {
        const resourceType = this.node.resource.path[1] || 'resources';
        const conflict = [...this.core.resourceNodes].find((node) => {
            if (node.resource.packType.constructor.name !== this.node.resource.packType.constructor.name) {
                return false;
            }
            return node.resource.packType.constructor === this.node.resource.packType.constructor && node.resource.path.join('') === this.node.resource.path.join('');
        });
        let add = false;
        if (conflict) {
            const oldResource = conflict.resource;
            const newResource = this.node.resource;
            switch (this.onConflict) {
                case 'throw': {
                    // eslint-disable-next-line max-len
                    throw new Error(`Created a ${resourceType.substring(0, resourceType.length - 1)} with the duplicate name ${newResource.name}, and onConflict was set to "throw".`);
                }
                case 'replace':
                    {
                        this.core.resourceNodes.forEach((node) => {
                            if (node.resource.path.join('') === oldResource.path.join('')) {
                                this.core.resourceNodes.delete(node);
                            }
                        });
                        add = true;
                    }
                    break;
                case 'warn':
                    {
                        console.warn([
                            'Warning:',
                            `Tried to create a ${resourceType.substring(0, resourceType.length - 1)} named "${newResource.name}", but found an already existing one.`,
                            "The new one has replaced the old one. To remove this warning, please change the options of the resource to { onConflict: '/* other option */' }.",
                        ].join('\n'));
                        this.core.resourceNodes.forEach((node) => {
                            if (node.resource.path.join('') === oldResource.path.join('')) {
                                this.core.resourceNodes.delete(node);
                            }
                        });
                        add = true;
                    }
                    break;
                case 'rename':
                    {
                        // eslint-disable-next-line no-plusplus
                        this.path[this.path.length - 1] += `${oldResource.renameIndex++}`;
                        add = true;
                    }
                    break;
                case 'prepend':
                    {
                        oldResource.unshift(newResource);
                    }
                    break;
                case 'append':
                    {
                        oldResource.push(newResource);
                    }
                    break;
                default: break;
            }
        }
        else {
            add = true;
        }
        if (this.addToSandstoneCore && add) {
            this.core.resourceNodes.add(this.node);
        }
    }
    getNode = () => this.node;
    get name() {
        return `${this.path[0]}:${this.path.slice(2).join('/')}`;
    }
    generate = () => { };
    toString() {
        return this.name;
    }
}
export class CallableResourceClass extends ResourceClass {
    // This is the resource, but as a callable
    _that;
    get asCallable() {
        if (!this._that) {
            throw new Error('Class has not been made callable.');
        }
        return this._that;
    }
    makeCallable(mcFunctionClass) {
        this._that = mcFunctionClass;
        this.node.resource = mcFunctionClass;
    }
}
export class ListResource {
    push(...args) { }
    unshift(...args) { }
}
