import { ContainerNode } from '../../nodes.js';
import { ResourceClass } from '../resource.js';
import type { SandstoneCore } from '../../sandstoneCore.js';
import type { ListResource, ResourceClassArguments, ResourceNode } from '../resource.js';
import type { BlockStateDefinition, BlockStateType } from 'sandstone/arguments';
/**
 * A node representing a Minecraft block state.
 */
export declare class BlockStateNode<Type extends BlockStateType> extends ContainerNode implements ResourceNode<BlockStateClass<Type>> {
    resource: BlockStateClass<Type>;
    constructor(sandstoneCore: SandstoneCore, resource: BlockStateClass<Type>);
    getValue: () => string;
}
export type BlockStateArguments<Type extends BlockStateType> = {
    /**
     * The block state's JSON.
     */
    blockState?: BlockStateDefinition<Type>;
} & ResourceClassArguments<'list'>;
export declare class BlockStateClass<Type extends BlockStateType> extends ResourceClass<BlockStateNode<Type>> implements ListResource {
    type: Type;
    blockStateJSON: NonNullable<BlockStateArguments<Type>['blockState']>;
    constructor(core: SandstoneCore, name: string, type: Type, args: BlockStateArguments<Type>);
    push(...states: BlockStateClass<Type>[] | BlockStateDefinition<Type>[]): void;
    unshift(...states: BlockStateClass<Type>[] | BlockStateDefinition<Type>[]): void;
    load(): Promise<void>;
}
