import { ContainerNode } from '../../nodes.js';
import { ResourceClass } from '../resource.js';
/**
 * A node representing a Minecraft block state.
 */
export class BlockStateNode extends ContainerNode {
    resource;
    constructor(sandstoneCore, resource) {
        super(sandstoneCore);
        this.resource = resource;
    }
    getValue = () => JSON.stringify(this.resource.blockStateJSON);
}
export class BlockStateClass extends ResourceClass {
    type;
    blockStateJSON;
    constructor(core, name, type, args) {
        super(core, { packType: core.pack.resourcePack() }, BlockStateNode, core.pack.resourceToPath(name, ['blockstates']), args);
        this.type = type;
        /** @ts-ignore */
        this.blockStateJSON = args.blockState || (type === 'variant' ? { variants: {} } : { multipart: [] });
        this.handleConflicts();
    }
    push(...states) {
        if (this.type === 'variant') {
            if (states[0] instanceof BlockStateClass) {
                for (const state of states) {
                    /** @ts-ignore */
                    this.blockStateJSON.variants = { ...this.blockStateJSON.variants, ...state.blockStateJSON.variants };
                }
            }
            else {
                for (const state of states) {
                    /** @ts-ignore */
                    this.blockStateJSON.variants = { ...this.blockStateJSON.variants, ...state.variants };
                }
            }
        }
        if (this.type === 'multipart') {
            if (states[0] instanceof BlockStateClass) {
                for (const state of states) {
                    /** @ts-ignore */
                    this.blockStateJSON.multipart.push(...state.blockStateJSON.multipart);
                }
            }
            else {
                for (const state of states) {
                    /** @ts-ignore */
                    this.blockStateJSON.multipart.push(...state.multipart);
                }
            }
        }
    }
    unshift(...states) {
        if (this.type === 'variant') {
            if (states[0] instanceof BlockStateClass) {
                for (const state of states) {
                    /** @ts-ignore */
                    this.blockStateJSON.variants = { ...state.blockStateJSON.variants, ...this.blockStateJSON.variants };
                }
            }
            else {
                for (const state of states) {
                    /** @ts-ignore */
                    this.blockStateJSON.variants = { ...state.variants, ...this.blockStateJSON.variants };
                }
            }
        }
        if (this.type === 'multipart') {
            if (states[0] instanceof BlockStateClass) {
                for (const state of states) {
                    /** @ts-ignore */
                    this.blockStateJSON.multipart.unshift(...state.blockStateJSON.multipart);
                }
            }
            else {
                for (const state of states) {
                    /** @ts-ignore */
                    this.blockStateJSON.multipart.unshift(...state.multipart);
                }
            }
        }
    }
    /*
     * blockWithState<B extends keyof Blockstates>(
     * block: LiteralUnion<B>,
     * state?: (B extends keyof Blockstates ? Blockstates[B] : Blockstates[keyof Blockstates]) | { [name: string]: string},
     * ): string {
     * let id: string = block
     * if (state) {
     *  id += `[${Object.entries(state).map(([name, val]) => `${name}=${val}`).join(',')}]`
     * }
     * return id
     * }
     */
    async load() {
    }
}
