"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
var model_exports = {};
__export(model_exports, {
  BlockFace: () => BlockFace,
  DisplayTransformClass: () => DisplayTransformClass,
  ElementClass: () => ElementClass,
  MAX_CUSTOM_MODEL_DATA: () => MAX_CUSTOM_MODEL_DATA,
  ModelClass: () => ModelClass,
  ModelNode: () => ModelNode
});
module.exports = __toCommonJS(model_exports);
var import_variables = require("sandstone/variables");
var import_nodes = require("../../nodes.js");
var import_resource = require("../resource.js");
const assert = (condition, errorMessage) => {
  if (!condition) {
    throw new Error(errorMessage);
  }
};
const MAX_CUSTOM_MODEL_DATA = 16777216;
const DisplayPositions = ["thirdpirson_righthand", "thirdpirson_lefthand", "firstpirson_righthand", "firstpirson_lefthand", "gui", "head", "ground", "fixed"];
const OverridePredicates = ["angle", "blocking", "broken", "cast", "cooldown", "damage", "damaged", "lefthanded", "pull", "pulling", "charged", "firework", "throwing", "time", "custom_model_data", "trim_material"];
class ModelNode extends import_nodes.ContainerNode {
  constructor(sandstoneCore, resource) {
    super(sandstoneCore);
    this.resource = resource;
    __publicField(this, "getValue", () => JSON.stringify(this.resource.toJSON()));
  }
}
class ModelClass extends import_resource.ResourceClass {
  constructor(core, type, name, args) {
    super(core, { packType: core.pack.resourcePack() }, ModelNode, core.pack.resourceToPath(name, ["models", type]), args);
    this.type = type;
    __publicField(this, "existingPath");
    __publicField(this, "parent", null);
    __publicField(this, "ambientocclusion", true);
    __publicField(this, "textures", {});
    __publicField(this, "elements", []);
    __publicField(this, "display", {});
    // TODO: Helper methods
    __publicField(this, "overrides", []);
    __publicField(this, "toString", () => `${this.path[0]}:${this.path.slice(2)}`);
    const data = args.model;
    if (data) {
      if (typeof data === "string") {
        this.existingPath = data;
      } else {
        this.fromJSON(data);
      }
    }
    this.handleConflicts();
  }
  /**
   * Generates the Minecraft model JSON
   */
  toJSON() {
    const obj = {};
    if (this.parent) {
      obj.parent = this.parent;
    }
    if (Object.keys(this.textures).length > 0) {
      obj.textures = this.textures;
    }
    if (!this.ambientocclusion) {
      obj.ambientocclusion = false;
    }
    if (this.elements && this.elements.length > 0) {
      obj.elements = this.elements.map((e) => e.toJSON());
    }
    if (Object.keys(this.display).length > 0) {
      obj.display = {};
      const entries = Object.entries(this.display);
      entries.forEach(([type, t]) => {
        obj.display[type] = t.toJSON();
      });
    }
    if (this.overrides && this.overrides.length > 0) {
      obj.overrides = this.overrides;
    }
    return obj;
  }
  generatedItem(...layers) {
    const textures = {};
    layers.forEach((texture, idx) => textures[`layer${idx}`] = texture);
    return {
      parent: "minecraft:item/generated"
    };
  }
  fromJSON(data) {
    if (data.parent) {
      this.parent = data.parent;
    }
    if (data.textures) {
      this.textures = data.textures;
    }
    if (data.display) {
      Object.values(DisplayPositions).filter((t) => t in data.display).forEach((t) => {
        this.display[t] = new DisplayTransformClass(data.display[t]);
      });
    }
    if (typeof data.ambientocclusion === "boolean") {
      this.ambientocclusion = data.ambientocclusion;
    }
    if (data.elements) {
      this.elements = data.elements.map((element) => new ElementClass(element));
    }
    if (data.overrides) {
      this.overrides = data.overrides;
    }
  }
  async load() {
    this.fromJSON(JSON.parse(await this.core.getExistingResource(this)));
    return this;
  }
}
class ElementClass {
  /**
   * Creates a new cube without any faces
   */
  constructor(data) {
    __publicField(this, "to");
    __publicField(this, "from");
    __publicField(this, "faces", {});
    __publicField(this, "rotation");
    __publicField(this, "shade", true);
    this.to = new Vector(...data.to);
    this.from = new Vector(...data.from);
    if (data.faces)
      this.faces = data.faces;
    if (data.rotation)
      this.rotation = data.rotation;
    if (data.shade)
      this.shade = data.shade;
  }
  hasFace(face) {
    if (this.hasFaces) {
      return face.name in this.faces;
    }
    return false;
  }
  get hasFaces() {
    return Object.keys(this.faces).length > 0;
  }
  setFace(blockFace, face) {
    this.faces[blockFace.name] = face;
  }
  deleteFace(blockFace) {
    delete this.faces[blockFace.name];
  }
  /**
   * Returns a new element with only the filtered faces
   */
  filterFaces(predicate) {
    const clone = this.clone();
    const faces = Object.entries(this.faces);
    faces.filter(([fname, face]) => predicate(BlockFace.fromName(fname), face)).forEach(([fname, face]) => {
      clone.faces[fname] = face;
    });
    return clone;
  }
  /**
   * Gets the culling for a face
   */
  getFaceCull(face) {
    const _face = this.faces[face.name];
    return (_face == null ? void 0 : _face.cullface) ? BlockFace.fromName(_face.cullface) : face;
  }
  /**
   * Returns a shallow copy of this element
   */
  clone() {
    const clone = new ElementClass(this.toJSON());
    clone.rotation = this.rotation;
    clone.shade = this.shade;
    return clone;
  }
  /**
   * Generates the Minecraft model JSON
   */
  toJSON() {
    const obj = {
      from: this.from.components,
      to: this.to.components
    };
    if (this.hasFaces) {
      obj.faces = this.faces;
    }
    if (this.rotation) {
      obj.rotation = this.rotation;
    }
    if (!this.shade) {
      obj.shade = false;
    }
    return obj;
  }
}
class DisplayTransformClass {
  constructor(data) {
    __publicField(this, "rotation", Vector3Props.ZERO);
    __publicField(this, "translation", Vector3Props.ZERO);
    __publicField(this, "scale", Vector3Props.ONE);
    if (data.rotation)
      this.rotation = new Vector(data.rotation);
    if (data.translation)
      this.translation = new Vector(data.translation);
    if (data.scale)
      this.scale = new Vector(data.scale);
  }
  /**
   * Returns a shallow copy of this transform
   */
  clone() {
    const clone = new DisplayTransformClass(this.toJSON());
    clone.rotation = this.rotation;
    clone.translation = this.translation;
    clone.scale = this.scale;
    return clone;
  }
  toJSON() {
    const obj = {};
    if (this.rotation.every((n) => n === 0))
      obj.rotation = this.rotation.components;
    if (this.translation.every((n) => n === 0))
      obj.translation = this.translation.components;
    if (this.scale.every((n) => n === 1))
      obj.scale = this.scale.components;
    return obj;
  }
}
class Vector {
  constructor(...args) {
    __publicField(this, "dimensions");
    __publicField(this, "_values");
    if (typeof args[0] === "number") {
      this._values = args.slice();
    } else {
      const arg = args[0];
      if (arg instanceof Vector) {
        this._values = arg._values;
      } else {
        this._values = arg;
      }
    }
    this.dimensions = args.length;
    Object.freeze(this._values);
    Object.freeze(this);
  }
  get(idx) {
    return this._values[idx];
  }
  get components() {
    return Object.freeze(this._values.slice());
  }
  get lengthSquared() {
    return this._values.reduce((a, b) => a + b * b, 0);
  }
  get length() {
    return Math.sqrt(this.lengthSquared);
  }
  [Symbol.iterator]() {
    return this._values[Symbol.iterator]();
  }
  every(predicate) {
    return this._values.every(predicate);
  }
  dot(...args) {
    if (typeof args[0] === "number") {
      assert(args.length === this.dimensions, "Vectors do not have a matching length");
      return this._values.map((n, i) => n * args[i]).reduce((a, b) => a + b);
    }
    const other = args[0];
    assert(other.dimensions === this.dimensions, "Vectors do not have a matching length");
    return this._values.map((n, i) => n * other._values[i]).reduce((a, b) => a + b);
  }
  /**
   * Applies a map function to each component
   */
  map(mapFn) {
    return new Vector(
      this._values.map(mapFn)
    );
  }
  round() {
    return this.map(Math.round);
  }
  floor() {
    return this.map(Math.floor);
  }
  ceil() {
    return this.map(Math.ceil);
  }
  square() {
    return this.map((n) => n * n);
  }
  add(...args) {
    if (typeof args[0] === "number") {
      assert(args.length === this.dimensions, "Vectors do not have a matching length");
      return new Vector(
        Object.freeze(this._values.map((n, i) => n + args[i]))
      );
    }
    const other = args[0];
    if (other instanceof Vector) {
      assert(other.dimensions === this.dimensions, "Vectors do not have a matching length");
      return new Vector(
        Object.freeze(this._values.map((n, i) => n + other._values[i]))
      );
    }
    return new Vector(
      Object.freeze(this._values.map((n) => n + other))
    );
  }
  sub(...args) {
    if (typeof args[0] === "number") {
      assert(args.length === this.dimensions, "Vectors do not have a matching length");
      return new Vector(
        Object.freeze(this._values.map((n, i) => n - args[i]))
      );
    }
    const other = args[0];
    if (other instanceof Vector) {
      assert(other.dimensions === this.dimensions, "Vectors do not have a matching length");
      return new Vector(
        Object.freeze(this._values.map((n, i) => n - other._values[i]))
      );
    }
    return new Vector(
      Object.freeze(this._values.map((n) => n - other))
    );
  }
  mul(...args) {
    if (typeof args[0] === "number") {
      assert(args.length === this.dimensions, "Vectors do not have a matching length");
      return new Vector(
        Object.freeze(this._values.map((n, i) => n * args[i]))
      );
    }
    const other = args[0];
    if (other instanceof Vector) {
      assert(other.dimensions === this.dimensions, "Vectors do not have a matching length");
      return new Vector(
        Object.freeze(this._values.map((n, i) => n * other._values[i]))
      );
    }
    return new Vector(
      Object.freeze(this._values.map((n) => n * other))
    );
  }
  div(...args) {
    if (typeof args[0] === "number") {
      assert(args.length === this.dimensions, "Vectors do not have a matching length");
      return new Vector(
        Object.freeze(this._values.map((n, i) => n / args[i]))
      );
    }
    const other = args[0];
    if (other instanceof Vector) {
      assert(other.dimensions === this.dimensions, "Vectors do not have a matching length");
      return new Vector(
        Object.freeze(this._values.map((n, i) => n / other._values[i]))
      );
    }
    return new Vector(
      Object.freeze(this._values.map((n) => n / other))
    );
  }
  toCoordsAbs() {
    return new import_variables.VectorClass(
      this._values.map((n) => `${n}`)
    );
  }
  toCoordsRel() {
    return new import_variables.VectorClass(
      this._values.map((n) => `~${n}`)
    );
  }
  toCoordsLoc() {
    return new import_variables.VectorClass(
      this._values.map((n) => `^${n}`)
    );
  }
}
const Vector2Props = {
  ZERO: new Vector(0, 0),
  ONE: new Vector(1, 1)
};
function vec2(x, y) {
  if (typeof x === "number") {
    return new Vector(x, y);
  }
  return new Vector(x);
}
const Vector3Props = {
  ZERO: new Vector(0, 0, 0),
  ONE: new Vector(1, 1, 1)
};
function vec3(x, y, z) {
  if (typeof x === "number") {
    return new Vector(x, y, z);
  }
  return new Vector(x);
}
const Vector4Props = {
  ZERO: new Vector(0, 0, 0, 0),
  ONE: new Vector(1, 1, 1, 1)
};
function vec4(x, y, z, w) {
  if (typeof x === "number") {
    return new Vector(x, y, z, w);
  }
  return new Vector(x);
}
function vecN(...args) {
  if (typeof args[0] === "number") {
    return new Vector(args);
  }
  return new Vector(args[0]);
}
const _BlockFace = class _BlockFace {
  constructor(name, id, normal, opposite) {
    this.name = name;
    this.id = id;
    __publicField(this, "opposite");
    /* @ts-ignore */
    __publicField(this, "normal");
    __publicField(this, "isSide");
    __publicField(this, "axis");
    this.normal = new Vector(normal);
    if (normal[0] !== 0)
      this.axis = "x";
    else if (normal[1] !== 0)
      this.axis = "y";
    else
      this.axis = "z";
    this.isSide = this.axis !== "y";
    this.opposite = opposite;
  }
  static fromName(name) {
    switch (name) {
      case "north":
        return this.North;
      case "east":
        return this.East;
      case "south":
        return this.South;
      case "west":
        return this.West;
      case "up":
        return this.Up;
      case "down":
        return this.Down;
      default:
        return null;
    }
  }
  static [Symbol.iterator]() {
    return BLOCK_FACES[Symbol.iterator]();
  }
  static get filter() {
    return BLOCK_FACES.filter;
  }
  static forEach(callback) {
    BLOCK_FACES.forEach((face, id) => callback(face, id));
  }
  toString() {
    return this.name;
  }
  toJSON() {
    return this.name;
  }
};
__publicField(_BlockFace, "North", new _BlockFace("north", 0, [0, 0, -1], "South"));
__publicField(_BlockFace, "East", new _BlockFace("east", 1, [1, 0, 0], "West"));
__publicField(_BlockFace, "South", new _BlockFace("south", 2, [0, 0, 1], "North"));
__publicField(_BlockFace, "West", new _BlockFace("west", 3, [-1, 0, 0], "East"));
__publicField(_BlockFace, "Up", new _BlockFace("up", 4, [0, 1, 0], "Down"));
__publicField(_BlockFace, "Down", new _BlockFace("down", 5, [0, -1, 0], "Up"));
let BlockFace = _BlockFace;
const BLOCK_FACES = Object.freeze([
  BlockFace.North,
  BlockFace.East,
  BlockFace.South,
  BlockFace.West,
  BlockFace.Up,
  BlockFace.Down
]);
BlockFace.forEach((face) => {
  const opp = face.opposite;
  face.opposite = BlockFace[opp];
});
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  BlockFace,
  DisplayTransformClass,
  ElementClass,
  MAX_CUSTOM_MODEL_DATA,
  ModelClass,
  ModelNode
});
//# sourceMappingURL=model.cjs.map