/// <reference types="node" resolution-mode="require"/>
import { ContainerNode } from '../../nodes.js';
import { ResourceClass } from '../resource.js';
import type { SOUND_TYPES, SoundsDefinitions } from 'sandstone/arguments';
import type { SandstoneCore } from '../../sandstoneCore.js';
import type { ListResource, ResourceClassArguments, ResourceNode } from '../resource.js';
/**
 * A node representing a Minecraft sound.
 */
export declare class SoundEventNode<Type extends SOUND_TYPES> extends ContainerNode implements ResourceNode<SoundEventClass<Type>> {
    resource: SoundEventClass<Type>;
    constructor(sandstoneCore: SandstoneCore, resource: SoundEventClass<Type>);
    getValue: () => Buffer | Promise<Buffer> | undefined;
}
export type SoundEventArguments = {
    /**
     * The sound's buffer. Path string in `./resources` will be used. Defaults to `/resources/resourcepack/assets/(namespace)/sounds/(type)/(name)`.
     */
    sound?: string | Promise<Buffer> | Buffer;
    /**
     * Whether to automatically add this to a sounds.json file. Defaults to false.
     */
    addToSounds?: boolean;
} & ResourceClassArguments<'default'>;
export declare class SoundEventClass<Type extends SOUND_TYPES> extends ResourceClass<SoundEventNode<Type>> {
    type: Type;
    buffer?: Promise<Buffer> | Buffer;
    constructor(core: SandstoneCore, type: Type, name: string, args: SoundEventArguments);
}
/**
 * A node representing a Minecraft sound.
 */
export declare class SoundsNode extends ContainerNode implements ResourceNode<SoundsClass> {
    resource: SoundsClass;
    constructor(sandstoneCore: SandstoneCore, resource: SoundsClass);
    getValue: () => string;
}
export type SoundsArguments = {
    /**
     * The sounds definition map.
     */
    definitions?: SoundsDefinitions;
} & ResourceClassArguments<'default'>;
export declare class SoundsClass extends ResourceClass<SoundsNode> implements ListResource {
    soundsJSON: SoundsDefinitions | Promise<SoundsDefinitions>;
    constructor(core: SandstoneCore, namespace: string, args: SoundsArguments);
    push(...soundEvents: SoundsClass[] | SoundEventClass<SOUND_TYPES>[]): Promise<void>;
    unshift(...soundEvents: SoundsClass[] | SoundEventClass<SOUND_TYPES>[]): Promise<void>;
}
