import path from 'path';
import { ContainerNode } from '../../nodes.js';
import { ResourceClass } from '../resource.js';
const sounds = new Map();
/**
 * A node representing a Minecraft sound.
 */
export class SoundEventNode extends ContainerNode {
    resource;
    constructor(sandstoneCore, resource) {
        super(sandstoneCore);
        this.resource = resource;
    }
    getValue = () => this.resource.buffer;
}
export class SoundEventClass extends ResourceClass {
    type;
    buffer;
    constructor(core, type, name, args) {
        super(core, { packType: core.pack.resourcePack(), extension: 'ogg', encoding: false }, (SoundEventNode), core.pack.resourceToPath(name, ['sounds', type]), args);
        this.type = type;
        if (args.addToSandstoneCore && args.sound !== undefined) {
            if (typeof args.sound === 'string') {
                this.buffer = core.getExistingResource(path.join('resourcepack', 'assets', ...this.path), false);
            }
            else {
                this.buffer = args.sound;
            }
            if (args.addToSounds) {
                let def = sounds.get(this.path[0]);
                if (!def) {
                    def = sounds.set(this.path[0], new SoundsClass(this.core, this.path[0], {
                        addToSandstoneCore: true,
                        creator: 'sandstone',
                    })).get(this.path[0]);
                }
                def.push(this);
            }
        }
        this.handleConflicts();
    }
}
/**
 * A node representing a Minecraft sound.
 */
export class SoundsNode extends ContainerNode {
    resource;
    constructor(sandstoneCore, resource) {
        super(sandstoneCore);
        this.resource = resource;
    }
    getValue = () => JSON.stringify(this.resource.soundsJSON);
}
export class SoundsClass extends ResourceClass {
    soundsJSON;
    constructor(core, namespace, args) {
        super(core, { packType: core.pack.resourcePack() }, SoundsNode, core.pack.resourceToPath(`${namespace}:sounds`, []), args);
        if (args.definitions) {
            this.soundsJSON = args.definitions;
        }
        else {
            this.soundsJSON = (async () => JSON.parse(await core.getExistingResource(this)))();
        }
    }
    async push(...soundEvents) {
        if (soundEvents[0] instanceof SoundsClass) {
            for await (const _sounds of soundEvents) {
                const def = _sounds;
                const s = await this.soundsJSON;
                // TODO: Implement sound event options
                this.soundsJSON = { ...s, ...def };
            }
        }
        else {
            for await (const _sound of soundEvents) {
                const sound = _sound;
                const s = await this.soundsJSON;
                // TODO: Implement sound event options
                s[`${sound.type}.${sound.name}`] = {
                    sounds: [`${sound.type}.${sound.name}`],
                };
            }
        }
    }
    async unshift(...soundEvents) {
        if (soundEvents[0] instanceof SoundsClass) {
            for await (const _sounds of soundEvents) {
                const def = _sounds;
                const s = await this.soundsJSON;
                // TODO: Implement sound event options
                this.soundsJSON = { ...def, ...s };
            }
        }
        else {
            for await (const _sound of soundEvents) {
                const sound = _sound;
                const s = await this.soundsJSON;
                // TODO: Implement sound event options
                s[`${sound.type}.${sound.name}`] = {
                    sounds: [`${sound.type}.${sound.name}`],
                };
            }
        }
    }
}
