/// <reference types="node" resolution-mode="require"/>
import { ContainerNode } from '../../nodes.js';
import { ResourceClass } from '../resource.js';
import type { TEXTURE_TYPES, TextureMeta } from 'sandstone/arguments';
import type { LiteralUnion } from 'sandstone/utils';
import type { SandstoneCore } from '../../sandstoneCore.js';
import type { ResourceClassArguments, ResourceNode } from '../resource.js';
type TextureType = LiteralUnion<TEXTURE_TYPES>;
/**
 * A node representing a Minecraft sound.
 */
export declare class TextureNode<Type extends TextureType> extends ContainerNode implements ResourceNode<TextureClass<Type>> {
    resource: TextureClass<Type>;
    constructor(sandstoneCore: SandstoneCore, resource: TextureClass<Type>);
    getValue: () => Buffer | Promise<Buffer> | undefined;
}
export type TextureArguments<Type extends TextureType> = {
    /**
     * The texture's buffer.
     */
    texture?: Promise<Buffer> | Buffer;
    sprite?: boolean | string;
    meta?: TextureMeta<Type>;
} & ResourceClassArguments<'default'>;
export declare class TextureClass<Type extends TextureType> extends ResourceClass<TextureNode<Type>> {
    type: Type;
    isSprite: boolean;
    spriteTarget?: string;
    meta?: TextureMeta<Type>;
    buffer: TextureArguments<Type>['texture'];
    constructor(core: SandstoneCore, type: Type, name: string, args: TextureArguments<Type>);
    toString(): string;
    videoToAnimation(path: string): void;
}
export {};
