"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
var sandstoneCore_exports = {};
__export(sandstoneCore_exports, {
  SandstoneCore: () => SandstoneCore
});
module.exports = __toCommonJS(sandstoneCore_exports);
var import_fs_extra = __toESM(require("fs-extra"), 1);
var import_path = __toESM(require("path"), 1);
var import_dependencies = require("../pack/dependencies.js");
var import_mcmeta = require("./mcmeta.js");
var import_dependency = require("./resources/dependency.js");
var import_smithed = require("./smithed.js");
class SandstoneCore {
  constructor(pack) {
    this.pack = pack;
    /** All Resources */
    __publicField(this, "resourceNodes");
    __publicField(this, "mcfunctionStack");
    __publicField(this, "awaitNodes");
    __publicField(this, "currentNode", "");
    __publicField(this, "mcmetaCache", new import_mcmeta.MCMetaCache());
    __publicField(this, "smithed", new import_smithed.SmithedDependencyCache(this));
    __publicField(this, "dependencies", []);
    __publicField(this, "getCurrentMCFunctionOrThrow", () => {
      const { currentMCFunction } = this;
      if (!currentMCFunction) {
        throw new Error("This operation is invalid when outside a MCFunction.");
      }
      return currentMCFunction;
    });
    __publicField(this, "insideContext", (...args) => this.getCurrentMCFunctionOrThrow().insideContext(...args));
    /**
     * Create a new MCFunction with the given name, and switch the currently active MCFunction to it.
     * @param mcfunction The MCFunction to switch to.
     * @return The newly created and active MCFunction.
     */
    __publicField(this, "enterMCFunction", (mcfunction) => {
      const node = mcfunction["node"];
      this.mcfunctionStack.push(node);
      return node;
    });
    /**
     * Leave the current MCFunction, and return to the previous one.
     * @return The previously active MCFunction.
     */
    __publicField(this, "exitMCFunction", () => this.mcfunctionStack.pop());
    __publicField(this, "generateResources", (opts) => {
      const originalResources = new Set(this.resourceNodes);
      for (const { resource } of this.resourceNodes) {
        resource["generate"]();
      }
      for (const visitor of opts.visitors) {
        visitor.onStart();
        for (const node of this.resourceNodes) {
          visitor.visit(node);
        }
        visitor.onEnd();
      }
      const finalResources = this.resourceNodes;
      this.resourceNodes = originalResources;
      return finalResources;
    });
    __publicField(this, "save", async (cliOptions, opts) => {
      if (!this.smithed.loaded) {
        await this.smithed.load();
      }
      await Promise.allSettled(this.dependencies);
      await this.mcmetaCache.save();
      const resources = this.generateResources(opts);
      for await (const node of resources) {
        const { packType, fileExtension } = node.resource;
        const _path = [packType.type, ...node.resource.path];
        if (packType.resourceSubFolder) {
          _path.splice(1, 0, packType.resourceSubFolder);
        }
        const resourcePath = import_path.default.join(..._path);
        const value = node.getValue();
        if (cliOptions.verbose) {
          console.log(
            `Path: ${resourcePath}.${fileExtension}

${typeof value === "string" ? value : "<Buffer>"}`
          );
        }
        if (!cliOptions.dry) {
          await cliOptions.fileHandler(`${resourcePath}.${fileExtension}`, value);
        }
      }
    });
    this.resourceNodes = /* @__PURE__ */ new Set();
    this.mcfunctionStack = [];
    this.awaitNodes = /* @__PURE__ */ new Set();
    for (const method of Object.getOwnPropertyNames(Object.getPrototypeOf(this))) {
      if (method !== "constructor" && typeof this[method] === "function" && typeof this[method].bind === "function") {
        this[method] = this[method].bind(this);
      }
    }
  }
  /**
   * The current MCFunction.
   */
  get currentMCFunction() {
    return this.mcfunctionStack[this.mcfunctionStack.length - 1];
  }
  async getExistingResource(pathOrResource, encoding = "utf-8") {
    if (typeof pathOrResource === "string") {
      if (encoding === false) {
        return import_fs_extra.default.readFile(pathOrResource);
      }
      return import_fs_extra.default.readFile(pathOrResource, encoding);
    }
    const _path = pathOrResource.path;
    if (_path[0] === "minecraft") {
      const type = pathOrResource.packType.resourceSubFolder;
      return this.mcmetaCache.get(type, `${type}/${_path.join("/")}${pathOrResource.fileExtension ? `.${pathOrResource.fileExtension}` : ""}`, encoding === "utf-8");
    }
    const fullPath = import_path.default.join(process.env.WORKING_DIR, `resources/${import_path.default.join(pathOrResource.packType.type, ..._path)}${pathOrResource.fileExtension ? `.${pathOrResource.fileExtension}` : ""}`);
    if (pathOrResource.fileEncoding === false) {
      return import_fs_extra.default.readFile(fullPath);
    }
    return import_fs_extra.default.readFile(fullPath, pathOrResource.fileEncoding);
  }
  async getVanillaResource(relativePath, text = true, type = "server") {
    return this.mcmetaCache.get(type === "server" ? "data" : "assets", relativePath, text);
  }
  /**
   * Add a dependency for a Smithed Library
   */
  depend(dependency, version = "latest") {
    const adding = this.dependencies.length === 0 ? false : Promise.allSettled(this.dependencies);
    this.dependencies.push((async () => {
      if (!this.smithed.loaded) {
        await this.smithed.load();
      }
      if (adding)
        await adding;
      if (!this.smithed.has(dependency)) {
        const _depend = await this.smithed.get(dependency, version);
        if (!this.pack.packTypes.has("datapack-dependencies")) {
          this.pack.packTypes.set("datapack-dependencies", new import_dependencies.DataPackDependencies());
        }
        const dependencies = [new import_dependency.SmithedDependencyClass(this, dependency, _depend, "server")];
        if (_depend.resourcepack) {
          if (!this.pack.packTypes.has("resourcepack-dependencies")) {
            this.pack.packTypes.set("resourcepack-dependencies", new import_dependencies.ResourcePackDependencies());
          }
          dependencies.push(new import_dependency.SmithedDependencyClass(this, dependency, _depend, "client"));
        }
        return dependencies;
      }
      return true;
    })());
  }
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  SandstoneCore
});
//# sourceMappingURL=sandstoneCore.cjs.map