/// <reference types="node" resolution-mode="require"/>
/// <reference types="node" resolution-mode="require"/>
import fs from 'fs-extra';
import { MCMetaCache } from './mcmeta.js';
import { SmithedDependencyClass } from './resources/dependency.js';
import { SmithedDependencyCache } from './smithed.js';
import type { SandstonePack } from 'sandstone/pack';
import type { AwaitNode } from './nodes.js';
import type { _RawMCFunctionClass, MCFunctionClass, MCFunctionNode } from './resources/datapack/mcfunction.js';
import type { ResourceClass, ResourceNode } from './resources/resource.js';
import type { GenericCoreVisitor } from './visitors.js';
export declare class SandstoneCore {
    pack: SandstonePack;
    /** All Resources */
    resourceNodes: Set<ResourceNode>;
    mcfunctionStack: MCFunctionNode[];
    awaitNodes: Set<AwaitNode>;
    currentNode: string;
    mcmetaCache: MCMetaCache;
    smithed: SmithedDependencyCache;
    dependencies: Promise<SmithedDependencyClass[] | true>[];
    constructor(pack: SandstonePack);
    /**
     * The current MCFunction.
     */
    get currentMCFunction(): MCFunctionNode | undefined;
    getCurrentMCFunctionOrThrow: () => MCFunctionNode;
    insideContext: MCFunctionNode['insideContext'];
    /**
     * Create a new MCFunction with the given name, and switch the currently active MCFunction to it.
     * @param mcfunction The MCFunction to switch to.
     * @return The newly created and active MCFunction.
     */
    enterMCFunction: (mcfunction: _RawMCFunctionClass<any, any> | MCFunctionClass<any, any>) => MCFunctionNode;
    /**
     * Leave the current MCFunction, and return to the previous one.
     * @return The previously active MCFunction.
     */
    exitMCFunction: () => MCFunctionNode | undefined;
    getExistingResource(relativePath: string): Promise<string>;
    getExistingResource(relativePath: string, encoding: false): Promise<Buffer>;
    getExistingResource(relativePath: string, encoding: fs.EncodingOption): Promise<Buffer | string>;
    getExistingResource(resource: ResourceClass, encoding?: 'utf-8'): Promise<string>;
    getExistingResource(resource: ResourceClass, encoding: false | fs.EncodingOption): Promise<Buffer>;
    getVanillaResource(relativePath: string): Promise<string>;
    getVanillaResource(relativePath: string, text: true, type: 'client' | 'server'): Promise<string>;
    getVanillaResource(relativePath: string, text: false, type: 'client' | 'server'): Promise<Buffer>;
    /**
     * Add a dependency for a Smithed Library
     */
    depend(dependency: string, version?: string): void;
    generateResources: (opts: {
        visitors: GenericCoreVisitor[];
    }) => Set<ResourceNode>;
    save: (cliOptions: {
        fileHandler: (relativePath: string, content: any) => Promise<void>;
        dry: boolean;
        verbose: boolean;
    }, opts: {
        visitors: GenericCoreVisitor[];
    }) => Promise<void>;
}
