"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
var smithed_exports = {};
__export(smithed_exports, {
  SmithedDependencyCache: () => SmithedDependencyCache
});
module.exports = __toCommonJS(smithed_exports);
var import_adm_zip = __toESM(require("adm-zip"), 1);
var import_fs_extra = __toESM(require("fs-extra"), 1);
var import_path = __toESM(require("path"), 1);
var import_utils = require("sandstone/utils");
class SmithedDependencyCache {
  constructor(core) {
    __publicField(this, "core");
    __publicField(this, "baseURL", "https://api.smithed.dev/v2/");
    __publicField(this, "path", import_path.default.join(process.env.WORKING_DIR, "../", "resources", "cache", "smithed"));
    __publicField(this, "manifest", import_path.default.join(this.path, "..", "..", "smithed.json"));
    __publicField(this, "lockFile", import_path.default.join(this.path, "..", "lock-smithed.json"));
    __publicField(this, "dependencies", /* @__PURE__ */ new Map());
    __publicField(this, "loaded", false);
    this.core = core;
  }
  async load() {
    this.loaded = true;
    if (!await import_fs_extra.default.pathExists(this.manifest)) {
      return;
    }
    const manifest = JSON.parse(await import_fs_extra.default.readFile(this.manifest, "utf-8"));
    for (const [id, version] of Object.entries(manifest)) {
      this.core.depend(id, version);
    }
  }
  async save() {
    if (this.dependencies.size !== 0) {
      const dependencies = {};
      for (const [dependency, { version }] of this.dependencies.entries()) {
        dependencies[dependency] = version;
      }
      await (0, import_utils.safeWrite)(this.manifest, JSON.stringify(dependencies));
      const lock = {};
      for (const [name, { version, date, resourcepack }] of this.dependencies.entries()) {
        lock[name] = {
          version,
          date,
          resourcepack: !!resourcepack
        };
      }
      await (0, import_utils.safeWrite)(this.lockFile, JSON.stringify(lock));
    }
  }
  async get(dependency, version) {
    if (!this.loaded) {
      await this.load();
    }
    const existing = this.dependencies.get(dependency);
    if (existing && existing.version === version) {
      existing.added = true;
      return existing;
    }
    let result;
    const lockFile = await (async () => {
      try {
        return JSON.parse(await import_fs_extra.default.readFile(this.lockFile, "utf-8"));
      } catch (e) {
        return false;
      }
    })();
    let download = false;
    if (!lockFile || !lockFile[dependency] || lockFile[dependency].version !== version) {
      download = true;
    } else {
      const date = Date.now();
      if (version === "latest" && (lockFile[dependency].date - date) / 36e5 > 1) {
        lockFile[dependency].date = date;
        download = true;
      } else {
        result = this.dependencies.set(dependency, {
          version,
          date: lockFile[dependency].date,
          datapack: await import_fs_extra.default.readFile(import_path.default.join(this.path, dependency, "datapack.zip")),
          resourcepack: lockFile[dependency].resourcepack ? await import_fs_extra.default.readFile(import_path.default.join(this.path, dependency, "resourcepack.zip")) : false,
          added: false
        }).get(dependency);
      }
    }
    if (download) {
      let url = `${this.baseURL}download?pack=${dependency}`;
      let hasResourcePack = false;
      const pack = JSON.parse(await (await (0, import_utils.fetch)(`${this.baseURL}packs/${dependency}`)).text());
      if (version === "latest") {
        const { versions } = pack;
        const ver = versions[versions.length - 1];
        if (ver.downloads.resourcepack) {
          hasResourcePack = true;
        }
      } else {
        const ver = pack.versions.find((_ver) => _ver.name === version);
        if (ver.downloads.resourcepack) {
          hasResourcePack = true;
        }
        url += `@${version}`;
      }
      const files = new import_adm_zip.default(await (await (0, import_utils.fetch)(url)).buffer()).getEntries();
      const datapack = await new Promise((res) => {
        files[0].getDataAsync((data) => res(data));
      });
      await (0, import_utils.safeWrite)(import_path.default.join(this.path, dependency, "datapack.zip"), datapack);
      let resourcepack = false;
      if (hasResourcePack) {
        resourcepack = await new Promise((res) => {
          files[1].getDataAsync((data) => res(data));
        });
        await (0, import_utils.safeWrite)(import_path.default.join(this.path, dependency, "resourcepack.zip"), datapack);
      }
      result = this.dependencies.set(dependency, {
        version,
        date: version === "latest" ? Date.now() : 0,
        datapack,
        resourcepack,
        added: true
      }).get(dependency);
      await this.save();
    }
    return result;
  }
  has(dependency) {
    const depend = this.dependencies.get(dependency);
    return depend && depend.added;
  }
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  SmithedDependencyCache
});
//# sourceMappingURL=smithed.cjs.map