import { ContainerCommandNode, ContainerNode, } from 'sandstone/core';
// eslint-disable-next-line @typescript-eslint/ban-types
export function ancestors(prototype) {
    if (!prototype.name) {
        return [];
    }
    return [prototype.name, ...(ancestors(Object.getPrototypeOf(prototype)))];
}
export class GenericCoreVisitor {
    core;
    methodCache;
    constructor(core) {
        this.core = core;
        this.methodCache = new Map();
    }
    getSingleNodeMethod = (nodeName) => {
        const methodName = `visit${nodeName}`;
        const that = this;
        const method = that[methodName];
        return method;
    };
    // eslint-disable-next-line @typescript-eslint/ban-types
    getNodeMethods = (prototype) => {
        // First, try to hit the cache
        const key = prototype.name;
        const cachedResult = this.methodCache.get(key);
        if (cachedResult) {
            return cachedResult;
        }
        // Else, find the different methods we need to chain
        const names = ancestors(prototype);
        for (const name of names) {
            const method = this.getSingleNodeMethod(name);
            if (method) {
                this.methodCache.set(key, method);
                return method;
            }
        }
        const result = (node_) => node_;
        this.methodCache.set(key, result);
        return result;
    };
    visit = (node) => this.getNodeMethods(node.constructor)(node);
    genericVisit = (node) => {
        if (node instanceof ContainerNode || node instanceof ContainerCommandNode) {
            node.body = node.body.flatMap((n) => this.visit(n));
        }
        return node;
    };
    onStart = () => { };
    onEnd = () => { };
    genericNodeVisitor = () => (node) => this.genericVisit(node);
    // Container nodes
    visitCommandNode = this.genericNodeVisitor();
    visitContainerCommandNode = this.genericNodeVisitor();
    visitContainerNode = this.genericNodeVisitor();
    // //// Those aren't necessary, but are here for better auto-completion & type-checking. //// //
    // Resources
    visitNode;
    visitMCFunctionNode;
    visitAdvancementNode;
    visitItemModifierNode;
    visitLootTableNode;
    visitPredicateNode;
    visitRecipeNode;
    visitTagNode;
    visitTrimMaterialNode;
    visitTrimPatternNode;
}
