import { Score } from 'sandstone/variables';
import { AndNode, ConditionNode, NotNode, OrNode } from './conditions/index.js';
import { IfStatement } from './if_else.js';
import { ForIStatement, ForOfStatement, WhileStatement } from './loops/index.js';
import { CaseStatement } from './switch_case.js';
import type { JSONTextComponent, MultiplePlayersArgument, NBTObject } from 'sandstone/arguments';
import type { ConditionClass, DataArray, DataArrayInitial, DataIndexMap, DataIndexMapInitial, DataPointClass, IterableDataClass, StringDataPointClass } from 'sandstone/variables';
import type { DataPointPickClass, SandstoneCore } from '../core/index.js';
import type { DefaultType } from './switch_case.js';
export type Condition = ConditionNode | ConditionClass;
export declare class Flow {
    sandstoneCore: SandstoneCore;
    constructor(sandstoneCore: SandstoneCore);
    conditionToNode(condition: Condition): ConditionNode;
    if: (condition: Condition, callback: () => void) => IfStatement;
    and: (...conditions: Condition[]) => AndNode;
    or: (...conditions: Condition[]) => OrNode;
    not: (condition: Condition) => NotNode;
    get return(): import("../commands/index.js").ReturnArgumentsCommand<false> & ((value?: number | undefined) => import("../commands/helpers.js").FinalCommandOutput);
    /**
     * Logs an error in data storage and chat by default and `return fail`'s from the context.
     *
     * @param error Error to log before `return`.
     * @param broadcast Whether to scope the broadcast of the error (from `@a`) or disable it.
     *
     * @returns DataPoint where error was stored.
     */
    throw(error?: JSONTextComponent, broadcast?: MultiplePlayersArgument<false> | false): DataPointClass;
    /**
     * Logs an error in data storage and chat by default and `return fail`'s from the context.
     *
     * @param error Error to log before `return`.
     * @param broadcast Whether to scope the broadcast of the error (from `@a`) or disable it.
     * @param dataPoint Disable storing to data point.
     *
     * @returns The full error text component string.
     */
    throw(error: JSONTextComponent | undefined, broadcast: MultiplePlayersArgument<false> | false | undefined, dataPoint: false): string;
    /**
     * Logs an error in data storage and chat by default and `return fail`'s from the context.
     *
     * @param error Error to log before `return`.
     * @param broadcast Whether to scope the broadcast of the error (from `@a`) or disable it.
     * @param dataPoint Specific point to store the error to.
     *
     * @returns DataPoint that was set.
     */
    throw(error: JSONTextComponent | undefined, broadcast: MultiplePlayersArgument<false> | false | undefined, dataPoint: DataPointClass): DataPointClass;
    while(condition: Condition, callback: () => void): WhileStatement;
    doWhile(condition: Condition, callback: () => void): WhileStatement;
    for(initial: number | Score, end: (iterator: Score) => Condition, iterate: (iterator: Score) => Score, callback: (iterator: Score | number) => any): ForIStatement;
    for(range: [start: number | Score, end: number | Score], type: 'iterate', callback: (iterator: Score) => any): ForIStatement;
    for(range: [start: number | Score, end: number | Score, maximum?: number], type: 'binary', callback: (num: number) => any): void;
    for(type: 'entry', _: 'of', iterable: IterableDataClass, callback: (entry: DataPointClass) => any): ForOfStatement<'entry', [entry: DataPointClass]>;
    for(type: ['key', 'value'], _: 'of', iterable: DataIndexMap<DataIndexMapInitial>, callback: (key: StringDataPointClass, value: DataPointClass) => any): ForOfStatement<['key', 'value'], [key: StringDataPointClass, value: DataPointClass]>;
    for(type: ['i', 'entry'], _: 'of', iterable: DataArray<DataArrayInitial>, callback: (i: Score, entry: DataPointClass) => any): ForOfStatement<['i', 'entry'], [i: Score, entry: DataPointClass]>;
    switch<ValueType extends DataPointClass | DataPointPickClass | Score, CheckType extends ValueType extends Score ? number : NBTObject>(value: ValueType, cases: CaseStatement<CheckType> | DefaultType<CheckType>): void;
    switch<ValueType extends DataPointClass | DataPointPickClass | Score, CheckType extends ValueType extends Score ? number : NBTObject>(value: ValueType, cases: ['case', CheckType, () => any][], _default?: ['default', () => any]): void;
    case<ValueType extends number | NBTObject>(value: ValueType, callback: () => any): CaseStatement<ValueType>;
}
