import { parseJSONText, Score } from 'sandstone/variables';
import { AndNode, ConditionNode, NotNode, OrNode, } from './conditions/index.js';
import { IfStatement } from './if_else.js';
import { binaryFor, ForIStatement, ForOfStatement, WhileStatement, } from './loops/index.js';
import { CaseStatement } from './switch_case.js';
let switches = 0;
export class Flow {
    sandstoneCore;
    constructor(sandstoneCore) {
        this.sandstoneCore = sandstoneCore;
    }
    conditionToNode(condition) {
        if (!(condition instanceof ConditionNode)) {
            return condition._toMinecraftCondition();
        }
        return condition;
    }
    if = (condition, callback) => new IfStatement(this.sandstoneCore, this.conditionToNode(condition), callback);
    and = (...conditions) => new AndNode(this.sandstoneCore, conditions.map((condition) => this.conditionToNode(condition)));
    or = (...conditions) => new OrNode(this.sandstoneCore, conditions.map((condition) => this.conditionToNode(condition)));
    not = (condition) => new NotNode(this.sandstoneCore, this.conditionToNode(condition));
    get return() {
        return this.sandstoneCore.pack.commands.returnCmd;
    }
    throw(error, broadcast, dataPoint) {
        const { pack, getCurrentMCFunctionOrThrow } = this.sandstoneCore;
        const { DataVariable, commands } = pack;
        const { tellraw, returnCmd } = commands;
        const node = getCurrentMCFunctionOrThrow();
        const fullError = [
            { text: '\nSandstone Pack Error:\n', color: 'yellow' },
            error || '[Function Threw]', '\n\n',
            `@ Function ${node.resource.name}. Node ${node.body.length - 1}.\n`,
        ];
        if (broadcast !== false) {
            tellraw(broadcast || '@a', fullError);
        }
        const errorString = parseJSONText(fullError);
        if (dataPoint !== false) {
            let point;
            if (dataPoint === undefined) {
                point = DataVariable(parseJSONText(fullError));
            }
            else {
                point = dataPoint.set(errorString);
            }
            returnCmd.fail();
            return point;
        }
        returnCmd.fail();
        return errorString;
    }
    while(condition, callback) {
        return new WhileStatement(this.sandstoneCore, this.conditionToNode(condition), callback);
    }
    doWhile(condition, callback) {
        callback();
        return new WhileStatement(this.sandstoneCore, this.conditionToNode(condition), callback);
    }
    for(arg1, arg2, arg3, arg4) {
        if (typeof arg1 === 'number' || arg1 instanceof Score) {
            return new ForIStatement(this.sandstoneCore, arg1, arg2, arg3, arg4);
        }
        // Yes these are dumb. Blame TypeScript. Yes I tried a generic, it didn't work.
        if (arg1 === 'entry') {
            return new ForOfStatement(this.sandstoneCore, arg1, arg3, arg4);
        }
        if (arg1[0] === 'key') {
            return new ForOfStatement(this.sandstoneCore, arg1, arg3, arg4);
        }
        if (typeof arg1[0] === 'number' || arg1[0] instanceof Score) {
            if (arg2 === 'iterate') {
                return new ForIStatement(this.sandstoneCore, arg1[0], (i) => i['<='](arg1[1]), (i) => i['++'], arg3);
            }
            return binaryFor(this, arg1[0], arg1[1], arg3, arg1[2]);
        }
        return new ForOfStatement(this.sandstoneCore, arg1, arg3, arg4);
    }
    switch(value, _cases, _default) {
        let cases;
        if (_cases instanceof CaseStatement) {
            cases = _cases['getCases']().map((c) => c['getValue']());
        }
        else if (Array.isArray(_cases)) {
            cases = _cases;
        }
        else {
            cases = _cases.cases.map((c) => c['getValue']());
            _default = ['default', _cases.default];
        }
        const { Data, initMCFunction, MCFunction, Macro, } = this.sandstoneCore.pack;
        // eslint-disable-next-line no-plusplus
        const id = switches++;
        const values = Data('storage', `__sandstone:switch_${id}`, 'Values');
        initMCFunction.push(() => values.set(cases.map(([_, v, callback], i) => {
            MCFunction(`__sandstone:switch_${id}_case_${i}`, [], () => callback());
            return { Value: v, Index: i };
        })));
        const flow = this;
        MCFunction(`__sandstone:switch_${id}`, [value], () => {
            const index = Data('storage', `__sandstone:switch_${id}`, 'Index');
            Macro.data.modify.storage(index.currentTarget, 'Index').set.from.storage(values.currentTarget, Macro `Values[{Value:${value}}}].Index`);
            const _if = flow.if(index, () => {
                MCFunction(`__sandstone:switch_${id}_inner`, [index], () => {
                    Macro.functionCmd(Macro `__sandstone:switch_${id}_case_${index}`);
                })();
            });
            if (_default)
                _if.else(() => _default?.[1]());
        })();
    }
    case(value, callback) {
        return new CaseStatement(value, callback);
    }
}
