import { ContainerNode } from '../core/index.js';
export class IfNode extends ContainerNode {
    condition;
    callback;
    nextFlowNode;
    _isElseIf = false;
    constructor(sandstoneCore, condition, callback, reset = false) {
        super(sandstoneCore);
        this.condition = condition;
        this.callback = callback;
        const currentNode = this.sandstoneCore.getCurrentMCFunctionOrThrow();
        if (reset) {
            currentNode.resource.push(() => sandstoneCore.pack.flowVariable.reset());
        }
        if (callback.toString() !== '() => {}') {
            // Generate the body of the If node.
            currentNode.enterContext(this);
            this.callback();
            currentNode.exitContext();
        }
    }
    getValue = () => {
        throw new Error('Minecraft does not support if statements. This must be postprocessed.');
    };
}
export class IfStatement {
    sandstoneCore;
    condition;
    callback;
    node;
    constructor(sandstoneCore, condition, callback) {
        this.sandstoneCore = sandstoneCore;
        this.condition = condition;
        this.callback = callback;
        // Generate the body of the If node.
        this.node = new IfNode(sandstoneCore, condition, callback);
    }
    elseIf = (condition, callback) => {
        const statement = new IfStatement(this.sandstoneCore, this.sandstoneCore.pack.flow.conditionToNode(condition), callback);
        this.node.nextFlowNode = statement['getNode']();
        statement.node['_isElseIf'] = true;
        return statement;
    };
    else = (callback) => {
        const statement = new ElseStatement(this.sandstoneCore, callback);
        this.node.nextFlowNode = statement['getNode']();
        return statement;
    };
    getNode = () => this.node;
}
export class ElseNode extends ContainerNode {
    callback;
    constructor(sandstoneCore, callback) {
        super(sandstoneCore);
        this.callback = callback;
        // Generate the body of the If node.
        this.sandstoneCore.getCurrentMCFunctionOrThrow().enterContext(this);
        this.callback();
        this.sandstoneCore.currentMCFunction?.exitContext();
    }
    getValue = () => null;
}
export class ElseStatement {
    sandstoneCore;
    callback;
    node;
    constructor(sandstoneCore, callback) {
        this.sandstoneCore = sandstoneCore;
        this.callback = callback;
        // Generate the body of the If node.
        this.node = new ElseNode(sandstoneCore, callback);
    }
    getNode = () => this.node;
}
