import { LoopArgument } from 'sandstone/variables';
import { IfStatement } from '../if_else.js';
import { LoopNode } from '../loop.js';
export class ForOfNode extends LoopNode {
    constructor(sandstoneCore, iteratorType, iterable, callback) {
        const _iterable = iterable._toDataPoint();
        let startCondition;
        let iterate;
        let continueCondition;
        if (Array.isArray(iteratorType)) {
            if (iteratorType[0] === 'key') {
                startCondition = ['if', 'data', _iterable.type, `${_iterable.path}{}`];
                iterate = iterable.iterator((dataPoints) => {
                    if (dataPoints.length === 1) {
                        throw new Error('Error: Attempt to use "for key value" loop with an Array dataset; this is not supported. (Hint: use "for entry" or "for i entry" instead)');
                    }
                    callback(...[dataPoints[0], dataPoints[1]]);
                });
            }
            else {
                startCondition = ['if', 'data', _iterable.type, `${_iterable.path}{}`];
                const _iterator = sandstoneCore.pack.Variable();
                _iterator.set(iterable.size()); // Store the size of the array to the iterator
                continueCondition = _iterator['!='](-1);
                iterate = iterable.iterator((dataPoints) => {
                    _iterator.decrease();
                    if (dataPoints.length === 1) {
                        return callback(...[_iterator, dataPoints[0]]);
                    }
                    return callback(...[_iterator, dataPoints[1]]);
                });
            }
        }
        else {
            startCondition = ['if', 'data', _iterable.type, `${_iterable.path}{}`];
            iterate = iterable.iterator((dataPoints) => {
                if (dataPoints.length === 1) {
                    return callback(...[dataPoints[0]]);
                }
                return callback(...[dataPoints[1]]);
            });
        }
        iterate();
        super(sandstoneCore, [startCondition], () => iterate, () => new IfStatement(sandstoneCore, continueCondition || iterable.continue, () => new LoopArgument(sandstoneCore.pack)));
    }
}
export class ForOfStatement {
    sandstoneCore;
    iteratorType;
    iterable;
    callback;
    node;
    constructor(sandstoneCore, iteratorType, iterable, callback) {
        this.sandstoneCore = sandstoneCore;
        this.iteratorType = iteratorType;
        this.iterable = iterable;
        this.callback = callback;
        this.node = new ForOfNode(sandstoneCore, iteratorType, iterable, callback);
    }
    getNode = () => this.node;
}
