"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
var pack_exports = {};
__export(pack_exports, {
  DataPack: () => DataPack,
  ResourcePack: () => ResourcePack,
  SandstonePack: () => SandstonePack
});
module.exports = __toCommonJS(pack_exports);
var import_commands = require("sandstone/commands");
var import_core = require("sandstone/core");
var import_custom = require("sandstone/core/resources/custom");
var import_flow = require("sandstone/flow");
var import_utils = require("sandstone/utils");
var import_variables = require("sandstone/variables");
var import_ResolveNBT = require("sandstone/variables/ResolveNBT");
var import_Score = require("sandstone/variables/Score");
var import_Sleep = require("sandstone/variables/Sleep");
var import_UUID = require("sandstone/variables/UUID");
var import_packType = require("./packType.js");
var import_addAwaitBodyToMCFunctions = require("./visitors/addAwaitBodyToMCFunctions.js");
var import_visitors = require("./visitors/index.js");
const conflictDefaults = (resourceType) => process.env[`${resourceType.toUpperCase()}S_CONFLICT_STRATEGY`] || process.env.DEFAULT_CONFLICT_STRATEGY;
let tempStorage;
let startTickedLoops;
const foo = {
  addToSandstoneCore: false
};
class DataPack extends import_packType.PackType {
  constructor(archiveOutput, options) {
    super("datapack", "saves/$worldName$/datapacks/$packName$", "world/datapacks/$packName$", "datapacks/$packName$", "server", archiveOutput, "data", true);
    // TODO: typing. low priority
    __publicField(this, "packMcmeta");
    __publicField(this, "handleOutput", async (type, readFile, writeFile) => {
      if (type === "output") {
        await writeFile("pack.mcmeta", JSON.stringify(this.packMcmeta));
      }
    });
    this.packMcmeta = {
      pack: {
        pack_format: options.packFormat,
        description: options.description
      }
    };
    if (options.features) {
      this.packMcmeta.features = { enabled: options.features };
    }
    if (options.filter) {
      this.packMcmeta.filter = { block: options.filter };
    }
  }
}
class ResourcePack extends import_packType.PackType {
  constructor(options) {
    super("resourcepack", "saves/$worldName$/resources", "resource_pack", "resourcepacks/$packName$", "client", true, "assets", true);
    // TODO: typing. low priority
    __publicField(this, "packMcmeta");
    __publicField(this, "handleOutput", async (type, readFile, writeFile) => {
      if (type === "output") {
        await writeFile("pack.mcmeta", JSON.stringify(this.packMcmeta));
      }
    });
    this.packMcmeta = {
      pack: {
        pack_format: options.packFormat,
        description: options.description
      }
    };
    if (options.filter) {
      this.packMcmeta.filter = { block: options.filter };
    }
  }
}
class SandstonePack {
  constructor(defaultNamespace, packUid) {
    this.defaultNamespace = defaultNamespace;
    this.packUid = packUid;
    __publicField(this, "core", new import_core.SandstoneCore(this));
    __publicField(this, "packTypes");
    __publicField(this, "packOptions", JSON.parse(process.env.PACK_OPTIONS));
    // Smithed Pack IDs
    __publicField(this, "dependencies");
    __publicField(this, "flow");
    __publicField(this, "commands");
    __publicField(this, "Macro");
    __publicField(this, "conditions", import_flow.SandstoneConditions);
    __publicField(this, "objectives");
    __publicField(this, "anonymousScoreId", 0);
    __publicField(this, "anonymousDataId", 0);
    __publicField(this, "constants");
    __publicField(this, "tickedLoops");
    __publicField(this, "loadTags");
    __publicField(this, "setupLantern", () => {
      const loadStatus = this.Objective.create("load.status");
      const privateInit = this.Tag("functions", "load:_private/init", [this.MCFunction("load:_private/init", () => {
        this.commands.comment("Reset scoreboards so packs can set values accurate for current load.");
        loadStatus.reset();
      })]);
      const privateLoad = this.Tag("functions", "load:_private/load", [
        privateInit,
        { id: this.loadTags.preLoad, required: false },
        { id: this.loadTags.load, required: false },
        { id: this.loadTags.postLoad, required: false }
      ]);
      this.Tag("functions", "minecraft:load", [privateLoad]);
      this.initMCFunction.push(() => loadStatus(this.defaultNamespace).set(process.env.LOAD_VERSION || 1));
    });
    __publicField(this, "resourceToPath", (name, resourceFolders) => {
      let namespace = this.defaultNamespace;
      let fullName = name;
      if (name.includes(":")) {
        [namespace, fullName] = name.split(":");
      }
      const fullPath = fullName.split("/");
      if (!resourceFolders) {
        return fullPath;
      }
      return [namespace, ...resourceFolders, ...fullPath];
    });
    __publicField(this, "registerNewObjective", (objective) => {
      this.objectives.add(objective);
    });
    __publicField(this, "Objective", {
      /**
       * Create a new objective. Defaults to `dummy` if unspecified.
       * @param name The name of the objective
       */
      create: (name, criteria = "dummy", display, alreadyExists) => {
        let namespace = false;
        if (name.includes(".") || name.includes("__")) {
          namespace = true;
        }
        const objective = new import_variables.ObjectiveClass(this, namespace ? name : `${this.defaultNamespace}.${name}`, criteria, display, { creator: "user" });
        if (!alreadyExists) {
          this.registerNewObjective(objective);
        }
        return objective;
      },
      /** Get an existing objective. */
      get: (name) => new import_variables.ObjectiveClass(this, name, "dummy", void 0, { creator: "user" })
    });
    __publicField(this, "__rootObjective");
    __publicField(this, "Variable", (...args) => {
      const score = this.rootObjective;
      if (args[0] instanceof import_variables.DataPointClass) {
        const [nbt, scale, name2] = args;
        return this.Variable(void 0, name2).set(nbt, scale);
      }
      const [initialValue, name] = args;
      const anonymousScore = score(`${name != null ? name : "anon"}_${this.packUid}_${this.anonymousScoreId++}`);
      if (initialValue === void 0) {
        return anonymousScore;
      }
      if (this.core.currentMCFunction) {
        return anonymousScore.set(initialValue);
      }
      this.initMCFunction.push(() => anonymousScore.set(initialValue));
      return anonymousScore;
    });
    /**
     * Creates a new label
     * @param label Label/tag name
     * @param description Label description (optional)
     */
    __publicField(this, "Label", (label, description) => new import_variables.LabelClass(this, label, description));
    __publicField(this, "__rootStorage");
    __publicField(this, "DataVariable", (...args) => {
      const data = this.rootStorage();
      if (args[0] instanceof import_Score.Score) {
        const [score, storeType, scale, name2] = args;
        return this.DataVariable(void 0, name2).set(score, storeType || "int", scale || 1);
      }
      const [initialValue, name] = args;
      const anonymousData = data.select(`${name != null ? name : "anon"}_${this.packUid}_${this.anonymousDataId++}`);
      if (initialValue === void 0) {
        return anonymousData;
      }
      if (this.core.currentMCFunction) {
        return anonymousData.set(initialValue);
      }
      this.initMCFunction.push(() => anonymousData.set(initialValue));
      return anonymousData;
    });
    __publicField(this, "getTempStorage", (name) => {
      if (tempStorage) {
        return tempStorage.select(name);
      }
      tempStorage = this.Data("storage", "__sandstone:temp");
      return tempStorage.select(name);
    });
    /**
     * Resolve NBT containing Data Points & Score's.
     * @param nbt NBT with Data Points & Scores initialized with ResolveNBTPart.
     * @param dataPoint Optional. Data Point to resolve to.
     */
    __publicField(this, "ResolveNBT", (nbt, dataPoint) => new import_ResolveNBT.ResolveNBTClass(this, nbt, dataPoint));
    __publicField(this, "Selector", (target, properties) => new import_variables.SelectorClass(this, target, properties));
    __publicField(this, "appendNode", (node) => this.core.getCurrentMCFunctionOrThrow().appendNode(node));
    __publicField(this, "__initMCFunction");
    __publicField(this, "sleep", (delay) => new import_Sleep.SleepClass(this.core, delay).promise());
    __publicField(this, "Loop", () => new import_variables.LoopArgument(this));
    __publicField(this, "__customResourceTypes", []);
    __publicField(this, "Advancement", (name, advancement, options) => new import_core.AdvancementClass(this.core, name, {
      advancement,
      creator: "user",
      addToSandstoneCore: true,
      onConflict: conflictDefaults("advancement"),
      ...options
    }));
    __publicField(this, "DamageType", (name, damageType, options) => new import_core.DamageTypeClass(this.core, name, {
      damageType,
      creator: "user",
      addToSandstoneCore: true,
      onConflict: conflictDefaults("damage_type"),
      ...options
    }));
    __publicField(this, "ItemModifier", (name, itemModifier, options) => new import_core.ItemModifierClass(this.core, name, {
      itemModifier,
      creator: "user",
      addToSandstoneCore: true,
      onConflict: conflictDefaults("item_modifier"),
      ...options
    }));
    __publicField(this, "LootTable", (name, lootTable, options) => new import_core.LootTableClass(this.core, name, {
      lootTable,
      creator: "user",
      addToSandstoneCore: true,
      onConflict: conflictDefaults("loot_table"),
      ...options
    }));
    __publicField(this, "Predicate", (name, predicate, options) => new import_core.PredicateClass(this.core, name, {
      predicate,
      creator: "user",
      addToSandstoneCore: true,
      onConflict: conflictDefaults("predicate"),
      ...options
    }));
    __publicField(this, "Recipe", (name, recipe, options) => new import_core.RecipeClass(this.core, name, {
      recipe,
      creator: "user",
      addToSandstoneCore: true,
      onConflict: conflictDefaults("recipe"),
      ...options
    }));
    /** @ts-ignore */
    __publicField(this, "Tag", (type, name, values, options) => new import_core.TagClass(this.core, type, name, {
      values,
      creator: "user",
      addToSandstoneCore: true,
      onConflict: conflictDefaults("tag"),
      ...options
    }));
    __publicField(this, "TrimMaterial", (name, trimMaterial, options) => new import_core.TrimMaterialClass(this.core, name, {
      trimMaterial,
      creator: "user",
      addToSandstoneCore: true,
      onConflict: conflictDefaults("trim_material"),
      ...options
    }));
    __publicField(this, "TrimPattern", (name, trimPattern, options) => new import_core.TrimPatternClass(this.core, name, {
      trimPattern,
      creator: "user",
      addToSandstoneCore: true,
      onConflict: conflictDefaults("trim_pattern"),
      ...options
    }));
    __publicField(this, "Atlas", (name, atlas, options) => new import_core.AtlasClass(this.core, name, {
      atlas,
      creator: "user",
      addToSandstoneCore: true,
      onConflict: conflictDefaults("atlas"),
      ...options
    }));
    __publicField(this, "Font", (name, providers, options) => new import_core.FontClass(this.core, name, {
      providers,
      creator: "user",
      addToSandstoneCore: true,
      onConflict: conflictDefaults("font"),
      ...options
    }));
    __publicField(this, "Language", (name, language, options) => new import_core.LanguageClass(this.core, name, {
      language,
      creator: "user",
      addToSandstoneCore: true,
      onConflict: conflictDefaults("language"),
      ...options
    }));
    __publicField(this, "Model", (type, name, model, options) => new import_core.ModelClass(this.core, type, name, {
      model,
      creator: "user",
      addToSandstoneCore: true,
      onConflict: conflictDefaults("model"),
      ...options
    }));
    __publicField(this, "PlainText", (name, text, options) => new import_core.PlainTextClass(this.core, name, {
      text,
      creator: "user",
      addToSandstoneCore: true,
      onConflict: conflictDefaults("atlas"),
      ...options
    }));
    __publicField(this, "save", async (cliOptions) => {
      await this.core.save(cliOptions, {
        visitors: [
          // Initialization visitors
          new import_visitors.InitObjectivesVisitor(this),
          new import_visitors.InitConstantsVisitor(this),
          new import_visitors.GenerateLazyMCFunction(this),
          // Transformation visitors
          new import_visitors.LoopTransformationVisitor(this),
          new import_visitors.IfElseTransformationVisitor(this),
          new import_visitors.ContainerCommandsToMCFunctionVisitor(this),
          // Special visitors
          new import_addAwaitBodyToMCFunctions.AwaitBodyVisitor(this),
          // Optimization
          new import_visitors.InlineFunctionCallVisitor(this),
          new import_visitors.UnifyChainedExecutesVisitor(this),
          new import_visitors.SimplifyExecuteFunctionVisitor(this),
          new import_visitors.SimplifyReturnRunFunctionVisitor(this)
        ]
      });
      return this.packTypes;
    });
    this.packTypes = /* @__PURE__ */ new Map();
    this.commands = new import_commands.SandstoneCommands(this);
    this.Macro = (0, import_utils.makeCallable)(this.commands, (strings, ...macros) => new import_core.MacroLiteral(this.core, strings, macros), true);
    this.flow = new import_flow.Flow(this.core);
    this.objectives = /* @__PURE__ */ new Set();
    this.constants = /* @__PURE__ */ new Set();
    this.tickedLoops = {};
    if (process.env.NAMESPACE) {
      this.defaultNamespace = process.env.NAMESPACE;
    }
    this.loadTags = {
      preLoad: this.Tag("functions", "load:pre_load", []),
      load: this.Tag("functions", "load:load", []),
      postLoad: this.Tag("functions", "load:post_load", [])
    };
    this.setupLantern();
    this.dependencies = /* @__PURE__ */ new Map();
    for (const method of Object.getOwnPropertyNames(Object.getPrototypeOf(this))) {
      if (method !== "constructor" && typeof this[method] === "function" && typeof this[method].bind === "function") {
        this[method] = this[method].bind(this);
      }
    }
  }
  dataPack() {
    let pack = this.packTypes.get("datapack");
    if (!pack) {
      pack = this.packTypes.set("datapack", new DataPack(false, this.packOptions.datapack)).get("datapack");
    }
    return pack;
  }
  resourcePack() {
    let pack = this.packTypes.get("resourcepack");
    if (!pack) {
      pack = this.packTypes.set("resourcepack", new ResourcePack(this.packOptions.resourcepack)).get("resourcepack");
    }
    return pack;
  }
  /** UTILS */
  get _() {
    return new import_flow.Flow(this.core);
  }
  /**
   * Register a new numeric constant.
   */
  registerNewConstant(amount) {
    this.constants.add(amount);
  }
  get rootObjective() {
    if (this.__rootObjective) {
      return this.__rootObjective;
    }
    this.__rootObjective = this.Objective.create("__sandstone", "dummy", [{ text: "Sandstone", color: "gold" }, " internals"]);
    return this.__rootObjective;
  }
  get flowVariable() {
    return this.rootObjective("if_result");
  }
  Data(type, target, path) {
    const dataPath = (path != null ? path : typeof path === "string") ? [path] : path;
    if (!path && !target) {
      return new import_variables.TargetlessDataClass(this, type);
    }
    if (!target) {
      return new import_variables.TargetlessDataPointClass(this, type, dataPath);
    }
    if (!path) {
      return new import_variables.DataClass(this, type, type === "block" ? (0, import_variables.coordinatesParser)(target) : target);
    }
    return new import_variables.DataPointClass(this, type, target instanceof import_variables.VectorClass ? (0, import_variables.coordinatesParser)(target) : target, dataPath);
  }
  rootStorage() {
    if (!this.__rootStorage) {
      this.__rootStorage = new import_variables.DataPointClass(this, "storage", "__sandstone:variable", []);
      return this.__rootStorage;
    }
    return this.__rootStorage;
  }
  DataIndexMap(initialize, dataPoint) {
    return (0, import_variables.DataIndexMap)(this, initialize, dataPoint);
  }
  DataArray(initialize, dataPoint) {
    return (0, import_variables.DataArray)(this, initialize, dataPoint);
  }
  UUID(source = (0, import_utils.randomUUID)(), options) {
    return new import_UUID.UUIDClass(this.core, source, options);
  }
  MCFunction(...args) {
    return new import_core.MCFunctionClass(this.core, args[0], {
      callback: typeof args[1] === "function" ? args[1] : args[2],
      creator: "user",
      addToSandstoneCore: true,
      onConflict: conflictDefaults("mcfunction"),
      ...typeof args[1] === "function" ? args[2] : args[3]
    }, typeof args[1] !== "function" && Array.isArray(args[1]) ? args[1] : void 0);
  }
  get initMCFunction() {
    if (!this.__initMCFunction) {
      this.__initMCFunction = new import_core.MCFunctionClass(this.core, `${this.defaultNamespace}:__init__`, {
        addToSandstoneCore: true,
        creator: "sandstone"
      });
      this.loadTags.load.push(this.__initMCFunction);
      return this.__initMCFunction;
    }
    return this.__initMCFunction;
  }
  /**
   * Register commands that will be ticked at the rate you specify
   */
  registerTickedCommands(runEvery, callback) {
    if (this.tickedLoops[runEvery]) {
      this.tickedLoops[runEvery].push(callback);
    } else {
      this.tickedLoops[runEvery] = this.MCFunction(`__sandstone:ticked/times/${runEvery}`, () => {
        this.tickedLoops[runEvery].schedule.function(runEvery, "replace");
        callback();
      });
      if (!startTickedLoops) {
        startTickedLoops = this.MCFunction("__sandstone:ticked/start", () => this.tickedLoops[runEvery].schedule.function(runEvery, "replace"));
        this.loadTags.load.push(startTickedLoops);
      } else {
        startTickedLoops.push(() => this.tickedLoops[runEvery].schedule.function(runEvery, "replace"));
      }
    }
  }
  registerCustomResource(type) {
    if (this.__customResourceTypes.includes(type)) {
      throw new Error(`A custom resource of the type '${type}' has already been created!`);
    }
    this.__customResourceTypes.push(type);
  }
  get makeCustomResource() {
    return [this.core, import_custom.CustomResourceClass];
  }
  RawResource(...args) {
    const [core, CustomResource] = this.makeCustomResource;
    if (args[0] instanceof import_packType.PackType) {
      const _path2 = args[1];
      const path2 = _path2.includes("/") ? _path2.split("/") : [_path2];
      const extension2 = path2[path2.length - 1].split(".")[1];
      path2[path2.length - 1] = path2[path2.length - 1].replace(`.${extension2}`, "");
      class RawResource2 extends CustomResource {
        constructor() {
          super(core, path2.join("/"), {
            type: `${Math.random()}`,
            packType: args[0],
            extension: extension2,
            addToSandstoneCore: true,
            creator: "user"
          });
          __publicField(this, "getValue", () => args[2]);
        }
      }
      return new RawResource2();
    }
    const _path = args[0];
    const path = _path.includes("/") ? _path.split("/") : [_path];
    const extension = path[path.length - 1].split(".")[1];
    path[path.length - 1] = path[path.length - 1].replace(`.${extension}`, "");
    const { dataPack } = this;
    class RawResource extends CustomResource {
      constructor() {
        super(core, path.join("/"), {
          type: `${Math.random()}`,
          extension,
          addToSandstoneCore: true,
          creator: "user"
        });
        __publicField(this, "getValue", () => args[1]);
      }
    }
    return new RawResource();
  }
  BlockState(type, name, blockState, options) {
    return new import_core.BlockStateClass(this.core, name, type, {
      blockState,
      creator: "user",
      addToSandstoneCore: true,
      onConflict: conflictDefaults("blockstate"),
      ...options
    });
  }
  SoundEvent(type, name, sound, options) {
    return new import_core.SoundEventClass(this.core, type, name, {
      sound,
      creator: "user",
      addToSandstoneCore: true,
      onConflict: conflictDefaults("sound_event"),
      ...options
    });
  }
  Texture(type, name, texture, options) {
    return new import_core.TextureClass(this.core, type, name, {
      texture,
      creator: "user",
      addToSandstoneCore: true,
      onConflict: conflictDefaults("sound_event"),
      ...options
    });
  }
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  DataPack,
  ResourcePack,
  SandstonePack
});
//# sourceMappingURL=pack.cjs.map