/* eslint-disable max-len */
/* eslint-disable no-plusplus */
import { SandstoneCommands } from 'sandstone/commands';
import { AdvancementClass, AtlasClass, BlockStateClass, DamageTypeClass, FontClass, ItemModifierClass, LanguageClass, LootTableClass, MacroLiteral, MCFunctionClass, ModelClass, PlainTextClass, PredicateClass, RecipeClass, SandstoneCore, SoundEventClass, TagClass, TextureClass, TrimMaterialClass, TrimPatternClass, } from 'sandstone/core';
import { CustomResourceClass } from 'sandstone/core/resources/custom';
import { Flow, SandstoneConditions } from 'sandstone/flow';
import { makeCallable, randomUUID } from 'sandstone/utils';
import { coordinatesParser, DataArray, DataClass, DataIndexMap, DataPointClass, LabelClass, LoopArgument, ObjectiveClass, SelectorClass, TargetlessDataClass, TargetlessDataPointClass, VectorClass, } from 'sandstone/variables';
import { ResolveNBTClass } from 'sandstone/variables/ResolveNBT';
import { Score } from 'sandstone/variables/Score';
import { SleepClass } from 'sandstone/variables/Sleep';
import { UUIDClass } from 'sandstone/variables/UUID';
import { PackType } from './packType.js';
import { AwaitBodyVisitor } from './visitors/addAwaitBodyToMCFunctions.js';
import { ContainerCommandsToMCFunctionVisitor, GenerateLazyMCFunction, IfElseTransformationVisitor, InitConstantsVisitor, InitObjectivesVisitor, InlineFunctionCallVisitor, LoopTransformationVisitor, SimplifyExecuteFunctionVisitor, SimplifyReturnRunFunctionVisitor, UnifyChainedExecutesVisitor, } from './visitors/index.js';
const conflictDefaults = (resourceType) => (process.env[`${resourceType.toUpperCase()}S_CONFLICT_STRATEGY`] || process.env.DEFAULT_CONFLICT_STRATEGY);
let tempStorage;
let startTickedLoops;
const foo = {
    addToSandstoneCore: false,
};
export class DataPack extends PackType {
    // TODO: typing. low priority
    packMcmeta;
    constructor(archiveOutput, options) {
        super('datapack', 'saves/$worldName$/datapacks/$packName$', 'world/datapacks/$packName$', 'datapacks/$packName$', 'server', archiveOutput, 'data', true);
        this.packMcmeta = {
            pack: {
                pack_format: options.packFormat,
                description: options.description,
            },
        };
        if (options.features) {
            this.packMcmeta.features = { enabled: options.features };
        }
        if (options.filter) {
            this.packMcmeta.filter = { block: options.filter };
        }
    }
    handleOutput = async (type, readFile, writeFile) => {
        if (type === 'output') {
            await writeFile('pack.mcmeta', JSON.stringify(this.packMcmeta));
        }
    };
}
export class ResourcePack extends PackType {
    // TODO: typing. low priority
    packMcmeta;
    constructor(options) {
        super('resourcepack', 'saves/$worldName$/resources', 'resource_pack', 'resourcepacks/$packName$', 'client', true, 'assets', true);
        this.packMcmeta = {
            pack: {
                pack_format: options.packFormat,
                description: options.description,
            },
        };
        if (options.filter) {
            this.packMcmeta.filter = { block: options.filter };
        }
    }
    handleOutput = async (type, readFile, writeFile) => {
        if (type === 'output') {
            await writeFile('pack.mcmeta', JSON.stringify(this.packMcmeta));
        }
        /*
         * TODO: language
         *
         * language: Contains additional languages to add to the language menu
         * Language code for a language, corresponding to a .json file with the same name in the folder assets/<namespace>/lang.
         *  name: The full name of the language
         *  region: The country or region name
         *  bidirectional: If true, the language reads right to left.
         */
    };
}
export class SandstonePack {
    defaultNamespace;
    packUid;
    core = new SandstoneCore(this);
    packTypes;
    packOptions = JSON.parse(process.env.PACK_OPTIONS);
    dataPack() {
        let pack = this.packTypes.get('datapack');
        if (!pack) {
            pack = this.packTypes.set('datapack', new DataPack(false, this.packOptions.datapack)).get('datapack');
        }
        return pack;
    }
    resourcePack() {
        let pack = this.packTypes.get('resourcepack');
        if (!pack) {
            pack = this.packTypes.set('resourcepack', new ResourcePack(this.packOptions.resourcepack)).get('resourcepack');
        }
        return pack;
    }
    // Smithed Pack IDs
    dependencies;
    flow;
    commands;
    Macro;
    conditions = SandstoneConditions;
    objectives;
    anonymousScoreId = 0;
    anonymousDataId = 0;
    constants;
    tickedLoops;
    loadTags;
    constructor(defaultNamespace, packUid) {
        this.defaultNamespace = defaultNamespace;
        this.packUid = packUid;
        this.packTypes = new Map();
        this.commands = new SandstoneCommands(this);
        // SandstonePack.Macro is only a type hack
        this.Macro = makeCallable(this.commands, (strings, ...macros) => new MacroLiteral(this.core, strings, macros), true);
        this.flow = new Flow(this.core);
        this.objectives = new Set();
        this.constants = new Set();
        this.tickedLoops = {};
        if (process.env.NAMESPACE) {
            this.defaultNamespace = process.env.NAMESPACE;
        }
        this.loadTags = {
            preLoad: this.Tag('functions', 'load:pre_load', []),
            load: this.Tag('functions', 'load:load', []),
            postLoad: this.Tag('functions', 'load:post_load', []),
        };
        this.setupLantern();
        this.dependencies = new Map();
        // ESM is funny
        for (const method of Object.getOwnPropertyNames(Object.getPrototypeOf(this))) {
            /* @ts-ignore */
            if (method !== 'constructor' && typeof this[method] === 'function' && typeof this[method].bind === 'function') {
                /* @ts-ignore */
                this[method] = this[method].bind(this);
            }
        }
    }
    setupLantern = () => {
        const loadStatus = this.Objective.create('load.status');
        const privateInit = this.Tag('functions', 'load:_private/init', [this.MCFunction('load:_private/init', () => {
                this.commands.comment('Reset scoreboards so packs can set values accurate for current load.');
                loadStatus.reset();
            })]);
        const privateLoad = this.Tag('functions', 'load:_private/load', [
            privateInit,
            { id: this.loadTags.preLoad, required: false },
            { id: this.loadTags.load, required: false },
            { id: this.loadTags.postLoad, required: false },
        ]);
        this.Tag('functions', 'minecraft:load', [privateLoad]);
        this.initMCFunction.push(() => loadStatus(this.defaultNamespace).set(process.env.LOAD_VERSION || 1));
    };
    resourceToPath = (name, resourceFolders) => {
        let namespace = this.defaultNamespace;
        let fullName = name;
        if (name.includes(':')) {
            ([namespace, fullName] = name.split(':'));
        }
        const fullPath = fullName.split('/');
        if (!resourceFolders) {
            return fullPath;
        }
        return [namespace, ...resourceFolders, ...fullPath];
    };
    /** UTILS */
    get _() {
        return new Flow(this.core);
    }
    registerNewObjective = (objective) => {
        this.objectives.add(objective);
    };
    /**
     * Register a new numeric constant.
     */
    registerNewConstant(amount) {
        this.constants.add(amount);
    }
    Objective = {
        /**
         * Create a new objective. Defaults to `dummy` if unspecified.
         * @param name The name of the objective
         */
        create: (name, criteria = 'dummy', display, alreadyExists) => {
            let namespace = false;
            if (name.includes('.') || name.includes('__')) {
                namespace = true;
            }
            const objective = new ObjectiveClass(this, namespace ? name : `${this.defaultNamespace}.${name}`, criteria, display, { creator: 'user' });
            if (!alreadyExists) {
                this.registerNewObjective(objective);
            }
            return objective;
        },
        /** Get an existing objective. */
        get: (name) => new ObjectiveClass(this, name, 'dummy', undefined, { creator: 'user' }),
    };
    __rootObjective;
    get rootObjective() {
        if (this.__rootObjective) {
            return this.__rootObjective;
        }
        this.__rootObjective = this.Objective.create('__sandstone', 'dummy', [{ text: 'Sandstone', color: 'gold' }, ' internals']);
        return this.__rootObjective;
    }
    Variable = (...args) => {
        // Get the objective
        const score = this.rootObjective;
        if (args[0] instanceof DataPointClass) {
            const [nbt, scale, name] = args;
            // If the value is a data point, leverage the .set method
            return this.Variable(undefined, name).set(nbt, scale);
        }
        const [initialValue, name] = args;
        // Get the specific anonymous score
        const anonymousScore = score(`${name ?? 'anon'}_${this.packUid}_${this.anonymousScoreId++}`);
        // No initial value => we can directly return the score
        if (initialValue === undefined) {
            return anonymousScore;
        }
        // We currently are in a MCFunction => that's where the initialization should take place
        if (this.core.currentMCFunction) {
            return anonymousScore.set(initialValue);
        }
        // Else, we should run it in the init MCFunction
        this.initMCFunction.push(() => anonymousScore.set(initialValue));
        return anonymousScore;
    };
    get flowVariable() {
        return this.rootObjective('if_result');
    }
    /**
     * Creates a new label
     * @param label Label/tag name
     * @param description Label description (optional)
     */
    Label = (label, description) => new LabelClass(this, label, description);
    Data(type, target, path) {
        const dataPath = path ?? typeof path === 'string' ? [path] : path;
        if (!path && !target) {
            return new TargetlessDataClass(this, type);
        }
        if (!target) {
            return new TargetlessDataPointClass(this, type, dataPath);
        }
        if (!path) {
            return new DataClass(this, type, (type === 'block' ? coordinatesParser(target) : target));
        }
        return new DataPointClass(this, type, (target instanceof VectorClass ? coordinatesParser(target) : target), dataPath);
    }
    __rootStorage;
    rootStorage() {
        if (!this.__rootStorage) {
            this.__rootStorage = new DataPointClass(this, 'storage', '__sandstone:variable', []);
            return this.__rootStorage;
        }
        return this.__rootStorage;
    }
    DataVariable = (...args) => {
        // Get the objective
        const data = this.rootStorage();
        if (args[0] instanceof Score) {
            const [score, storeType, scale, name] = args;
            // If the value is a score, leverage the .set method
            return this.DataVariable(undefined, name).set(score, (storeType || 'int'), scale || 1);
        }
        const [initialValue, name] = args;
        // Get the specific anonymous data point
        const anonymousData = data.select(`${name ?? 'anon'}_${this.packUid}_${this.anonymousDataId++}`);
        // No initial value => we can directly return the data point
        if (initialValue === undefined) {
            return anonymousData;
        }
        // We currently are in a MCFunction => that's where the initialization should take place
        if (this.core.currentMCFunction) {
            return anonymousData.set(initialValue);
        }
        // Else, we should run it in the init MCFunction
        this.initMCFunction.push(() => anonymousData.set(initialValue));
        return anonymousData;
    };
    DataIndexMap(initialize, dataPoint) {
        return DataIndexMap(this, initialize, dataPoint);
    }
    DataArray(initialize, dataPoint) {
        return DataArray(this, initialize, dataPoint);
    }
    getTempStorage = (name) => {
        if (tempStorage) {
            return tempStorage.select(name);
        }
        tempStorage = this.Data('storage', '__sandstone:temp');
        return tempStorage.select(name);
    };
    /**
     * Resolve NBT containing Data Points & Score's.
     * @param nbt NBT with Data Points & Scores initialized with ResolveNBTPart.
     * @param dataPoint Optional. Data Point to resolve to.
     */
    ResolveNBT = (nbt, dataPoint) => new ResolveNBTClass(this, nbt, dataPoint);
    Selector = ((target, properties) => new SelectorClass(this, target, properties));
    UUID(source = randomUUID(), options) { return new UUIDClass(this.core, source, options); }
    MCFunction(...args) {
        return new MCFunctionClass(this.core, args[0], {
            callback: (typeof args[1] === 'function' ? args[1] : args[2]),
            creator: 'user',
            addToSandstoneCore: true,
            onConflict: conflictDefaults('mcfunction'),
            ...(typeof args[1] === 'function' ? args[2] : args[3]),
        }, (typeof args[1] !== 'function' && Array.isArray(args[1])) ? args[1] : undefined);
    }
    appendNode = (node) => this.core.getCurrentMCFunctionOrThrow().appendNode(node);
    __initMCFunction;
    get initMCFunction() {
        if (!this.__initMCFunction) {
            this.__initMCFunction = new MCFunctionClass(this.core, `${this.defaultNamespace}:__init__`, {
                addToSandstoneCore: true,
                creator: 'sandstone',
            });
            this.loadTags.load.push(this.__initMCFunction);
            return this.__initMCFunction;
        }
        return this.__initMCFunction;
    }
    /**
     * Register commands that will be ticked at the rate you specify
     */
    registerTickedCommands(runEvery, callback) {
        if (this.tickedLoops[runEvery]) {
            this.tickedLoops[runEvery].push(callback);
        }
        else {
            this.tickedLoops[runEvery] = this.MCFunction(`__sandstone:ticked/times/${runEvery}`, () => {
                this.tickedLoops[runEvery].schedule.function(runEvery, 'replace');
                callback();
            });
            if (!startTickedLoops) {
                startTickedLoops = this.MCFunction('__sandstone:ticked/start', () => this.tickedLoops[runEvery].schedule.function(runEvery, 'replace'));
                this.loadTags.load.push(startTickedLoops);
            }
            else {
                startTickedLoops.push(() => this.tickedLoops[runEvery].schedule.function(runEvery, 'replace'));
            }
        }
    }
    sleep = (delay) => (new SleepClass(this.core, delay)).promise();
    Loop = () => new LoopArgument(this);
    __customResourceTypes = [];
    registerCustomResource(type) {
        if (this.__customResourceTypes.includes(type)) {
            throw new Error(`A custom resource of the type '${type}' has already been created!`);
        }
        this.__customResourceTypes.push(type);
    }
    get makeCustomResource() {
        return [this.core, CustomResourceClass];
    }
    RawResource(...args) {
        const [core, CustomResource] = this.makeCustomResource;
        if (args[0] instanceof PackType) {
            const _path = args[1];
            const path = _path.includes('/') ? _path.split('/') : [_path];
            const extension = path[path.length - 1].split('.')[1];
            path[path.length - 1] = path[path.length - 1].replace(`.${extension}`, '');
            class RawResource extends CustomResource {
                constructor() {
                    super(core, path.join('/'), {
                        type: `${Math.random()}`, packType: args[0], extension, addToSandstoneCore: true, creator: 'user',
                    });
                }
                getValue = () => args[2];
            }
            return new RawResource();
        }
        const _path = args[0];
        const path = _path.includes('/') ? _path.split('/') : [_path];
        const extension = path[path.length - 1].split('.')[1];
        path[path.length - 1] = path[path.length - 1].replace(`.${extension}`, '');
        const { dataPack } = this;
        class RawResource extends CustomResource {
            constructor() {
                super(core, path.join('/'), {
                    type: `${Math.random()}`, extension, addToSandstoneCore: true, creator: 'user',
                });
            }
            getValue = () => args[1];
        }
        return new RawResource();
    }
    Advancement = (name, advancement, options) => new AdvancementClass(this.core, name, {
        advancement,
        creator: 'user',
        addToSandstoneCore: true,
        onConflict: conflictDefaults('advancement'),
        ...options,
    });
    DamageType = (name, damageType, options) => new DamageTypeClass(this.core, name, {
        damageType,
        creator: 'user',
        addToSandstoneCore: true,
        onConflict: conflictDefaults('damage_type'),
        ...options,
    });
    ItemModifier = (name, itemModifier, options) => new ItemModifierClass(this.core, name, {
        itemModifier,
        creator: 'user',
        addToSandstoneCore: true,
        onConflict: conflictDefaults('item_modifier'),
        ...options,
    });
    LootTable = (name, lootTable, options) => new LootTableClass(this.core, name, {
        lootTable,
        creator: 'user',
        addToSandstoneCore: true,
        onConflict: conflictDefaults('loot_table'),
        ...options,
    });
    Predicate = (name, predicate, options) => new PredicateClass(this.core, name, {
        predicate,
        creator: 'user',
        addToSandstoneCore: true,
        onConflict: conflictDefaults('predicate'),
        ...options,
    });
    Recipe = (name, recipe, options) => new RecipeClass(this.core, name, {
        recipe,
        creator: 'user',
        addToSandstoneCore: true,
        onConflict: conflictDefaults('recipe'),
        ...options,
    });
    /** @ts-ignore */
    Tag = (type, name, values, options) => new TagClass(this.core, type, name, {
        values,
        creator: 'user',
        addToSandstoneCore: true,
        onConflict: conflictDefaults('tag'),
        ...options,
    });
    TrimMaterial = (name, trimMaterial, options) => new TrimMaterialClass(this.core, name, {
        trimMaterial,
        creator: 'user',
        addToSandstoneCore: true,
        onConflict: conflictDefaults('trim_material'),
        ...options,
    });
    TrimPattern = (name, trimPattern, options) => new TrimPatternClass(this.core, name, {
        trimPattern,
        creator: 'user',
        addToSandstoneCore: true,
        onConflict: conflictDefaults('trim_pattern'),
        ...options,
    });
    Atlas = (name, atlas, options) => new AtlasClass(this.core, name, {
        atlas,
        creator: 'user',
        addToSandstoneCore: true,
        onConflict: conflictDefaults('atlas'),
        ...options,
    });
    BlockState(type, name, blockState, options) {
        return new BlockStateClass(this.core, name, type, {
            blockState,
            creator: 'user',
            addToSandstoneCore: true,
            onConflict: conflictDefaults('blockstate'),
            ...options,
        });
    }
    Font = (name, providers, options) => new FontClass(this.core, name, {
        providers,
        creator: 'user',
        addToSandstoneCore: true,
        onConflict: conflictDefaults('font'),
        ...options,
    });
    Language = (name, language, options) => new LanguageClass(this.core, name, {
        language,
        creator: 'user',
        addToSandstoneCore: true,
        onConflict: conflictDefaults('language'),
        ...options,
    });
    Model = (type, name, model, options) => new ModelClass(this.core, type, name, {
        model,
        creator: 'user',
        addToSandstoneCore: true,
        onConflict: conflictDefaults('model'),
        ...options,
    });
    SoundEvent(type, name, sound, options) {
        return new SoundEventClass(this.core, type, name, {
            sound,
            creator: 'user',
            addToSandstoneCore: true,
            onConflict: conflictDefaults('sound_event'),
            ...options,
        });
    }
    PlainText = (name, text, options) => new PlainTextClass(this.core, name, {
        text,
        creator: 'user',
        addToSandstoneCore: true,
        onConflict: conflictDefaults('atlas'),
        ...options,
    });
    Texture(type, name, texture, options) {
        return new TextureClass(this.core, type, name, {
            texture,
            creator: 'user',
            addToSandstoneCore: true,
            onConflict: conflictDefaults('sound_event'),
            ...options,
        });
    }
    save = async (cliOptions) => {
        await this.core.save(cliOptions, {
            visitors: [
                // Initialization visitors
                new InitObjectivesVisitor(this),
                new InitConstantsVisitor(this),
                new GenerateLazyMCFunction(this),
                // Transformation visitors
                new LoopTransformationVisitor(this),
                new IfElseTransformationVisitor(this),
                new ContainerCommandsToMCFunctionVisitor(this),
                // Special visitors
                new AwaitBodyVisitor(this),
                // Optimization
                new InlineFunctionCallVisitor(this),
                new UnifyChainedExecutesVisitor(this),
                new SimplifyExecuteFunctionVisitor(this),
                new SimplifyReturnRunFunctionVisitor(this),
            ],
        });
        return this.packTypes;
    };
}
