import fs from 'fs-extra';
import type { Response } from 'node-fetch';
import type { Static } from '@sinclair/typebox';
import type { MultipleEntitiesArgument } from './arguments/selector.js';
import type { UUIDinNumber } from './variables/UUID.js';
export declare const fetch: (input: URL | RequestInfo, init?: RequestInit | undefined) => Promise<Response>;
/**
 * Allows to get autocompletion on string unions, while still allowing generic strings.
 * @see https://github.com/microsoft/TypeScript/issues/29729#issuecomment-700527227
 */
export type LiteralUnion<T extends string> = T | string & Record<never, never>;
export type AtLeastOne<T> = [T, ...T[]];
export type HideFunctionProperties<T extends Function> = T & {
    /** @deprecated */
    call: T['call'];
    /** @deprecated */
    apply: T['apply'];
    /** @deprecated */
    bind: T['bind'];
    /** @deprecated */
    prototype: T['prototype'];
    /** @deprecated */
    length: T['length'];
    /** @deprecated */
    arguments: T['arguments'];
    /** @deprecated */
    caller: T['caller'];
    /** @deprecated */
    readonly name: T['name'];
    /** @deprecated */
    [Symbol.hasInstance]: (value: unknown) => boolean;
};
export type BASIC_CONFLICT_STRATEGIES = 'throw' | 'replace' | 'warn' | 'rename' | 'ignore';
export type OmitFirst<T extends unknown[]> = (T extends [infer A, ...infer B] ? B : []);
export type InstanceOfClass<X extends (abstract new (...args: any[]) => any)> = ((new (..._args: [...one: ConstructorParameters<X>, ...two: any[]]) => InstanceType<X>));
export declare function isAsyncFunction(func: ((...args: any[]) => void) | ((...args: any[]) => Promise<void>)): func is (...args: any[]) => Promise<void>;
export declare function isPromise(promise: any): promise is Promise<unknown>;
export type Either<A extends Record<string, any>, B extends Record<string, any>> = {
    [K in (keyof A | keyof B)]?: K extends keyof A ? A[K] : never;
} | {
    [K in (keyof A | keyof B)]?: K extends keyof B ? B[K] : never;
};
export type WithMCNamespace<T extends string> = `minecraft:${T}` | T;
export declare function makeCallableProxy(func: any, object: any): any;
export interface CallableInstance {
    __call__: (...args: any[]) => any;
}
type CallableClass = new (...args: any[]) => CallableInstance;
export declare function makeClassCallable<C extends CallableClass>(Class_: C): MakeClassCallable<C>;
export type MakeInstanceCallable<T extends CallableInstance> = T & T['__call__'];
export type MakeClassCallable<T extends CallableClass> = new (...args: ConstructorParameters<T>) => MakeInstanceCallable<InstanceType<T>>;
export declare function makeCallable<T, F extends ((...args: any[]) => any)>(object: T, func: F, useProxy?: boolean): T & F;
type TupleSplit<T, N extends number, O extends readonly any[] = readonly []> = O['length'] extends N ? [O, T] : T extends readonly [infer F, ...infer R] ? TupleSplit<readonly [...R], N, readonly [...O, F]> : [O, T];
type TakeFirst<T extends readonly any[], N extends number> = TupleSplit<T, N>[0];
type SkipFirst<T extends readonly any[], N extends number> = TupleSplit<T, N>[1];
type TupleSlice<T extends readonly any[], S extends number, E extends number> = SkipFirst<TakeFirst<T, E>, S>;
export type SlicedArguments<T extends (...args: any[]) => any, FROM extends number | undefined = undefined, TO extends number | undefined = undefined> = (TupleSlice<Parameters<T>, FROM extends undefined ? 0 : FROM, TO extends undefined ? Parameters<T>['length'] : TO>);
export type PartialFunction<T extends (...args: any[]) => any, FROM extends number | undefined = undefined, TO extends number | undefined = undefined> = (...args: SlicedArguments<T, FROM, TO>) => ReturnType<T>;
export declare function toMinecraftResourceName(path: readonly string[], typeNested?: number): string;
export declare function randomUUID(): UUIDinNumber;
export declare const capitalize: (s: string) => string;
type InputObject<ValueType extends any | NonNullable<any>, Input extends Record<string, ValueType>> = Input;
/** Returns a modified object with only entries that are not undefined. If the input is undefined, {} is returned instead. */
export declare function add<Value extends any | NonNullable<any>, Input extends InputObject<Value, Record<string, Value>>>(object: Input | undefined): InputObject<NonNullable<any>, Input>;
export type RemoveArrayRepeats<T extends readonly any[]> = {
    [K in keyof T]: (T[number] extends {
        [P in keyof T]: P extends K ? never : T[P];
    }[number] ? never : T[K]);
};
export declare function iterateEntries(node: Map<string, any>, fn: (val: any) => any): Record<string, any>;
export interface PackEntry {
    added: number;
    downloads: {
        [key: string]: number;
    };
    updated: number;
    owner: string;
}
export declare const supportedMinecraftVersions: string[];
export declare const latestMinecraftVersion = "1.19";
export declare const MinecraftVersionSchema: import("@sinclair/typebox").TUnion<import("@sinclair/typebox").TLiteral<string>[]>;
export declare const packCategories: string[];
declare const PackDependencySchema: import("@sinclair/typebox").TObject<{
    id: import("@sinclair/typebox").TString;
    version: import("@sinclair/typebox").TString;
}>;
export declare const PackVersionSchema: import("@sinclair/typebox").TObject<{
    name: import("@sinclair/typebox").TString;
    downloads: import("@sinclair/typebox").TObject<{
        datapack: import("@sinclair/typebox").TString;
        resourcepack: import("@sinclair/typebox").TString;
    }>;
    supports: import("@sinclair/typebox").TArray<import("@sinclair/typebox").TUnion<import("@sinclair/typebox").TLiteral<string>[]>>;
    dependencies: import("@sinclair/typebox").TArray<import("@sinclair/typebox").TObject<{
        id: import("@sinclair/typebox").TString;
        version: import("@sinclair/typebox").TString;
    }>>;
}>;
export declare const PackDataSchema: import("@sinclair/typebox").TObject<{
    id: import("@sinclair/typebox").TString;
    display: import("@sinclair/typebox").TObject<{
        name: import("@sinclair/typebox").TString;
        description: import("@sinclair/typebox").TString;
        icon: import("@sinclair/typebox").TString;
        hidden: import("@sinclair/typebox").TBoolean;
        webPage: import("@sinclair/typebox").TOptional<import("@sinclair/typebox").TString>;
    }>;
    versions: import("@sinclair/typebox").TArray<import("@sinclair/typebox").TObject<{
        name: import("@sinclair/typebox").TString;
        downloads: import("@sinclair/typebox").TObject<{
            datapack: import("@sinclair/typebox").TString;
            resourcepack: import("@sinclair/typebox").TString;
        }>;
        supports: import("@sinclair/typebox").TArray<import("@sinclair/typebox").TUnion<import("@sinclair/typebox").TLiteral<string>[]>>;
        dependencies: import("@sinclair/typebox").TArray<import("@sinclair/typebox").TObject<{
            id: import("@sinclair/typebox").TString;
            version: import("@sinclair/typebox").TString;
        }>>;
    }>>;
    categories: import("@sinclair/typebox").TArray<import("@sinclair/typebox").TUnion<import("@sinclair/typebox").TLiteral<string>[]>>;
}>;
export declare const MetaDataSchema: import("@sinclair/typebox").TObject<{
    docId: import("@sinclair/typebox").TString;
    rawId: import("@sinclair/typebox").TString;
    stats: import("@sinclair/typebox").TObject<{
        updated: import("@sinclair/typebox").TOptional<import("@sinclair/typebox").TNumber>;
        added: import("@sinclair/typebox").TNumber;
        downloads: import("@sinclair/typebox").TObject<{
            total: import("@sinclair/typebox").TNumber;
            today: import("@sinclair/typebox").TNumber;
        }>;
    }>;
    owner: import("@sinclair/typebox").TString;
    contributors: import("@sinclair/typebox").TArray<import("@sinclair/typebox").TString>;
}>;
export type PackMetaData = Static<typeof MetaDataSchema>;
export type MinecraftVersion = Static<typeof MinecraftVersionSchema>;
export type PackDependency = Static<typeof PackDependencySchema>;
export type PackVersion = Static<typeof PackVersionSchema>;
export type PackData = Static<typeof PackDataSchema>;
export interface UserData {
    displayName: string;
}
export declare enum HTTPResponses {
    OK = 200,
    CREATED = 201,
    BAD_REQUEST = 400,
    UNAUTHORIZED = 401,
    FORBIDDEN = 403,
    NOT_FOUND = 404,
    CONFLICT = 409,
    SERVER_ERROR = 500
}
export type ReviewState = 'verified' | 'pending' | 'unsubmitted' | 'rejected';
export declare function safeWrite(...args: Partial<Parameters<typeof fs['writeFile']>>): Promise<void>;
export declare function sanitizeTarget(target: MultipleEntitiesArgument<false>): string;
export {};
