/* eslint-disable operator-linebreak */
/* eslint-disable @typescript-eslint/no-non-null-assertion */
import fs from 'fs-extra';
import { coerce } from 'semver';
import * as util from 'util';
import { FormatRegistry, Type } from '@sinclair/typebox';
/* @ts-ignore */
export const fetch = Object.hasOwn(globalThis, 'fetch') ?
    globalThis.fetch : async (...args) => (await import('node-fetch')).default(...args);
export function isAsyncFunction(func) {
    if (util.types) {
        // We are in NodeJS, so we can use the builtin check
        return util.types.isAsyncFunction(func);
    }
    /*
     * We might be in a browser, or another weird environment
     * Try a native way first.
     */
    if (func.constructor.name === 'AsyncFunction') {
        return true;
    }
    // Okay, we need a stronger check.
    const string = func.toString().trim();
    return !!(
    // native
    string.match(/^async /)
        // babel (this may change, but hey...)
        || string.match(/return _ref[^.]*\.apply/));
}
export function isPromise(promise) {
    return promise && typeof promise.then === 'function' && promise[Symbol.toStringTag] === 'Promise';
}
export function makeCallableProxy(func, object) {
    return new Proxy(func, {
        get: (target, p, receiver) => object[p],
        set: (target, p, value, receiver) => {
            object[p] = value;
            return value;
        },
        getPrototypeOf: (target) => Object.getPrototypeOf(object),
    });
}
export function makeClassCallable(Class_) {
    return new Proxy(Class_, {
        construct: (target, argArray, newTarget) => {
            const obj = new Class_(...argArray);
            const result = makeCallableProxy(obj.__call__, obj);
            // TODO: Figure this out
            // result.makeCallable(result)
            return result;
        },
    });
}
export function makeCallable(object, func, useProxy = false) {
    if (useProxy) {
        return makeCallableProxy(func, object);
    }
    const { name, ...objectExceptName } = object;
    // Everything can be assigned except the name
    const result = Object.assign(func, objectExceptName);
    if (name !== undefined) {
        const descriptor = Object.getOwnPropertyDescriptor(func, 'name');
        descriptor.value = name;
        Object.defineProperty(func, 'name', descriptor);
    }
    return result;
}
export function toMinecraftResourceName(path, typeNested = 1) {
    const [namespace, ...folders] = path;
    folders.splice(0, typeNested);
    return `${namespace}:${folders.join('/')}`;
}
/* Ported from https://github.com/AjaxGb/mc-uuid-converter/blob/master/convert.js */
const uuidBytes = new Uint8Array(16);
const uuid = new DataView(uuidBytes.buffer);
export function randomUUID() {
    crypto.getRandomValues(uuidBytes);
    // Set version to 4 (random)
    // eslint-disable-next-line no-bitwise
    uuidBytes[6] = (uuidBytes[6] & 0x0f) | (4 << 4);
    // Set variant to 1 (Leach–Salz)
    // eslint-disable-next-line no-bitwise
    uuidBytes[8] = (uuidBytes[8] & 0x3f) | 0x80;
    const array = [];
    // eslint-disable-next-line no-plusplus
    for (let i = 0; i < 4; i++) {
        array.push(uuid.getInt32(i * 4, false));
    }
    return array;
}
export const capitalize = (s) => s.charAt(0).toUpperCase() + s.slice(1);
/** Returns a modified object with only entries that are not undefined. If the input is undefined, {} is returned instead. */
export function add(object) {
    if (object === undefined) {
        return {};
    }
    const output = {};
    for (const [key, value] of object.entries()) {
        if (value !== undefined) {
            output[key] = value;
        }
    }
    return output;
}
export function iterateEntries(node, fn) {
    const newNode = {};
    // eslint-disable-next-line no-return-assign
    node.forEach(([key, val]) => (newNode[key] = fn(val)));
    return newNode;
}
FormatRegistry.Set('semver', (v) => coerce(v) != null);
export const supportedMinecraftVersions = [
    '1.18',
    '1.18.1',
    '1.18.2',
    '1.19',
];
export const latestMinecraftVersion = '1.19';
export const MinecraftVersionSchema = Type.Union(supportedMinecraftVersions.map((v) => Type.Literal(v)));
export const packCategories = [
    'Extensive',
    'Lightweight',
    'QoL',
    'Vanilla+',
    'Tech',
    'Magic',
    'Library',
    'Exploration',
    'World Overhaul',
    'No Resource Pack',
];
const PackDependencySchema = Type.Object({
    id: Type.String(),
    version: Type.String(),
});
export const PackVersionSchema = Type.Object({
    name: Type.String({ minLength: 1 }),
    downloads: Type.Object({
        datapack: Type.String({ minLength: 1 }),
        resourcepack: Type.String(),
    }),
    supports: Type.Array(MinecraftVersionSchema, { minItems: 1 }),
    dependencies: Type.Array(PackDependencySchema),
});
export const PackDataSchema = Type.Object({
    id: Type.String({ minLength: 3 }),
    display: Type.Object({
        name: Type.String({ minLength: 3 }),
        description: Type.String({ minLength: 3 }),
        icon: Type.String({ default: '' }),
        hidden: Type.Boolean({ default: false }),
        webPage: Type.Optional(Type.String()),
    }),
    versions: Type.Array(PackVersionSchema, { minItems: 1 }),
    categories: Type.Array(Type.Union(packCategories.map((c) => Type.Literal(c)))),
});
export const MetaDataSchema = Type.Object({
    docId: Type.String(),
    rawId: Type.String(),
    stats: Type.Object({
        updated: Type.Optional(Type.Number()),
        added: Type.Number(),
        downloads: Type.Object({
            total: Type.Number(),
            today: Type.Number(),
        }),
    }),
    owner: Type.String(),
    contributors: Type.Array(Type.String(), { default: [] }),
});
// eslint-disable-next-line no-shadow
export var HTTPResponses;
(function (HTTPResponses) {
    HTTPResponses[HTTPResponses["OK"] = 200] = "OK";
    HTTPResponses[HTTPResponses["CREATED"] = 201] = "CREATED";
    HTTPResponses[HTTPResponses["BAD_REQUEST"] = 400] = "BAD_REQUEST";
    HTTPResponses[HTTPResponses["UNAUTHORIZED"] = 401] = "UNAUTHORIZED";
    HTTPResponses[HTTPResponses["FORBIDDEN"] = 403] = "FORBIDDEN";
    HTTPResponses[HTTPResponses["NOT_FOUND"] = 404] = "NOT_FOUND";
    HTTPResponses[HTTPResponses["CONFLICT"] = 409] = "CONFLICT";
    HTTPResponses[HTTPResponses["SERVER_ERROR"] = 500] = "SERVER_ERROR";
})(HTTPResponses || (HTTPResponses = {}));
export async function safeWrite(...args) {
    if (typeof args[0] !== 'string')
        throw new Error('unimplemented');
    await fs.ensureDir(args[0].replace(/(?:\/|\\)(?:.(?!(?:\/|\\)))+$/, ''));
    return fs.writeFile(...args);
}
export function sanitizeTarget(target) {
    return `${target}`.toLowerCase().replaceAll(/-|=|\[|\]|\{|\}|:|"|'|,|@/g, '_');
}
