import { inspect } from 'util';
/** A root class that can be used to defined a N-dimension vector */
export class VectorClass {
    values;
    constructor(values) {
        if (!Array.isArray(values) || !values.every((i) => typeof i === 'string')) {
            throw new Error(`Expected array of string for Vector values, got ${inspect(values)}`);
        }
        this.values = values;
    }
    toString() {
        return this.values.join(' ');
    }
    toJSON() {
        return this.toString();
    }
    [Symbol.iterator]() {
        return this.values[Symbol.iterator]();
    }
}
export function absolute(...coordinates) {
    if (coordinates.length === 1) {
        return coordinates[0].toString();
    }
    return new VectorClass(coordinates.map((coord) => coord.toString()));
}
export function relative(...coordinates) {
    if (coordinates.length === 0) {
        return new VectorClass(['~', '~', '~']);
    }
    if (coordinates.length === 1) {
        return `~${coordinates[0] || ''}`;
    }
    return new VectorClass(coordinates.map((coord) => `~${coord || ''}`));
}
export function local(...coordinates) {
    if (coordinates.length === 0) {
        return new VectorClass(['^', '^', '^']);
    }
    if (coordinates.length === 1) {
        return `^${coordinates[0] || ''}`;
    }
    return new VectorClass(coordinates.map((coord) => `^${coord || ''}`));
}
