import { DataPointPickClass, MacroArgument } from '../core/Macro.js';
import { Score } from './Score.js';
import type { Coordinates, JSONTextComponent, NBTObject, SingleEntityArgument } from 'sandstone/arguments';
import type { DataModifyTypeCommand, DataModifyValuesCommand, StoreType } from 'sandstone/commands/implementations';
import type { SandstonePack } from 'sandstone/pack';
import type { ConditionTextComponentClass } from './abstractClasses.js';
export type DATA_TYPES = 'entity' | 'block' | 'storage';
export type DATA_TARGET = {
    'entity': SingleEntityArgument<false>;
    'block': Coordinates<false>;
    'storage': string;
};
export type DATA_PATH = string | Record<string, NBTObject> | NBTObject[];
export declare function NBTpathToString(path: DATA_PATH[]): string;
export declare class TargetlessDataClass<TYPE extends DATA_TYPES = DATA_TYPES> {
    protected sandstonePack: SandstonePack;
    type: TYPE;
    constructor(sandstonePack: SandstonePack, type: TYPE);
    target: (target: DATA_TARGET[TYPE]) => DataClass<TYPE>;
    select: (...path: DATA_PATH[]) => TargetlessDataPointClass<TYPE>;
}
export declare class TargetlessDataPointClass<TYPE extends DATA_TYPES = DATA_TYPES> {
    protected sandstonePack: SandstonePack;
    type: TYPE;
    path: string;
    constructor(sandstonePack: SandstonePack, type: TYPE, path: DATA_PATH[]);
    target: (target: DATA_TARGET[TYPE]) => DataPointClass<TYPE>;
    select: (...path: DATA_PATH[]) => TargetlessDataPointClass<TYPE>;
}
export declare class DataClass<TYPE extends DATA_TYPES = DATA_TYPES> {
    protected sandstonePack: SandstonePack;
    type: TYPE;
    currentTarget: DATA_TARGET[TYPE];
    constructor(sandstonePack: SandstonePack, type: TYPE, target: DATA_TARGET[TYPE]);
    /**
     * Merge the given NBT to the current target.
     */
    merge: (value: NBTObject) => void;
    target: (target: DATA_TARGET[TYPE]) => DataClass<TYPE>;
    select: (...path: DATA_PATH[]) => DataPointClass<TYPE>;
    toString: () => DATA_TARGET[TYPE];
    toJSON: () => DATA_TARGET[TYPE];
}
export declare class DataPointClass<TYPE extends DATA_TYPES = any> extends MacroArgument implements ConditionTextComponentClass {
    sandstonePack: SandstonePack;
    type: TYPE;
    path: string;
    currentTarget: DATA_TARGET[TYPE];
    constructor(sandstonePack: SandstonePack, type: TYPE, target: DATA_TARGET[TYPE], path: DATA_PATH[]);
    target: (target: DATA_TARGET[TYPE]) => DataPointClass<TYPE>;
    select: (...path: DATA_PATH[]) => DataPointClass<TYPE>;
    protected modify: (cb: (data: DataModifyTypeCommand<false>) => DataModifyValuesCommand<false>, value: NBTObject | DataPointClass | DataPointPickClass) => this;
    protected string: (cb: (data: DataModifyTypeCommand<false>) => DataModifyValuesCommand<false>, value: DataPointClass, start: number, end?: number) => void;
    protected executeStore: (storeType: StoreType, scale: number) => import("sandstone/commands/implementations").ExecuteCommand<false>;
    set: (
    /**
     * Set the data point to the given NBT.
     */
    ((value: NBTObject | DataPointClass) => DataPointClass) & 
    /**
     * Set the data point to the given Data Point Pick
     */
    ((value: DataPointPickClass) => DataPointClass) & 
    /**
     * Set the data point to the given score, with a given type and a scale.
     */
    ((value: Score, storeType?: StoreType, scale?: number) => DataPointClass) & 
    /**
     * Set the data point to the given NBT string.
     */
    ((value: StringDataPointClass) => DataPointClass));
    /**
     * Set the data point to the given NBT.
     */
    merge: (value: NBTObject | DataPointClass | DataPointPickClass) => this;
    /**
     * Append the given NBT to the current data point.
     */
    append: (value: NBTObject | DataPointClass | DataPointPickClass) => this;
    /**
     * Prepend the given NBT to the current data point.
     */
    prepend: (value: NBTObject | DataPointClass | DataPointPickClass) => this;
    /**
     * Insert the given NBT to the given index of the current data point.
     */
    insert: (value: NBTObject | DataPointClass | DataPointPickClass, index: number) => this;
    /**
     * Remove the current NBT value.
     */
    remove: () => import("../commands/helpers.js").FinalCommandOutput;
    /**
     * Extracts a section of a string available for setting to another path, without modifying the original path.
     *
     * Use slice(0) to get a type-safe string data point; it compiles to a regular `from` argument.
     *
     * @param start Index of first character to include at the start of the string.
     * @param end Optional. Index of the first character to exclude at the end of the string.
     */
    slice: (start: number, end?: number) => StringDataPointClass<TYPE>;
    _toMinecraftCondition: () => import("../flow/conditions/variables/dataPoint.js").DataPointExistsConditionNode;
    equals: (value: NBTObject | Score | DataPointClass | DataPointPickClass) => import("../flow/conditions/variables/dataPoint.js").DataPointEqualsConditionNode;
    /**
     * @internal
     */
    _toChatComponent: () => JSONTextComponent;
}
export declare class StringDataPointClass<TYPE extends DATA_TYPES = any> extends DataPointClass {
    sandstonePack: SandstonePack;
    type: TYPE;
    readonly sliceBounds: [number] | [number, number];
    constructor(sandstonePack: SandstonePack, type: TYPE, target: DATA_TARGET[TYPE], path: DATA_PATH, start: number, end?: number);
}
