import { ConditionalDataPointPickClass } from '../core/Macro.js';
import { DataPointClass } from './Data.js';
import { Score } from './Score.js';
import type { NBTObject, RootNBT } from 'sandstone/arguments';
import type { DataPointPickClass } from 'sandstone/core';
import type { ConditionNode } from 'sandstone/flow';
import type { LiteralUnion } from 'sandstone/utils';
import type { Macroable } from '../core/Macro.js';
import type { SandstonePack } from '../pack/index.js';
import type { StringDataPointClass } from './Data.js';
export declare abstract class IterableDataClass extends ConditionalDataPointPickClass {
    iterator(callback: (dataPoints: [DataPointClass] | [StringDataPointClass, DataPointClass]) => void): () => void;
    size(): DataPointClass<'storage'>;
    last(): readonly [StringDataPointClass, DataPointClass] | DataPointClass;
    get continue(): ConditionNode;
}
type GetKeys<T> = T extends unknown[] ? T extends [] ? never : '0' extends keyof T ? Exclude<keyof T, keyof []> : number : keyof T;
export type DataIndexMapInitial = RootNBT | Record<string, DataPointClass | DataPointPickClass>;
export declare class DataIndexMapClass<INITIAL extends DataIndexMapInitial> extends IterableDataClass {
    private pack;
    private initialize;
    entries: Record<string, number>;
    dataPoint: NonNullable<DataPointClass<'storage'>>;
    constructor(pack: SandstonePack, initialize: INITIAL, dataPoint?: DataPointClass<'storage'>);
    __entryIterator?: DataPointClass<'storage'>;
    get entryIterator(): DataPointClass<"storage">;
    iterator(callback: (data: [key: StringDataPointClass, value: DataPointClass]) => void): () => void;
    get continue(): import("../flow/conditions/variables/dataPoint.js").DataPointExistsConditionNode;
    /**
     * Use in conjunction with a Score to get the count of entries.
     *
     * @example
     * Variable(map.size(), 'map_size')
     *
     * @returns The entries of the map.
     */
    size: () => DataPointClass<"storage">;
    last: () => readonly [key: StringDataPointClass<any>, value: DataPointClass<any>];
    /**
     * @returns The index of the entry.
     */
    set(key: Macroable<LiteralUnion<GetKeys<INITIAL>>, true>, value: NBTObject | DataPointClass | DataPointPickClass): number | Score;
    get(key: GetKeys<INITIAL>): INITIAL[typeof key] extends DataPointClass ? INITIAL[typeof key] : DataPointClass<'storage'>;
    get(key: Macroable<string, true>): DataPointClass<'storage'>;
    remove(key: Macroable<LiteralUnion<GetKeys<INITIAL>>, true>): this;
    _toDataPoint: () => DataPointClass<"storage">;
    _toMinecraftCondition(): import("../flow/conditions/variables/dataPoint.js").DataPointExistsConditionNode;
}
export type DataArrayInitial = readonly NBTObject[] | readonly [string, DataPointClass | DataPointPickClass];
export declare class DataArrayClass<INITIAL extends DataArrayInitial> extends IterableDataClass {
    private pack;
    dataPoint: NonNullable<DataPointClass<'storage'>>;
    constructor(pack: SandstonePack, initialize: INITIAL, dataPoint?: DataPointClass<'storage'>);
    __entryIterator?: DataPointClass<'storage'>;
    get entryIterator(): DataPointClass<"storage">;
    iterator(callback: (data: [entry: DataPointClass]) => void): () => void;
    get continue(): import("../flow/conditions/variables/dataPoint.js").DataPointExistsConditionNode;
    /**
     * Use in conjunction with a Score to get the count of entries.
     *
     * @example
     * Variable(array.size(), 'array_size')
     *
     * @returns The entries of the array.
     */
    size: () => DataPointClass<"storage">;
    last: () => DataPointClass<"storage">;
    set(index: Score | number, value: NBTObject | DataPointClass | DataPointPickClass): this;
    push(value: NBTObject | DataPointClass | DataPointPickClass): this;
    unshift(value: NBTObject | DataPointClass | DataPointPickClass): this;
    get(index: Score | number): DataPointClass<"storage">;
    remove(index: Score | number): this;
    _toDataPoint: () => DataPointClass<"storage">;
    _toMinecraftCondition(): import("../flow/conditions/variables/dataPoint.js").DataPointExistsConditionNode;
}
export declare function DataIndexMap<INITIAL extends DataIndexMapInitial>(pack: SandstonePack, initialize: INITIAL, dataPoint?: DataPointClass<'storage'>): DataIndexMapClass<INITIAL> & { [K in keyof INITIAL]: NBTObject | DataPointPickClass | DataPointClass<any> | undefined; } & {
    [x: string]: NBTObject | DataPointPickClass | DataPointClass<any> | undefined;
};
export type DataIndexMap<INITIAL extends DataIndexMapInitial> = DataIndexMapClass<INITIAL> & {
    [K in keyof INITIAL]: DataIndexMapInitial[string];
} & {
    [K in string]: DataIndexMapInitial[string];
};
export declare function DataArray<INITIAL extends DataArrayInitial>(pack: SandstonePack, initialize: INITIAL, dataPoint?: DataPointClass<'storage'>): DataArrayClass<INITIAL> & { [K in keyof INITIAL]: NBTObject | DataPointPickClass | DataPointClass<any>; } & {
    [x: number]: NBTObject | DataPointPickClass | DataPointClass<any>;
};
export type DataArray<INITIAL extends DataArrayInitial> = DataArrayClass<INITIAL> & {
    [K in keyof INITIAL]: DataArrayInitial[number];
} & {
    [K in number]: DataArrayInitial[number];
};
export {};
