/* eslint-disable @typescript-eslint/no-non-null-assertion */
/* eslint-disable max-len */
import { ConditionalDataPointPickClass } from '../core/Macro.js';
import { DataPointClass } from './Data.js';
import { Score } from './Score.js';
export class IterableDataClass extends ConditionalDataPointPickClass {
    iterator(callback) {
        throw new Error('Not implemented');
    }
    size() {
        throw new Error('Not implemented');
    }
    last() {
        throw new Error('Not implemented');
    }
    get continue() {
        throw new Error('Not implemented');
    }
}
export class DataIndexMapClass extends IterableDataClass {
    pack;
    initialize;
    entries = {};
    dataPoint;
    constructor(pack, initialize, dataPoint) {
        super(pack.core);
        this.pack = pack;
        this.initialize = initialize;
        if (dataPoint) {
            this.dataPoint = dataPoint;
        }
        else {
            this.dataPoint = this.pack.DataVariable(undefined, 'IndexMap');
        }
        for (const [i, [key, value]] of Object.entries(initialize).entries()) {
            this.dataPoint.select('Index').select(key).set(i);
            this.dataPoint.select('Entries').append([]);
            const entry = this.dataPoint.select(`Entries[${i}]`);
            entry.append(key);
            entry.append(value);
            this.entries[key] = i;
        }
    }
    __entryIterator;
    get entryIterator() {
        if (!this.__entryIterator) {
            this.__entryIterator = this.pack.DataVariable(this.dataPoint.select('Entries')).select('[-1]');
        }
        return this.__entryIterator;
    }
    iterator(callback) {
        this.__entryIterator = undefined;
        return () => {
            callback([this.entryIterator.select('[0]').slice(0), this.entryIterator.select('[1]')]);
            this.entryIterator.remove();
        };
    }
    get continue() {
        return this.entryIterator._toMinecraftCondition();
    }
    /**
     * Use in conjunction with a Score to get the count of entries.
     *
     * @example
     * Variable(map.size(), 'map_size')
     *
     * @returns The entries of the map.
     */
    size = () => this.dataPoint.select('Entries');
    last = () => [this.dataPoint.select('Entries[-1][0]').slice(0), this.dataPoint.select('Entries[-1][1]')];
    /**
     * @returns The index of the entry.
     */
    set(key, value) {
        if (typeof key === 'string') {
            if (this.entries[key]) {
                const index = this.entries[key];
                this.dataPoint.select(`Entries[${index}]`).set(value);
                return index;
            }
            this.dataPoint.select('Keys').append(key);
            this.dataPoint.select('Values').append(value);
            const index = this.pack.Variable(this.size());
            this.dataPoint.select('Index').select(key).set(index);
            this.dataPoint.select('Entries').append([]);
            const entry = this.dataPoint.select('Entries[-1]');
            entry.append(key);
            entry.append(value);
            return index;
        }
        const { Macro, MCFunction, Variable, DataVariable, } = this.pack;
        let _key;
        if (key instanceof DataPointClass) {
            _key = key;
        }
        else if (Object.hasOwn(key, '_toDataPoint')) {
            _key = key._toDataPoint();
        }
        else if (key instanceof Score) {
            _key = DataVariable().set(key);
        }
        const index = Variable(this.size());
        this.dataPoint.select('Keys').append(_key);
        this.dataPoint.select('Values').append(value);
        MCFunction('__sandstone:variable/index_map/set', [_key], () => {
            const indexData = DataVariable().set(index);
            Macro.data.modify.storage(this.dataPoint.currentTarget, Macro `${this.dataPoint.path}.Index[${_key}]`).set.from.storage(indexData.currentTarget, indexData.path);
        })();
        this.dataPoint.select('Entries').append([]);
        const entry = this.dataPoint.select('Entries[-1]');
        entry.append(_key);
        entry.append(value);
        return index;
    }
    get(key) {
        const { MCFunction, Macro, DataVariable, } = this.pack;
        if (typeof key === 'string') {
            if (this.entries[key]) {
                return this.dataPoint.select(`Entries[${this.entries[key]}]`);
            }
            const index = this.dataPoint.select('Index').select(key);
            const value = DataVariable();
            MCFunction('__sandstone:variable/index_map/get', [index], () => {
                Macro.data.modify.storage(value.currentTarget, value.path).set.from.storage(this.dataPoint.currentTarget, Macro `Entries[${index}]`);
            })();
            return value;
        }
        let _key;
        if (key instanceof DataPointClass) {
            _key = key;
        }
        else if (Object.hasOwn(key, '_toDataPoint')) {
            _key = key._toDataPoint();
        }
        else if (key instanceof Score) {
            _key = DataVariable().set(key);
        }
        const output = DataVariable();
        MCFunction('__sandstone:variable/index_map/get', [_key], () => {
            const index = DataVariable();
            Macro.data.modify.storage(index.currentTarget, index.path).set.from.storage(this.dataPoint.currentTarget, Macro `${this.dataPoint.path}.Index[${_key}]`);
            MCFunction('__sandstone:variable/index_map/_get', [index], () => {
                Macro.data.modify.storage(output.currentTarget, output.path).set.from.storage(this.dataPoint.currentTarget, Macro `${this.dataPoint.path}.Entries[${index}]`);
            })();
        })();
        return output;
    }
    remove(key) {
        const { MCFunction, Macro, DataVariable } = this.pack;
        if (typeof key === 'string') {
            const index = this.dataPoint.select('Index').select(key);
            MCFunction('__sandstone:variable/index_map/remove', [index], () => {
                Macro.data.modify.storage(this.dataPoint.currentTarget, Macro `Entries[${index}]`).set.value(0);
                index.remove();
            })();
            return this;
        }
        let _key;
        if (key instanceof DataPointClass || key instanceof Score) {
            _key = key;
        }
        else {
            _key = key._toDataPoint();
        }
        const index = DataVariable();
        MCFunction('__sandstone:variable/index_map/remove', [_key], () => {
            Macro.data.modify.storage(index.currentTarget, index.path).set.from.storage(this.dataPoint.currentTarget, Macro `${this.dataPoint.path}.Index[${_key}]`);
            Macro.data.remove.storage(this.dataPoint.currentTarget, Macro `${this.dataPoint.path}.Index[${_key}]`);
            MCFunction('__sandstone:variable/index_map/_remove', [index], () => {
                Macro.data.modify.storage(this.dataPoint.currentTarget, Macro `${this.dataPoint.path}.Entries[${index}]`).set.value(0);
            })();
        })();
        return this;
    }
    _toDataPoint = () => this.dataPoint;
    _toMinecraftCondition() {
        return this.dataPoint._toMinecraftCondition();
    }
}
export class DataArrayClass extends IterableDataClass {
    pack;
    dataPoint;
    constructor(pack, initialize, dataPoint) {
        super(pack.core);
        this.pack = pack;
        if (dataPoint) {
            this.dataPoint = dataPoint;
        }
        else {
            this.dataPoint = this.pack.DataVariable([], 'Array');
        }
        for (const entry of initialize) {
            this.dataPoint.append(entry);
        }
    }
    __entryIterator;
    get entryIterator() {
        if (!this.__entryIterator) {
            this.__entryIterator = this.pack.DataVariable(this.dataPoint).select('[-1]');
        }
        return this.__entryIterator;
    }
    iterator(callback) {
        this.__entryIterator = undefined;
        return () => {
            callback([this.entryIterator]);
            this.entryIterator.remove();
        };
    }
    get continue() {
        return this.entryIterator._toMinecraftCondition();
    }
    /**
     * Use in conjunction with a Score to get the count of entries.
     *
     * @example
     * Variable(array.size(), 'array_size')
     *
     * @returns The entries of the array.
     */
    size = () => this.dataPoint;
    last = () => this.dataPoint.select('[-1]');
    set(index, value) {
        if (typeof index === 'number') {
            this.dataPoint.select(`[${index}]`).set(value);
            return this;
        }
        const { Macro, MCFunction } = this.pack;
        MCFunction('__sandstone:variable/array/set', [index], () => {
            if (value instanceof DataPointClass) {
                Macro.data.modify.storage(this.dataPoint.currentTarget, Macro `${this.dataPoint.path}[${index}]`).set.from.storage(value.currentTarget, value.path);
            }
            else if (typeof value === 'object' && Object.hasOwn(value, '_toDataPoint')) {
                const point = value._toDataPoint();
                Macro.data.modify.storage(this.dataPoint.currentTarget, Macro `${this.dataPoint.path}[${index}]`).set.from.storage(point.currentTarget, point.path);
            }
            else {
                Macro.data.modify.storage(this.dataPoint.currentTarget, Macro `${this.dataPoint.path}[${index}]`).set.value(value);
            }
        })();
        return this;
    }
    push(value) {
        this.dataPoint.append(value);
        return this;
    }
    unshift(value) {
        this.dataPoint.prepend(value);
        return this;
    }
    get(index) {
        if (typeof index === 'number') {
            return this.dataPoint.select(`[${index}]`);
        }
        const { Macro, MCFunction, DataVariable } = this.pack;
        const output = DataVariable();
        MCFunction('__sandstone:variable/array/get', [index], () => {
            Macro.data.modify.storage(output.currentTarget, output.path).set.from.storage(this.dataPoint.currentTarget, Macro `${this.dataPoint.path}[${index}]`);
        })();
        return output;
    }
    remove(index) {
        if (typeof index === 'number') {
            this.dataPoint.select(`[${index}]`).remove();
            return this;
        }
        const { MCFunction, Macro } = this.pack;
        MCFunction('__sandstone:variable/array/remove', [index], () => {
            Macro.data.remove.storage(this.dataPoint.currentTarget, Macro `${this.dataPoint.path}[${index}]`);
        })();
        return this;
    }
    _toDataPoint = () => this.dataPoint;
    _toMinecraftCondition() {
        return this.dataPoint._toMinecraftCondition();
    }
}
export function DataIndexMap(pack, initialize, dataPoint) {
    const indexMap = new DataIndexMapClass(pack, initialize, dataPoint);
    return new Proxy(indexMap, {
        get(target, p, receiver) {
            if (p in target) {
                return Reflect.get(target, p, receiver);
            }
            return target.get(p);
        },
        set(target, p, value, receiver) {
            if (p in target) {
                return Reflect.set(target, p, value, receiver);
            }
            target.set(p, value);
            return true;
        },
    });
}
export function DataArray(pack, initialize, dataPoint) {
    const array = new DataArrayClass(pack, initialize, dataPoint);
    return new Proxy(array, {
        get(target, p, receiver) {
            if (typeof p === 'string' && p in target) {
                return Reflect.get(target, p, receiver);
            }
            return target.get(Number(p));
        },
        set(target, p, value, receiver) {
            if (typeof p === 'string' && p in target) {
                return Reflect.set(target, p, value, receiver);
            }
            target.set(Number(p), value);
            return true;
        },
    });
}
