import { SelectorClass } from './Selector.js';
import type { MakeInstanceCallable } from 'sandstone/utils';
import type { SandstonePack } from '../pack/index.js';
import type { ConditionClass, SelectorPickClass } from './abstractClasses.js';
type SingleEntity = SelectorClass<false, true, boolean> | SelectorPickClass<true, boolean>;
/**
 * Label tag (/tag) handler
 */
export declare class _RawLabelClass {
    private pack;
    /**
     * Label name
     */
    name: string;
    /**
     * Label Tag name with namespace
     */
    fullName: string;
    /**
     * Label Description (optional)
     */
    description?: string;
    constructor(pack: SandstonePack, name: string, description?: string);
    LabelHolder: (entity: SingleEntity | '@s' | '@p' | '@r') => EntityLabel;
    /**
     * Contains the name and description of the Label (eg. 'wasd:is_walking; Whether the player is not mounted')
     */
    toString: () => string;
    /**
     * @internal
     */
    __call__: (entity: SingleEntity | '@s' | '@p' | '@r') => EntityLabel;
}
export declare const LabelClass: import("sandstone/utils").MakeClassCallable<typeof _RawLabelClass>;
export type LabelClass = MakeInstanceCallable<_RawLabelClass>;
export declare class EntityLabel implements ConditionClass, SelectorPickClass<true, false> {
    private pack;
    /** Label */
    label: _RawLabelClass;
    /**
     * Selects entity with the label
     */
    selector: SelectorClass<false, true, false>;
    /**
     * Selects entity
     */
    originalSelector: string | SelectorClass<false, true, false>;
    /** Test for label on entity */
    test: ConditionClass;
    constructor(pack: SandstonePack, entity: SingleEntity, label: _RawLabelClass);
    /**
     * Add label to entity
     */
    add: () => import("../commands/helpers.js").FinalCommandOutput;
    /**
     * Remove label from entity
     */
    remove: () => import("../commands/helpers.js").FinalCommandOutput;
    /**
     * Set label on/off for entity
     */
    set(set: boolean | ConditionClass): void;
    /**
     * Toggle label on/off for entity
     */
    toggle(): void;
    /**
     * Contains the selector, and the name/description of the Label (eg. 'Whether @s has the label wasd:is_walking; Whether the player is not mounted')
     */
    toString: () => string;
    /**
     * @internal
     */
    _toMinecraftCondition: () => import("../flow/conditions/variables/label.js").LabelConditionNode;
    /**
     * @internal
     */
    _toSelector: () => SelectorClass<false, true, false>;
}
export {};
