/* eslint-disable max-len */
import { makeClassCallable } from 'sandstone/utils';
import { SelectorClass } from './Selector.js';
/**
 * Label tag (/tag) handler
 */
export class _RawLabelClass {
    pack;
    /**
     * Label name
     */
    name;
    /**
     * Label Tag name with namespace
     */
    fullName;
    /**
     * Label Description (optional)
     */
    description;
    constructor(pack, name, description) {
        this.pack = pack;
        if (name.includes('.')) {
            this.name = name.split('.').slice(1).join('.');
            this.fullName = name;
        }
        else {
            this.name = name;
            this.fullName = `${pack.defaultNamespace}.${name}`;
        }
        if (description)
            this.description = description;
    }
    LabelHolder = (entity) => {
        if (typeof entity === 'string')
            return new EntityLabel(this.pack, new SelectorClass(this.pack, entity), this);
        return new EntityLabel(this.pack, entity, this);
    };
    /**
     * Contains the name and description of the Label (eg. 'wasd:is_walking; Whether the player is not mounted')
     */
    toString = () => `${this.fullName}${this.description ? `; ${this.description}` : ''}`;
    /**
     * @internal
     */
    __call__ = (entity) => this.LabelHolder(entity);
}
export const LabelClass = makeClassCallable(_RawLabelClass);
export class EntityLabel {
    pack;
    /** Label */
    label;
    /**
     * Selects entity with the label
     */
    selector;
    /**
     * Selects entity
     */
    originalSelector;
    /** Test for label on entity */
    test = this;
    constructor(pack, entity, label) {
        this.pack = pack;
        this.originalSelector = entity._toSelector();
        this.label = label;
        // Haha brrrrrrr
        const selector = (typeof this.originalSelector === 'string' ? new SelectorClass(this.pack, '@s') : new SelectorClass(this.pack, this.originalSelector.target, { ...this.originalSelector.arguments }));
        if (selector.arguments) {
            if (selector.arguments.tag) {
                if (typeof selector.arguments.tag === 'string' || !Array.isArray(selector.arguments.tag)) {
                    selector.arguments.tag = [selector.arguments.tag, label.fullName];
                }
                else {
                    selector.arguments.tag = [...selector.arguments.tag, label.fullName];
                }
            }
            else
                selector.arguments.tag = [label.fullName];
        }
        else
            selector.arguments = { tag: [label.fullName] };
        this.selector = selector;
    }
    /**
     * Add label to entity
     */
    add = () => this.pack.commands.tag(this.originalSelector).add(this.label.fullName);
    /**
     * Remove label from entity
     */
    remove = () => this.pack.commands.tag(this.originalSelector).remove(this.label.fullName);
    /**
     * Set label on/off for entity
     */
    set(set) {
        if (typeof set === 'boolean') {
            if (set)
                this.add();
            else
                this.remove();
        }
        else {
            this.pack._.if(set, () => this.add())
                .else(() => this.remove());
        }
    }
    /**
     * Toggle label on/off for entity
     */
    toggle() {
        this.pack._.if(this.test, () => this.remove())
            .else(() => this.add());
    }
    /**
     * Contains the selector, and the name/description of the Label (eg. 'Whether @s has the label wasd:is_walking; Whether the player is not mounted')
     */
    toString = () => `Whether ${this.originalSelector.toString()} has the label ${this.label.toString()}`;
    /**
     * @internal
     */
    _toMinecraftCondition = () => new this.pack.conditions.Label(this.pack.core, this);
    /**
     * @internal
     */
    _toSelector = () => this.selector._toSelector();
}
