/// <reference types="node" resolution-mode="require"/>
import * as util from 'util';
import { DataPointPickClass } from '../core/Macro.js';
import { StringDataPointClass } from './Data.js';
import { NBTClass, NBTInt, NBTIntArray } from './nbt/index.js';
import { Score } from './Score.js';
import type { NBTObject } from '../arguments/nbt.js';
import type { SandstonePack } from '../pack/index.js';
import type { DataPointClass } from './Data.js';
import type { NBTAllArrays, NBTAllNumbers, NBTAllValues, NBTString } from './nbt/index.js';
export declare class ResolveNBTClass extends DataPointPickClass {
    private pack;
    dataPoint: NonNullable<DataPointClass<'storage'>>;
    constructor(pack: SandstonePack, nbt: NBTObject, dataPoint?: DataPointClass<'storage'>);
    /**
     * @internal
     */
    _resolveNBT(nbt: NBTObject, path?: string, index?: number): NBTObject;
    /**
     * @internal
     */
    _resolveData(part: ResolveNBTPartClass<'data', NBTAllValues>, path: string, index?: number): undefined;
    /**
     * @internal
     */
    _resolveScore(value: ResolveNBTPartClass<'score', NBTAllNumbers>, path: string, index?: number): undefined;
    /**
     * @internal
     */
    _resolveScores(value: ResolveNBTPartClass<'scores', NBTAllArrays>, path: string, index?: number): void;
    /**
     * @internal
     */
    _toDataPoint(): DataPointClass<"storage">;
}
export declare class ResolveNBTPartClass<ValueType extends 'data' | 'score' | 'scores', Primitive extends NBTAllValues> extends NBTClass {
    value: Score | DataPointClass<any> | StringDataPointClass<any> | Score[];
    type: ValueType;
    primitive: Primitive;
    scale?: number;
    constructor(value: StringDataPointClass | DataPointClass | Score | Score[], type: ValueType, primitive: Primitive, scale?: number);
    [util.inspect.custom]: () => string;
}
export declare function ResolveNBTPart(data: StringDataPointClass): ResolveNBTPartClass<'data', typeof NBTString>;
export declare function ResolveNBTPart(data: DataPointClass<any>, type: NBTAllValues): ResolveNBTPartClass<'data', NBTAllValues>;
export declare function ResolveNBTPart(score: Score, scale?: number, type?: NBTAllNumbers): ResolveNBTPartClass<'score', typeof NBTInt | NBTAllNumbers>;
export declare function ResolveNBTPart(scores: Score[], scale?: number, type?: NBTAllArrays): ResolveNBTPartClass<'scores', typeof NBTIntArray | NBTAllArrays>;
