/* eslint-disable max-len */
import * as util from 'util';
import { DataPointPickClass } from '../core/Macro.js';
import { capitalize } from '../utils.js';
import { StringDataPointClass } from './Data.js';
import { NBTAnyValue, NBTClass, NBTInt, NBTIntArray, NBTPrimitive, } from './nbt/index.js';
import { Score } from './Score.js';
export class ResolveNBTClass extends DataPointPickClass {
    pack;
    dataPoint;
    constructor(pack, nbt, dataPoint) {
        super(pack.core);
        this.pack = pack;
        if (dataPoint) {
            this.dataPoint = dataPoint;
        }
        else {
            this.dataPoint = this.pack.Data('storage', '__sandstone:temp', 'Resolve');
        }
        const out = this._resolveNBT(nbt);
        if (Object.keys(out).length !== 0) {
            pack.commands.data.modify.storage(this.dataPoint.currentTarget, this.dataPoint.path).merge.value(this._resolveNBT(nbt));
        }
    }
    /**
     * @internal
     */
    _resolveNBT(nbt, path, index) {
        let resolvedNBT = {};
        if (typeof nbt !== 'object') {
            throw new Error(`ResolveNBT expected an object or array, received ${typeof nbt}!`);
        }
        if (Array.isArray(nbt)) {
            resolvedNBT = [];
            this.dataPoint.set([]);
            if (resolvedNBT.length !== 0) {
                for (const [i, value] of nbt.entries()) {
                    const resolved = this._resolveNBT(value, `${path === undefined ? '' : path}`, i);
                    if (resolved !== undefined) {
                        resolvedNBT.push(resolved);
                    }
                }
            }
            return resolvedNBT;
        }
        if (nbt instanceof ResolveNBTPartClass) {
            /* @ts-ignore */
            return this[`_resolve${capitalize(nbt.type)}`](nbt, path, index);
        }
        if (nbt instanceof NBTPrimitive) {
            return resolvedNBT;
        }
        this.dataPoint.set({});
        for (const [key, value] of Object.entries(nbt)) {
            const resolved = this._resolveNBT(value, `${path === undefined ? '' : `${path}.`}${key}`);
            if (resolved !== undefined) {
                resolvedNBT[key] = resolved;
            }
        }
        return resolvedNBT;
    }
    /**
     * @internal
     */
    _resolveData(part, path, index) {
        const { value } = part;
        const dataPoint = this.dataPoint.select(path);
        if (index) {
            let _value = value;
            if (value instanceof StringDataPointClass) {
                _value = this.pack.DataVariable(value);
            }
            dataPoint.insert(_value, index);
            return undefined;
        }
        if (value instanceof StringDataPointClass) {
            dataPoint.set(value);
            return undefined;
        }
        dataPoint.set(value);
        return undefined;
    }
    /**
     * @internal
     */
    _resolveScore(value, path, index) {
        const dataPoint = this.dataPoint.select(path);
        // Yes this is cursed
        const args = [value.value, value.primitive.toString().split(' ')[1].slice(3).toLowerCase(), value.scale];
        if (index) {
            const temp = this.pack.getTempStorage('Score');
            /* @ts-ignore */
            temp.set(...args);
            dataPoint.insert(temp, index);
        }
        /* @ts-ignore */
        dataPoint.set(...args);
        return undefined;
    }
    /**
     * @internal
     */
    _resolveScores(value, path, index) {
        const temp = this.pack.getTempStorage('Score');
        const args = [value.primitive.constructor.name.slice(3).slice(0, -5).toLowerCase(), value.scale];
        const dataPoint = this.dataPoint.select(path);
        if (index) {
            /* @ts-ignore */
            // eslint-disable-next-line new-cap
            dataPoint.insert(new value.primitive([]), index);
            const _dataPoint = dataPoint.select([index]);
            for (const score of value) {
                /* @ts-ignore */
                temp.set(score, ...args);
                _dataPoint.append(temp);
            }
        }
        /* @ts-ignore */
        // eslint-disable-next-line new-cap
        dataPoint.set(new value.primitive([]));
        for (const score of value) {
            /* @ts-ignore */
            temp.set(score, ...args);
            dataPoint.append(temp);
        }
    }
    /**
     * @internal
     */
    _toDataPoint() {
        return this.dataPoint;
    }
}
export class ResolveNBTPartClass extends NBTClass {
    value;
    type;
    primitive;
    scale;
    constructor(value, type, primitive, scale) {
        super();
        this.value = value;
        this.type = type;
        this.primitive = primitive;
        if (type.indexOf('score') !== -1) {
            this.scale = scale;
        }
    }
    [util.inspect.custom] = () => 'Unresolved NBT! Make sure to run ResolveNBT on the nbt before use.';
}
export function ResolveNBTPart(value, option1, option2) {
    if (Array.isArray(value)) {
        return new ResolveNBTPartClass(value, 'scores', (option2 || NBTIntArray), (option1 || 1));
    }
    if (value instanceof Score) {
        return new ResolveNBTPartClass(value, 'score', (option2 || NBTInt), (option1 || 1));
    }
    return new ResolveNBTPartClass(value, 'data', (option1 || NBTAnyValue));
}
