/* eslint-disable no-plusplus */
import { MacroArgument } from '../core/Macro.js';
import { SelectorPickClass } from './abstractClasses.js';
import { rangeParser } from './parsers.js';
let matchers = 0;
function createVariable(pack, ...args) {
    const anonymousScore = pack.Variable();
    if (typeof args[0] === 'number' || args[0] instanceof Score) {
        return anonymousScore.set(args[0]);
    }
    return anonymousScore.set(args[0], args[1]);
}
export class Score extends MacroArgument {
    sandstonePack;
    target;
    display;
    objective;
    commands;
    constructor(sandstonePack, target, display, objective) {
        super(sandstonePack.core);
        this.sandstonePack = sandstonePack;
        this.target = target;
        this.display = display;
        this.objective = objective;
        this.commands = sandstonePack.commands;
        if (display !== undefined) {
            if (sandstonePack.core.currentMCFunction) {
                this.setDisplay(display);
            }
            else {
                sandstonePack.initMCFunction.push(() => this.setDisplay(display));
            }
        }
    }
    toString() {
        return `${this.target} ${this.objective}`;
    }
    toJSON() {
        return {
            type: 'minecraft:score',
            target: {
                type: 'minecraft:fixed',
                name: this.target.toString(),
            },
            score: this.objective.toString(),
            scale: 1,
        };
    }
    /**
     * @internal
     */
    _toChatComponent() {
        return {
            score: { name: this.target, objective: this.objective.name },
        };
    }
    /**
     * @internal
     */
    _toMinecraftCondition = () => this.sandstonePack._.not(new this.sandstonePack.conditions.Score(this.sandstonePack.core, [`${this.target}`, `${this.objective}`, 'matches', '0']));
    unaryOperation(operation, operator, ...args) {
        if (typeof args[0] === 'number') {
            this.commands.scoreboard.players[operation](this, args[0]);
        }
        else if (args[0] instanceof Score) {
            this.commands.scoreboard.players.operation(this, operator, args[0].target, args[0].objective);
        }
        else {
            this.commands.scoreboard.players.operation(this, operator, args[0], args[1] ?? this.objective);
        }
        return this;
    }
    binaryOperation(operator, ...args) {
        if (args[0] instanceof Score) {
            this.commands.scoreboard.players.operation(this, operator, args[0].target, args[0].objective);
            return this;
        }
        let objective = args[1] ?? this.objective;
        if (typeof args[0] === 'number') {
            this.sandstonePack.registerNewConstant(args[0]);
            objective = 'sandstone_const';
        }
        this.commands.scoreboard.players.operation(this, operator, args[0], objective);
        return this;
    }
    setDisplay = (display) => {
        if (display.name) {
            this.commands.scoreboard.players.display.name(this, display.name);
        }
        if (display.numberformat) {
            if (display.numberformat === 'blank') {
                this.commands.scoreboard.players.display.numberformat(this, 'blank');
            }
            else if (display.numberformat[0] === 'styled') {
                this.commands.scoreboard.players.display.numberformat(this, 'styled', display.numberformat[1]);
            }
            else {
                this.commands.scoreboard.players.display.numberformat(this, 'fixed', display.numberformat[1]);
            }
        }
        this.display = display;
        return this;
    };
    /** INLINE OPERATORS */
    /**
     * Reset the entity's score.
     */
    reset = () => {
        this.commands.scoreboard.players.reset(this);
    };
    set(...args) {
        if (typeof args[0] === 'object' && !(args[0] instanceof SelectorPickClass) && !(args[0] instanceof Score)) {
            const [data, scale] = args;
            this.commands.execute.store.result.score(this).run.data.get[data.type](data.currentTarget, data.path, scale);
            return this;
        }
        return this.unaryOperation('set', '=', ...args);
    }
    '=' = this.set;
    add(...args) {
        return this.unaryOperation('add', '+=', ...args);
    }
    '+=' = this.add;
    remove(...args) {
        return this.unaryOperation('remove', '-=', ...args);
    }
    '-=' = this.remove;
    /**
     * Adds one to the current entity's score.
     */
    increase() {
        return this.add(1);
    }
    get '++'() {
        return this.increase();
    }
    /**
     * Subtracts one from the current entity's score.
     */
    decrease() {
        return this.remove(1);
    }
    get '--'() {
        return this.decrease();
    }
    multiply(...args) {
        return this.binaryOperation('*=', ...args);
    }
    '*=' = this.multiply;
    divide(...args) {
        return this.binaryOperation('/=', ...args);
    }
    '/=' = this.divide;
    modulo(...args) {
        return this.binaryOperation('%=', ...args);
    }
    '%=' = this.modulo;
    swap(...args) {
        this.binaryOperation('><', ...args);
    }
    '><' = this.swap;
    plus(...args) {
        return createVariable(this.sandstonePack, this).unaryOperation('add', '+=', ...args);
    }
    '+' = this.plus;
    minus(...args) {
        return createVariable(this.sandstonePack, this).unaryOperation('remove', '-=', ...args);
    }
    '-' = this.minus;
    multipliedBy(...args) {
        return createVariable(this.sandstonePack, this).binaryOperation('*=', ...args);
    }
    '*' = this.multipliedBy;
    dividedBy(...args) {
        return createVariable(this.sandstonePack, this).binaryOperation('/=', ...args);
    }
    '/' = this.dividedBy;
    moduloBy(...args) {
        return createVariable(this.sandstonePack, this).binaryOperation('%=', ...args);
    }
    '%' = this.moduloBy;
    /** COMPARISONS OPERATORS */
    comparison(operator, matchesRange, args) {
        const playerScore = this;
        if (typeof args[0] === 'number') {
            return {
                _toMinecraftCondition: () => new this.sandstonePack.conditions.Score(this.sandstonePack.core, [`${playerScore.target}`, `${playerScore.objective}`, 'matches', matchesRange]),
            };
        }
        const endArgs = args[1] ? args : [args[0]];
        return {
            // eslint-disable-next-line max-len
            _toMinecraftCondition: () => new this.sandstonePack.conditions.Score(this.sandstonePack.core, [`${playerScore.target}`, `${playerScore.objective}`, operator, ...(endArgs.map((arg) => arg.toString()))]),
        };
    }
    greaterThan(...args) {
        return this.comparison('>', `${typeof args[0] === 'number' ? args[0] + 1 : null}..`, args);
    }
    '>' = this.greaterThan;
    greaterOrEqualThan(...args) {
        return this.comparison('>=', `${args[0]}..`, args);
    }
    '>=' = this.greaterOrEqualThan;
    lessThan(...args) {
        return this.comparison('<', `..${typeof args[0] === 'number' ? args[0] - 1 : null}`, args);
    }
    '<' = this.lessThan;
    lessOrEqualThan(...args) {
        return this.comparison('<=', `..${args[0]}`, args);
    }
    '<=' = this.lessOrEqualThan;
    equalTo(...args) {
        return this.comparison('=', args[0].toString(), args);
    }
    '==' = this.equalTo;
    notEqualTo(...args) {
        return this.sandstonePack._.not(this.comparison('=', args[0].toString(), args));
    }
    '!=' = this.notEqualTo;
    /**
     * Check if the current score matches a certain range.
     *
     * @param range The range to compare the current score against.
     */
    matches = (range) => ({
        _toMinecraftCondition: () => new this.sandstonePack.conditions.Score(this.sandstonePack.core, [`${this.target}`, `${this.objective}`, 'matches', rangeParser(this.sandstoneCore, range)]),
    });
    match = (minimum, maximum, callback) => {
        const { MCFunction, Macro } = this.sandstonePack;
        if (maximum > 1000) {
            console.warn(`\nWarning: Score.match() will have to create ${maximum - minimum} mcfunction files, this *will* take a while. `
                + 'Consider only compiling this once and saving it to external resources.');
        }
        const matcher = matchers++;
        const score = this;
        for (let i = minimum; i < maximum; i++) {
            MCFunction(`__sandstone:score_match/${matcher}/${i}`, () => callback(i));
        }
        return MCFunction(`__sandstone:score_match/${matcher}`, [score], () => Macro.returnCmd.run.functionCmd(Macro `__sandstone:score_match/${matcher}/${score}`));
    };
}
