"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
var Selector_exports = {};
__export(Selector_exports, {
  SelectorClass: () => SelectorClass
});
module.exports = __toCommonJS(Selector_exports);
var import_NBTs = require("sandstone/variables/nbt/NBTs");
var import_parsers = require("sandstone/variables/parsers");
function parseScore(core, scores) {
  return `{${Object.entries(scores).map(([scoreName, value]) => [scoreName, (0, import_parsers.rangeParser)(core, value)].join("=")).join(", ")}}`;
}
function parseAdvancements(advancements) {
  return `{${Object.entries(advancements).map(([advancementName, value]) => {
    if (typeof value === "boolean") {
      return [advancementName, value].join("=");
    }
    if (Array.isArray(value)) {
      if (Array.isArray(value[0])) {
        return [advancementName, `{${value.map((_value) => _value.join("=")).join(",")}}`].join("=");
      }
      return [advancementName, `{${value.join("=")}}`].join("=");
    }
    return [advancementName, parseAdvancements(value)].join("=");
  }).join(", ")}}`;
}
class SelectorClass {
  constructor(sandstonePack, target, selectorArguments) {
    this.sandstonePack = sandstonePack;
    this.target = target;
    __publicField(this, "arguments");
    // Custom actions //
    /**
     * List all scores of this entity.
     */
    __publicField(this, "listScores", () => {
      this.sandstonePack.commands.scoreboard.players.list(this.toString());
    });
    /**
     * @internal
     */
    __publicField(this, "_toMinecraftCondition", () => new this.sandstonePack.conditions.Selector(this.sandstonePack.core, this.toString()));
    this.arguments = selectorArguments != null ? selectorArguments : {};
  }
  toString() {
    if (!this.arguments || !Object.keys(this.arguments).length) {
      return this.target;
    }
    const result = [];
    const args = { ...this.arguments };
    const modifiers = {
      // Parse scores
      scores: (scores) => result.push(["scores", parseScore(this.sandstonePack.core, scores)]),
      // Parse potentially multiple nbt
      nbt: (nbt) => {
        const nbts = Array.isArray(nbt) ? nbt : [nbt];
        result.push(...nbts.map((nbt_) => ["nbt", (0, import_NBTs.nbtStringifier)(nbt_)]));
      },
      // Parse advancements
      advancements: (advancements) => result.push(
        ["advancements", parseAdvancements(advancements)]
      ),
      // Parse potentially multiple tags
      tag: (tag) => {
        const tags = Array.isArray(tag) ? tag : [tag];
        result.push(...tags.map((tag_) => ["tag", tag_]));
      },
      // Parse potentially multiple gamemodes
      gamemode: (gamemode) => {
        const gamemodes = Array.isArray(gamemode) ? gamemode : [gamemode];
        result.push(...gamemodes.map((gamemode_) => ["gamemode", gamemode_]));
      },
      // Parse potentially multiple predicates
      predicate: (predicate) => {
        const predicates = Array.isArray(predicate) ? predicate : [predicate];
        result.push(...predicates.map((pred) => ["predicate", pred]));
      },
      // Handle boolean values for teams
      team: (team) => {
        let teamRepr;
        if (team === true) {
          teamRepr = "!";
        } else if (team === false) {
          teamRepr = "";
        } else {
          teamRepr = team;
        }
        result.push(["team", teamRepr]);
      },
      distance: (range_) => result.push(["distance", (0, import_parsers.rangeParser)(this.sandstonePack.core, range_)])
    };
    for (const [baseName, modifier] of Object.entries(modifiers)) {
      const name = baseName;
      const value = args[name];
      if (value !== void 0) {
        modifier(value);
        delete args[name];
      }
    }
    Object.entries(args).forEach(([key, value]) => {
      if (value !== void 0) {
        result.push([key, `${value}`]);
      }
    });
    return `${this.target}[${result.map(([key, value]) => `${key}=${value}`).join(", ")}]`;
  }
  /**
   * @internal
   */
  _toChatComponent() {
    return {
      selector: this.toString()
    };
  }
  /**
   * @internal
   */
  _toSelector() {
    return this;
  }
  toJSON() {
    return this.toString();
  }
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  SelectorClass
});
//# sourceMappingURL=Selector.cjs.map