/* eslint-disable camelcase */
/* eslint-disable @typescript-eslint/ban-types */
import { nbtStringifier } from 'sandstone/variables/nbt/NBTs';
import { rangeParser } from 'sandstone/variables/parsers';
// Returns the string representation of a score argument. `{ myScore: [0, null] } => {myScore=0..}`, `{ myScore: [-Infinity, 5] } => {myScore='..5'}`, 8 => '8'
function parseScore(core, scores) {
    return `{${Object.entries(scores).map(([scoreName, value]) => [scoreName, rangeParser(core, value)].join('=')).join(', ')}}`;
}
// Returns the string representation of advancements
function parseAdvancements(advancements) {
    return `{${Object.entries(advancements).map(([advancementName, value]) => {
        if (typeof value === 'boolean') {
            return [advancementName, value].join('=');
        }
        if (Array.isArray(value)) {
            if (Array.isArray(value[0])) {
                return [advancementName, `{${value.map((_value) => _value.join('=')).join(',')}}`].join('=');
            }
            return [advancementName, `{${value.join('=')}}`].join('=');
        }
        return [advancementName, parseAdvancements(value)].join('=');
    }).join(', ')}}`;
}
export class SelectorClass {
    sandstonePack;
    target;
    arguments;
    constructor(sandstonePack, target, selectorArguments) {
        this.sandstonePack = sandstonePack;
        this.target = target;
        this.arguments = selectorArguments ?? {};
    }
    // Custom actions //
    /**
     * List all scores of this entity.
     */
    listScores = () => {
        this.sandstonePack.commands.scoreboard.players.list(this.toString());
    };
    toString() {
        if (!this.arguments || !Object.keys(this.arguments).length) {
            return this.target;
        }
        const result = [];
        const args = { ...this.arguments };
        const modifiers = {
            // Parse scores
            scores: (scores) => result.push(['scores', parseScore(this.sandstonePack.core, scores)]),
            // Parse potentially multiple nbt
            nbt: (nbt) => {
                const nbts = Array.isArray(nbt) ? nbt : [nbt];
                result.push(...nbts.map((nbt_) => ['nbt', nbtStringifier(nbt_)]));
            },
            // Parse advancements
            advancements: (advancements) => result.push(['advancements', parseAdvancements(advancements)]),
            // Parse potentially multiple tags
            tag: (tag) => {
                const tags = Array.isArray(tag) ? tag : [tag];
                result.push(...tags.map((tag_) => ['tag', tag_]));
            },
            // Parse potentially multiple gamemodes
            gamemode: (gamemode) => {
                const gamemodes = Array.isArray(gamemode) ? gamemode : [gamemode];
                result.push(...gamemodes.map((gamemode_) => ['gamemode', gamemode_]));
            },
            // Parse potentially multiple predicates
            predicate: (predicate) => {
                const predicates = Array.isArray(predicate) ? predicate : [predicate];
                result.push(...predicates.map((pred) => ['predicate', pred]));
            },
            // Handle boolean values for teams
            team: (team) => {
                let teamRepr;
                if (team === true) {
                    teamRepr = '!';
                }
                else if (team === false) {
                    teamRepr = '';
                }
                else {
                    teamRepr = team;
                }
                result.push(['team', teamRepr]);
            },
            distance: (range_) => result.push(['distance', rangeParser(this.sandstonePack.core, range_)]),
        };
        for (const [baseName, modifier] of Object.entries(modifiers)) {
            const name = baseName;
            const value = args[name];
            if (value !== undefined) {
                modifier(value);
                delete args[name];
            }
        }
        Object.entries(args).forEach(([key, value]) => {
            if (value !== undefined) {
                result.push([key, `${value}`]);
            }
        });
        return `${this.target}[${result.map(([key, value]) => `${key}=${value}`).join(', ')}]`;
    }
    /**
     * @internal
     */
    _toMinecraftCondition = () => new this.sandstonePack.conditions.Selector(this.sandstonePack.core, this.toString());
    /**
     * @internal
     */
    _toChatComponent() {
        return {
            selector: this.toString(),
        };
    }
    /**
     * @internal
     */
    _toSelector() {
        return this;
    }
    toJSON() {
        return this.toString();
    }
}
