import { SelectorPickClass } from './abstractClasses.js';
import type { JSONTextComponent } from 'sandstone/arguments';
import type { SandstoneCore } from 'sandstone/core';
import type { ConditionTextComponentClass } from './abstractClasses.js';
import type { DataPointClass } from './Data.js';
import type { Score } from './Score.js';
import type { SelectorClass } from './Selector.js';
export type UUIDinNumber = [number, number, number, number];
export type UUIDinScore = [Score, Score, Score, Score];
export type UUIDSource = string | UUIDinNumber | UUIDinScore | SelectorPickClass<true, boolean> | DataPointClass;
export type UUIDOptions = {
    /** Can set source types for the UUID that you did not set in `source`. */
    sources?: {
        /**
         * Four `Score`'s representing the UUID.
         *
         * Options:
         * - `true` will automatically generate and set scores. If UUID is constructed within the context of a mcfunction commands will be called in-place, otherwise the commands will be added to __init__.
         * - `[UUIDinScore]` will automatically set the scores with the above rules.
         * - `UUIDinScore` will assume the UUID is already set to these scores.
         */
        scores?: true | [UUIDinScore] | UUIDinScore;
        /**
         * `DataPoint` representing the UUID.
         *
         * Options:
         * - `true` will automatically generate and set a data point. If UUID is constructed within the context of a mcfunction the command will be called in-place, otherwise the command will be added to __init__.
         * - `[DataPointClass]` will automatically set the data point with the above rules.
         * - `DataPointClass` will assume the UUID is already set to these scores.
         */
        data?: true | [DataPointClass] | DataPointClass;
        /** Selector for the entity with the UUID. If `known` is set this will go unused. */
        selector?: SelectorClass<true, boolean>;
    };
};
export declare class UUIDClass<PrimarySource extends 'known' | 'scores' | 'selector' | 'data'> implements ConditionTextComponentClass, SelectorPickClass<true, false> {
    protected core: SandstoneCore;
    readonly primarySource: PrimarySource;
    known: PrimarySource extends 'known' ? UUIDinNumber : (UUIDinNumber | undefined);
    scores: PrimarySource extends 'scores' ? UUIDinScore : (UUIDinScore | undefined);
    data: PrimarySource extends 'data' ? DataPointClass : (DataPointClass | undefined);
    selector: PrimarySource extends 'selector' ? SelectorClass<true, boolean> : (SelectorClass<true, boolean> | undefined);
    /**
     *
     * @param _source A primary source for the UUID. Either a UUID string, a UUID integer array, 4 scores set to the UUID, a selector, or a Data Point set to the UUID.
     */
    constructor(core: SandstoneCore, _source: UUIDSource, options?: UUIDOptions);
    /**
     * Converts a string UUID to an integer array
     * @param uuid A UUID in string form
     */
    stringToArray(uuid: string): UUIDinNumber;
    /**
     * Converts an integer array to a string UUID
     * @param array Optional. An integer array. Defaults to contained UUID (known; this does not work at pack runtime)
     */
    arrayToString(array?: UUIDinNumber): string;
    setKnown(uuid: UUIDinNumber): this;
    setScores(scores?: UUIDinScore | [UUIDinScore]): this;
    setData(data?: DataPointClass | DataPointClass[]): this;
    /** Selector for the entity with the UUID. If `known` is set this will go unused. */
    setSelector(selector: SelectorClass<true, any>): this;
    /** Partial execute command executing as (not at) the entity */
    get execute(): import("../commands/index.js").ExecuteCommand<false> | ((targets: import("sandstone/arguments").MultipleEntitiesArgument<false>) => import("../commands/index.js").ExecuteCommand<false>);
    toString(): string;
    /**
     * @internal
     */
    _toChatComponent(): JSONTextComponent;
    /**
     * @internal
     */
    _toMinecraftCondition: () => ConditionClass;
    /**
     * @internal
     */
    _toSelector(): string | NonNullable<PrimarySource extends "selector" ? SelectorClass<true, boolean, false> : SelectorClass<true, boolean, false> | undefined>;
}
