"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
var NBTs_exports = {};
__export(NBTs_exports, {
  NBT: () => NBT,
  NBTAnyValue: () => NBTAnyValue,
  NBTByte: () => NBTByte,
  NBTByteArray: () => NBTByteArray,
  NBTClass: () => NBTClass,
  NBTDouble: () => NBTDouble,
  NBTFloat: () => NBTFloat,
  NBTInt: () => NBTInt,
  NBTIntArray: () => NBTIntArray,
  NBTLong: () => NBTLong,
  NBTLongArray: () => NBTLongArray,
  NBTPrimitive: () => NBTPrimitive,
  NBTShort: () => NBTShort,
  NBTString: () => NBTString,
  NBTTypedArray: () => NBTTypedArray,
  NotNBT: () => NotNBT,
  nbtStringifier: () => nbtStringifier
});
module.exports = __toCommonJS(NBTs_exports);
var import_utils = require("sandstone/utils");
var util = __toESM(require("util"), 1);
var import_parser = require("./parser.js");
var _a, _b, _c;
class NBTClass {
  constructor() {
    __publicField(this, "toString", this[util.inspect.custom]);
  }
}
class NBTPrimitive extends NBTClass {
  constructor(value, unit) {
    super();
    __publicField(this, "value");
    __publicField(this, "unit");
    __publicField(this, _a, () => `${this.value}${this.unit}`);
    this.value = value;
    this.unit = unit;
  }
}
_a = util.inspect.custom;
class NBTLong extends NBTPrimitive {
  constructor(value) {
    super(value, "l");
  }
}
class NBTByte extends NBTPrimitive {
  constructor(value) {
    super(value, "b");
  }
}
class NBTShort extends NBTPrimitive {
  constructor(value) {
    super(value, "s");
  }
}
class NBTFloat extends NBTPrimitive {
  constructor(value) {
    super(value, "f");
  }
}
class NBTDouble extends NBTPrimitive {
  constructor(value) {
    super(value, "d");
  }
}
class NBTTypedArray extends NBTClass {
  constructor(values, unit) {
    super();
    __publicField(this, "values");
    __publicField(this, "unit");
    __publicField(this, _b, () => `[${this.unit};${this.values.length === 0 ? "" : this.values.join(",")}]`);
    this.values = values;
    this.unit = unit;
  }
}
_b = util.inspect.custom;
class NBTLongArray extends NBTTypedArray {
  constructor(values) {
    super(values, "L");
  }
}
class NBTByteArray extends NBTTypedArray {
  constructor(values) {
    super(values, "B");
  }
}
class NBTIntArray extends NBTTypedArray {
  constructor(values) {
    super(values, "I");
  }
}
class NotNBT extends NBTClass {
  constructor(nbt) {
    super();
    __publicField(this, "nbt");
    __publicField(this, _c, () => `!${nbtStringifier(this.nbt)}`);
    this.nbt = nbt;
  }
}
_c = util.inspect.custom;
class NBTInt extends NBTPrimitive {
}
class NBTString extends NBTPrimitive {
}
class NBTAnyValue extends NBTPrimitive {
}
function customNumber(num, _class) {
  if (Array.isArray(num)) {
    return num.map((n) => new _class(n));
  }
  return new _class(num);
}
function dynamicNBT(template, ...args) {
  const mixedArgs = template.flatMap((s, i) => [s, args[i]]).slice(0, -1);
  const result = mixedArgs.map((element) => element instanceof NBTClass ? nbtStringifier(element) : element.toString()).join("");
  return (0, import_parser.parseNBT)(NBT, result);
}
const NBT = (0, import_utils.makeCallable)({
  float: (num) => customNumber(num, NBTFloat),
  double: (num) => customNumber(num, NBTDouble),
  byte: (num) => customNumber(num, NBTByte),
  short: (num) => customNumber(num, NBTShort),
  long: (num) => customNumber(num, NBTLong),
  intArray: (numbers) => new NBTIntArray(numbers),
  longArray: (numbers) => new NBTLongArray(numbers),
  byteArray: (numbers) => new NBTByteArray(numbers),
  not: (nbt) => new NotNBT(nbt),
  stringify: (nbt) => nbtStringifier(nbt),
  parse: (nbt) => (0, import_parser.parseNBT)(NBT, nbt)
}, dynamicNBT);
const nbtStringifier = (nbt) => {
  if (typeof nbt === "number") {
    return nbt.toString();
  }
  if (typeof nbt === "boolean") {
    return nbt.toString();
  }
  if (typeof nbt === "string") {
    const inspectedStr = util.inspect(nbt, {
      breakLength: Infinity,
      compact: true,
      maxStringLength: Infinity,
      depth: Infinity
    });
    if (inspectedStr[0] === "`") {
      return JSON.stringify(nbt);
    }
    return inspectedStr;
  }
  if (Array.isArray(nbt)) {
    const itemsStr = nbt.map(nbtStringifier).join(",");
    return `[${itemsStr}]`;
  }
  if (nbt instanceof NBTClass) {
    return nbt[util.inspect.custom]();
  }
  if (typeof nbt === "object" && nbt.toMacro) {
    return nbt.toMacro();
  }
  const objectStr = Object.entries(nbt).map(([key, value]) => `${key}:${nbtStringifier(value)}`).join(",");
  return `{${objectStr}}`;
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  NBT,
  NBTAnyValue,
  NBTByte,
  NBTByteArray,
  NBTClass,
  NBTDouble,
  NBTFloat,
  NBTInt,
  NBTIntArray,
  NBTLong,
  NBTLongArray,
  NBTPrimitive,
  NBTShort,
  NBTString,
  NBTTypedArray,
  NotNBT,
  nbtStringifier
});
//# sourceMappingURL=NBTs.cjs.map