export declare const isCompoundSymbol: unique symbol;
type DataType<TYPE extends string, VALUE, PROPERTIES extends Record<string, unknown> | unknown = unknown> = {
    dataType: TYPE;
    value: VALUE;
} & PROPERTIES;
export type NBTExpression = NBTCompound | NBTArray | NBTLongArray | NBTByteArray | NBTIntArray | NBTString | NBTLong | NBTDouble | NBTFloat | NBTByte | NBTNumber | NBTShort | NBTBoolean;
export type NBTCompound = {
    [k: string]: NBTExpression;
};
export type NBTArray = NBTExpression[];
export type NBTByteArray = DataType<'byteArray', NBTByte[]>;
export type NBTLongArray = DataType<'longArray', NBTLong[]>;
export type NBTIntArray = DataType<'intArray', NBTShort[]>;
export type NBTString = DataType<'string', string>;
export type NBTLong = DataType<'long', number>;
export type NBTShort = DataType<'short', number>;
export type NBTByte = DataType<'byte', number>;
export type NBTNumber = DataType<'number', number>;
export type NBTDouble = DataType<'double', number>;
export type NBTFloat = DataType<'float', number>;
export type NBTBoolean = DataType<'boolean', boolean>;
interface NearleyToken {
    value: any;
    [key: string]: any;
}
interface NearleyLexer {
    reset: (chunk: string, info: any) => void;
    next: () => NearleyToken | undefined;
    save: () => any;
    formatError: (token: never) => string;
    has: (tokenType: string) => boolean;
}
interface NearleyRule {
    name: string;
    symbols: NearleySymbol[];
    postprocess?: (d: any[], loc?: number, reject?: {}) => any;
}
type NearleySymbol = string | {
    literal: any;
} | {
    test: (token: any) => boolean;
};
interface Grammar {
    Lexer: NearleyLexer | undefined;
    ParserRules: NearleyRule[];
    ParserStart: string;
}
declare const grammar: Grammar;
export default grammar;
