"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var parsers_exports = {};
__export(parsers_exports, {
  arrayToArgsParser: () => arrayToArgsParser,
  coordinatesParser: () => coordinatesParser,
  rangeParser: () => rangeParser,
  rotationParser: () => rotationParser,
  sanitizeValue: () => sanitizeValue,
  structureMirrorParser: () => structureMirrorParser,
  structureRotationParser: () => structureRotationParser,
  targetParser: () => targetParser
});
module.exports = __toCommonJS(parsers_exports);
var import_Macro = require("../core/Macro.js");
var import_Coordinates = require("./Coordinates.js");
function arrayToArgsParser(args) {
  if (Array.isArray(args) && args.length === 3) {
    return new import_Coordinates.VectorClass(args.map((arg) => arg.toString()));
  }
  return args;
}
function isRawCoordinates(arg) {
  return Array.isArray(arg) && (arg.length === 3 || arg.length === 2) && arg.every((c) => typeof c === "string");
}
function isRawRotation(arg) {
  return Array.isArray(arg) && arg.length === 2 && arg.every((c) => typeof c === "string");
}
function coordinatesParser(coordinates) {
  return isRawCoordinates(coordinates) ? new import_Coordinates.VectorClass(coordinates) : coordinates;
}
function rotationParser(rotation) {
  return isRawRotation(rotation) ? new import_Coordinates.VectorClass(rotation) : rotation;
}
const sanitizeValue = (core, value) => {
  if (value === void 0 || value === null) {
    return "";
  }
  if (typeof value === "string") {
    return value;
  }
  if ((0, import_Macro.isMacroArgument)(core, value)) {
    return value.toMacro();
  }
  if (Number.isFinite(value)) {
    return value.toString();
  }
  return "";
};
const rangeParser = (core, range) => {
  if (Array.isArray(range)) {
    return `${sanitizeValue(core, range[0])}..${sanitizeValue(core, range[1])}`;
  }
  return range.toString();
};
const structureRotationParser = (rotation) => {
  if (!rotation) {
    return "none";
  }
  if (typeof rotation === "object") {
    return rotation;
  }
  const numToLiteral = (angle) => {
    switch (angle) {
      case 0:
        return "none";
      case 90:
        return "clockwise_90";
      case 180:
        return "180";
      case 270:
        return "counterclockwise_90";
      case -90:
        return "counterclockwise_90";
      case -180:
        return "180";
      case -270:
        return "clockwise_90";
      default: {
        if (!Number.isInteger(angle / 90)) {
          throw new Error("Structure rotation must be in increments of 90!");
        }
        angle %= 360;
        angle = (angle + 360) % 360;
        if (angle > 180)
          angle -= 360;
        return numToLiteral(angle);
      }
    }
  };
  if (typeof rotation === "number") {
    return numToLiteral(rotation);
  }
  if (rotation === "clockwise_90" || rotation === "counterclockwise_90") {
    return rotation;
  }
  return numToLiteral(Number(rotation));
};
const structureMirrorParser = (mirror) => {
  if (typeof mirror === "object") {
    return mirror.toMacro();
  }
  if (typeof mirror === "string") {
    const lastCharacter = mirror.slice(0, -1);
    if (lastCharacter === "x") {
      return "left_right";
    }
    if (lastCharacter === "z") {
      return "front_back";
    }
    return mirror;
  }
  if (mirror) {
    return "left_right";
  }
  return "none";
};
function targetParser(target) {
  if (typeof target === "string") {
    return target;
  }
  if (target._toSelector) {
    return target._toSelector();
  }
  if (target.toMacro) {
    return target;
  }
  return target.toString();
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  arrayToArgsParser,
  coordinatesParser,
  rangeParser,
  rotationParser,
  sanitizeValue,
  structureMirrorParser,
  structureRotationParser,
  targetParser
});
//# sourceMappingURL=parsers.cjs.map